@@ -71,13 +71,21 @@ $ADB push "$INSTALL/lib/libEGL.so" /vendor/lib64/egl/libEGL_mesa.so
 $ADB push "$INSTALL/lib/libGLESv1_CM.so" /vendor/lib64/egl/libGLESv1_CM_mesa.so
 $ADB push "$INSTALL/lib/libGLESv2.so" /vendor/lib64/egl/libGLESv2_mesa.so
 
-$ADB shell rm -f /vendor/lib64/hw/vulkan.lvp.so*
-$ADB shell rm -f /vendor/lib64/hw/vulkan.virtio.so*
-$ADB shell rm -f /vendor/lib64/hw/vulkan.intel.so*
-
-$ADB push "$INSTALL/lib/libvulkan_lvp.so" /vendor/lib64/hw/vulkan.lvp.so
-$ADB push "$INSTALL/lib/libvulkan_virtio.so" /vendor/lib64/hw/vulkan.virtio.so
-$ADB push "$INSTALL/lib/libvulkan_intel.so" /vendor/lib64/hw/vulkan.intel.so
+# Remove and replace Vulkan drivers:
+# - For Cuttlefish virtual machines, replace lavapipe or venus
+# - For Android hardware, replace the Vulkan driver specified by VK_DRIVER
+if [ -n "${CUTTLEFISH_GPU_MODE:-}" ]; then
+  if [ "$CUTTLEFISH_GPU_MODE" = "mesa_swrast" ] || [ "$CUTTLEFISH_GPU_MODE" = "mesa_swrast_guest_angle" ]; then
+    $ADB shell rm -f /vendor/lib64/hw/vulkan.lvp.so*
+    $ADB push "$INSTALL/lib/libvulkan_lvp.so" /vendor/lib64/hw/vulkan.lvp.so
+  else
+    $ADB shell rm -f /vendor/lib64/hw/vulkan.virtio.so*
+    $ADB push "$INSTALL/lib/libvulkan_virtio.so" /vendor/lib64/hw/vulkan.virtio.so
+  fi
+else
+  $ADB shell rm -f /vendor/lib64/hw/vulkan.${VK_DRIVER}.so*
+  $ADB push "$INSTALL/lib/libvulkan_${VK_DRIVER}.so" /vendor/lib64/hw/vulkan.${VK_DRIVER}.so
+fi
 
 $ADB shell rm -f /vendor/lib64/egl/libEGL_emulation.so*
 $ADB shell rm -f /vendor/lib64/egl/libGLESv1_CM_emulation.so*
@@ -139,12 +147,10 @@ if [ "$OLD_SF_PID" == "$NEW_SF_PID" ]; then
      exit 1
 fi
 
+# These should be the last commands of the script in order to correctly
+# propagate the exit code.
 if [ -n "${ANDROID_CTS_TAG:-}" ]; then
-  # The script sets EXIT_CODE
   . "$(dirname "$0")/android-cts-runner.sh"
 else
-  # The script sets EXIT_CODE
   . "$(dirname "$0")/android-deqp-runner.sh"
 fi
-
-exit $EXIT_CODE
