@@ -51,7 +51,6 @@ DEPS=(
     "pkgconfig(libglvnd)"
     "pkgconfig(libva)"
     "pkgconfig(pciaccess)"
-    "pkgconfig(vdpau)"
     "pkgconfig(vulkan)"
     "pkgconfig(x11)"
     "pkgconfig(x11-xcb)"
@@ -79,7 +78,6 @@ DEPS=(
     python3-ply
     python3-pycparser
     python3-yaml
-    rust-packaging
     spirv-tools-devel
     spirv-llvm-translator-devel
     vulkan-headers
@@ -105,6 +103,8 @@ rm -rf $XORGMACROS_VERSION
 
 . .gitlab-ci/container/install-meson.sh
 
+. .gitlab-ci/container/build-rust.sh build
+
 . .gitlab-ci/container/build-bindgen.sh
 
 . .gitlab-ci/container/build-mold.sh
