@@ -281,7 +281,7 @@ ms_get_out_layout_part(unsigned location,
       }
    }
 
-   unreachable("Couldn't figure out mesh shader output mode.");
+   UNREACHABLE("Couldn't figure out mesh shader output mode.");
 }
 
 static void
@@ -339,15 +339,7 @@ ms_store_arrayed_output(nir_builder *b,
                            .access = ACCESS_COHERENT | ACCESS_IS_SWIZZLED_AMD,
                            .align_mul = 16, .align_offset = const_off % 16u);
    } else if (out_mode == ms_out_mode_var) {
-      unsigned write_mask_32 = write_mask;
-      if (store_val->bit_size > 32) {
-         /* Split 64-bit store values to 32-bit components. */
-         store_val = nir_bitcast_vector(b, store_val, 32);
-         /* Widen the write mask so it is in 32-bit components. */
-         write_mask_32 = util_widen_mask(write_mask, store_val->bit_size / 32);
-      }
-
-      u_foreach_bit(comp, write_mask_32) {
+      u_foreach_bit(comp, write_mask) {
          unsigned idx = io_sem.location * 4 + comp + component_offset;
          nir_def *val = nir_channel(b, store_val, comp);
          nir_def *v = nir_load_var(b, s->out_variables[idx]);
@@ -363,7 +355,7 @@ ms_store_arrayed_output(nir_builder *b,
          nir_store_var(b, s->out_variables[idx], val, 0x1);
       }
    } else {
-      unreachable("Invalid MS output mode for store");
+      UNREACHABLE("Invalid MS output mode for store");
    }
 }
 
@@ -447,7 +439,7 @@ ms_load_arrayed_output(nir_builder *b,
       }
       return nir_vec(b, arr, num_components);
    } else {
-      unreachable("Invalid MS output mode for load");
+      UNREACHABLE("Invalid MS output mode for load");
    }
 }
 
@@ -521,7 +513,7 @@ lower_ms_intrinsic(nir_builder *b, nir_instr *instr, void *state)
    case nir_intrinsic_set_vertex_and_primitive_count:
       return lower_ms_set_vertex_and_primitive_count(b, intrin, s);
    default:
-      unreachable("Not a lowerable mesh shader intrinsic.");
+      UNREACHABLE("Not a lowerable mesh shader intrinsic.");
    }
 }
 
@@ -1319,7 +1311,7 @@ ms_calculate_output_layout(const struct radeon_info *hw_info, unsigned api_share
       else if (l.lds.vtx_attr.mask)
          ms_move_output(&l.lds.vtx_attr, &l.scratch_ring.vtx_attr);
       else
-         unreachable("API shader uses too much shared memory.");
+         UNREACHABLE("API shader uses too much shared memory.");
 
       ms_calculate_arrayed_output_layout(&l, max_vertices, max_primitives);
    }
