@@ -18,7 +18,6 @@ extern "C" {
 
 typedef struct nir_shader nir_shader;
 struct radeon_info;
-struct radv_pipeline_layout;
 struct radv_shader_stage;
 struct radv_shader_info;
 struct radv_shader_args;
@@ -96,7 +95,11 @@ typedef struct radv_nir_opt_tid_function_options {
 
 bool radv_nir_opt_tid_function(nir_shader *shader, const radv_nir_opt_tid_function_options *options);
 
-bool radv_nir_opt_fs_builtins(nir_shader *shader, const struct radv_graphics_state_key *gfx_state);
+bool radv_nir_opt_fs_builtins(nir_shader *shader, const struct radv_graphics_state_key *gfx_state,
+                              unsigned vgt_outprim_type);
+
+bool radv_nir_lower_immediate_samplers(nir_shader *shader, struct radv_device *device,
+                                       const struct radv_shader_stage *stage);
 
 #ifdef __cplusplus
 }
