@@ -19,6 +19,8 @@
 typedef struct {
    enum amd_gfx_level gfx_level;
    uint32_t address32_hi;
+   uint32_t combined_image_sampler_desc_size;
+   uint32_t combined_image_sampler_offset;
    bool disable_aniso_single_level;
    bool has_image_load_dcc_bug;
    bool disable_tg4_trunc_coord;
@@ -45,14 +47,14 @@ static nir_def *
 load_desc_ptr(nir_builder *b, apply_layout_state *state, unsigned set)
 {
    const struct radv_userdata_locations *user_sgprs_locs = &state->info->user_sgprs_locs;
-   if (user_sgprs_locs->shader_data[AC_UD_INDIRECT_DESCRIPTOR_SETS].sgpr_idx != -1) {
-      nir_def *addr = get_scalar_arg(b, 1, state->args->descriptor_sets[0]);
+   if (user_sgprs_locs->shader_data[AC_UD_INDIRECT_DESCRIPTORS].sgpr_idx != -1) {
+      nir_def *addr = get_scalar_arg(b, 1, state->args->descriptors[0]);
       addr = convert_pointer_to_64_bit(b, state, addr);
-      return nir_load_smem_amd(b, 1, addr, nir_imm_int(b, set * 4));
+      return ac_nir_load_smem(b, 1, addr, nir_imm_int(b, set * 4), 4, 0);
    }
 
-   assert(state->args->descriptor_sets[set].used);
-   return get_scalar_arg(b, 1, state->args->descriptor_sets[set]);
+   assert(state->args->descriptors[set].used);
+   return get_scalar_arg(b, 1, state->args->descriptors[set]);
 }
 
 static void
@@ -67,8 +69,8 @@ visit_vulkan_resource_index(nir_builder *b, apply_layout_state *state, nir_intri
    nir_def *set_ptr;
    if (vk_descriptor_type_is_dynamic(layout->binding[binding].type)) {
       unsigned idx = state->layout->set[desc_set].dynamic_offset_start + layout->binding[binding].dynamic_offset_offset;
-      set_ptr = get_scalar_arg(b, 1, state->args->ac.push_constants);
-      offset = state->layout->push_constant_size + idx * 16;
+      set_ptr = get_scalar_arg(b, 1, state->args->ac.dynamic_descriptors);
+      offset = idx * 16;
       stride = 16;
    } else {
       set_ptr = load_desc_ptr(b, state, desc_set);
@@ -76,10 +78,10 @@ visit_vulkan_resource_index(nir_builder *b, apply_layout_state *state, nir_intri
    }
 
    nir_def *binding_ptr = nir_imul_imm(b, intrin->src[0].ssa, stride);
-   nir_instr_as_alu(binding_ptr->parent_instr)->no_unsigned_wrap = true;
+   nir_def_as_alu(binding_ptr)->no_unsigned_wrap = true;
 
    binding_ptr = nir_iadd_imm(b, binding_ptr, offset);
-   nir_instr_as_alu(binding_ptr->parent_instr)->no_unsigned_wrap = true;
+   nir_def_as_alu(binding_ptr)->no_unsigned_wrap = true;
 
    if (layout->binding[binding].type == VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR) {
       assert(stride == 16);
@@ -99,7 +101,7 @@ visit_vulkan_resource_reindex(nir_builder *b, apply_layout_state *state, nir_int
       nir_def *binding_ptr = nir_unpack_64_2x32_split_y(b, intrin->src[0].ssa);
 
       nir_def *index = nir_imul_imm(b, intrin->src[1].ssa, 16);
-      nir_instr_as_alu(index->parent_instr)->no_unsigned_wrap = true;
+      nir_def_as_alu(index)->no_unsigned_wrap = true;
 
       binding_ptr = nir_iadd_nuw(b, binding_ptr, index);
 
@@ -111,7 +113,7 @@ visit_vulkan_resource_reindex(nir_builder *b, apply_layout_state *state, nir_int
       nir_def *stride = nir_channel(b, intrin->src[0].ssa, 2);
 
       nir_def *index = nir_imul(b, intrin->src[1].ssa, stride);
-      nir_instr_as_alu(index->parent_instr)->no_unsigned_wrap = true;
+      nir_def_as_alu(index)->no_unsigned_wrap = true;
 
       binding_ptr = nir_iadd_nuw(b, binding_ptr, index);
 
@@ -166,7 +168,7 @@ load_buffer_descriptor(nir_builder *b, apply_layout_state *state, nir_def *rsrc,
       return nir_iadd(b, nir_channel(b, rsrc, 0), nir_channel(b, rsrc, 1));
 
    nir_def *desc_set = convert_pointer_to_64_bit(b, state, nir_channel(b, rsrc, 0));
-   return nir_load_smem_amd(b, 4, desc_set, nir_channel(b, rsrc, 1), .align_mul = 16);
+   return ac_nir_load_smem(b, 4, desc_set, nir_channel(b, rsrc, 1), 4, 0);
 }
 
 static void
@@ -184,7 +186,7 @@ visit_get_ssbo_size(nir_builder *b, apply_layout_state *state, nir_intrinsic_ins
    } else {
       /* load the entire descriptor so it can be CSE'd */
       nir_def *ptr = convert_pointer_to_64_bit(b, state, nir_channel(b, rsrc, 0));
-      nir_def *desc = nir_load_smem_amd(b, 4, ptr, nir_channel(b, rsrc, 1), .align_mul = 16);
+      nir_def *desc = ac_nir_load_smem(b, 4, ptr, nir_channel(b, rsrc, 1), 4, 0);
       size = nir_channel(b, desc, 2);
    }
 
@@ -204,27 +206,6 @@ get_sampler_desc(nir_builder *b, apply_layout_state *state, nir_deref_instr *der
    struct radv_descriptor_set_layout *layout = state->layout->set[desc_set].layout;
    struct radv_descriptor_set_binding_layout *binding = &layout->binding[binding_index];
 
-   /* Handle immutable and embedded (compile-time) samplers
-    * (VkDescriptorSetLayoutBinding::pImmutableSamplers) We can only do this for constant array
-    * index. Note that indexing is forbidden with embedded samplers.
-    */
-   if (desc_type == AC_DESC_SAMPLER && binding->immutable_samplers_offset && !indirect) {
-      unsigned constant_index = 0;
-
-      while (deref->deref_type != nir_deref_type_var) {
-         assert(deref->deref_type == nir_deref_type_array);
-         unsigned array_size = MAX2(glsl_get_aoa_size(deref->type), 1);
-         constant_index += nir_src_as_uint(deref->arr.index) * array_size;
-         deref = nir_deref_instr_parent(deref);
-      }
-
-      uint32_t dword0_mask =
-         tex->op == nir_texop_tg4 && state->disable_tg4_trunc_coord ? C_008F30_TRUNC_COORD : 0xffffffffu;
-      const uint32_t *samplers = radv_immutable_samplers(layout, binding);
-      return nir_imm_ivec4(b, samplers[constant_index * 4 + 0] & dword0_mask, samplers[constant_index * 4 + 1],
-                           samplers[constant_index * 4 + 2], samplers[constant_index * 4 + 3]);
-   }
-
    unsigned size = 8;
    unsigned offset = binding->offset;
    switch (desc_type) {
@@ -235,18 +216,21 @@ get_sampler_desc(nir_builder *b, apply_layout_state *state, nir_deref_instr *der
       offset += 32;
       break;
    case AC_DESC_PLANE_1:
-      offset += RADV_COMBINED_IMAGE_SAMPLER_DESC_SIZE;
+      offset += state->combined_image_sampler_desc_size;
       break;
    case AC_DESC_SAMPLER:
+      /* Immutable/embedded samplers are lowered earlier. */
+      assert(!binding->immutable_samplers_offset || indirect);
+
       size = RADV_SAMPLER_DESC_SIZE / 4;
       if (binding->type == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
-         offset += RADV_COMBINED_IMAGE_SAMPLER_DESC_SAMPLER_OFFSET;
+         offset += state->combined_image_sampler_offset;
       break;
    case AC_DESC_BUFFER:
       size = RADV_BUFFER_DESC_SIZE / 4;
       break;
    case AC_DESC_PLANE_2:
-      offset += 2 * RADV_COMBINED_IMAGE_SAMPLER_DESC_SIZE;
+      offset += 2 * state->combined_image_sampler_desc_size;
       break;
    }
 
@@ -258,11 +242,11 @@ get_sampler_desc(nir_builder *b, apply_layout_state *state, nir_deref_instr *der
 
       nir_def *tmp = nir_imul_imm(b, deref->arr.index.ssa, array_size);
       if (tmp != deref->arr.index.ssa)
-         nir_instr_as_alu(tmp->parent_instr)->no_unsigned_wrap = true;
+         nir_def_as_alu(tmp)->no_unsigned_wrap = true;
 
       if (index) {
          index = nir_iadd(b, tmp, index);
-         nir_instr_as_alu(index->parent_instr)->no_unsigned_wrap = true;
+         nir_def_as_alu(index)->no_unsigned_wrap = true;
       } else {
          index = tmp;
       }
@@ -272,13 +256,13 @@ get_sampler_desc(nir_builder *b, apply_layout_state *state, nir_deref_instr *der
 
    nir_def *index_offset = index ? nir_iadd_imm(b, index, offset) : nir_imm_int(b, offset);
    if (index && index_offset != index)
-      nir_instr_as_alu(index_offset->parent_instr)->no_unsigned_wrap = true;
+      nir_def_as_alu(index_offset)->no_unsigned_wrap = true;
 
    if (non_uniform)
       return nir_iadd(b, load_desc_ptr(b, state, desc_set), index_offset);
 
    nir_def *addr = convert_pointer_to_64_bit(b, state, load_desc_ptr(b, state, desc_set));
-   nir_def *desc = nir_load_smem_amd(b, size, addr, index_offset, .align_mul = size * 4u);
+   nir_def *desc = ac_nir_load_smem(b, size, addr, index_offset, size * 4u, 0);
 
    if (desc_type == AC_DESC_IMAGE && state->has_image_load_dcc_bug && !tex && !write) {
       nir_def *comp[8];
@@ -393,7 +377,7 @@ load_push_constant(nir_builder *b, apply_layout_state *state, nir_intrinsic_inst
       if (size < (count - start) && can_increase_load_size(intrin, start * 4, size, size * 2))
          size *= 2;
 
-      data[num_loads++] = nir_load_smem_amd(b, size, addr, nir_iadd_imm_nuw(b, offset, start * 4));
+      data[num_loads++] = ac_nir_load_smem(b, size, addr, nir_iadd_imm_nuw(b, offset, start * 4), 4, 0);
       start += size;
    }
    return nir_extract_bits(b, data, num_loads, 0, intrin->def.num_components, bit_size);
@@ -459,6 +443,9 @@ apply_layout_to_tex(nir_builder *b, apply_layout_state *state, nir_tex_instr *te
    nir_deref_instr *sampler_deref_instr = NULL;
    int plane = -1;
 
+   nir_def *image = NULL;
+   nir_def *sampler = NULL;
+
    for (unsigned i = 0; i < tex->num_srcs; i++) {
       switch (tex->src[i].src_type) {
       case nir_tex_src_texture_deref:
@@ -470,13 +457,14 @@ apply_layout_to_tex(nir_builder *b, apply_layout_state *state, nir_tex_instr *te
       case nir_tex_src_plane:
          plane = nir_src_as_int(tex->src[i].src);
          break;
+      case nir_tex_src_sampler_handle:
+         sampler = tex->src[i].src.ssa;
+         break;
       default:
          break;
       }
    }
 
-   nir_def *image = NULL;
-   nir_def *sampler = NULL;
    if (plane >= 0) {
       assert(tex->op != nir_texop_txf_ms && tex->op != nir_texop_samples_identical);
       assert(tex->sampler_dim != GLSL_SAMPLER_DIM_BUF);
@@ -491,29 +479,31 @@ apply_layout_to_tex(nir_builder *b, apply_layout_state *state, nir_tex_instr *te
    }
 
    if (sampler_deref_instr) {
+      assert(!sampler);
       sampler = get_sampler_desc(b, state, sampler_deref_instr, AC_DESC_SAMPLER, tex->sampler_non_uniform, tex, false);
+   }
 
-      if (state->disable_aniso_single_level && tex->sampler_dim < GLSL_SAMPLER_DIM_RECT && state->gfx_level < GFX8) {
-         /* Disable anisotropic filtering if BASE_LEVEL == LAST_LEVEL.
-          *
-          * GFX6-GFX7:
-          *   If BASE_LEVEL == LAST_LEVEL, the shader must disable anisotropic
-          *   filtering manually. The driver sets img7 to a mask clearing
-          *   MAX_ANISO_RATIO if BASE_LEVEL == LAST_LEVEL. The shader must do:
-          *     s_and_b32 samp0, samp0, img7
-          *
-          * GFX8:
-          *   The ANISO_OVERRIDE sampler field enables this fix in TA.
-          */
-         /* TODO: This is unnecessary for combined image+sampler.
-          * We can do this when updating the desc set. */
-         nir_def *comp[4];
-         for (unsigned i = 0; i < 4; i++)
-            comp[i] = nir_channel(b, sampler, i);
-         comp[0] = nir_iand(b, comp[0], nir_channel(b, image, 7));
+   if (sampler && state->disable_aniso_single_level && tex->sampler_dim < GLSL_SAMPLER_DIM_RECT &&
+       state->gfx_level < GFX8) {
+      /* Disable anisotropic filtering if BASE_LEVEL == LAST_LEVEL.
+       *
+       * GFX6-GFX7:
+       *   If BASE_LEVEL == LAST_LEVEL, the shader must disable anisotropic
+       *   filtering manually. The driver sets img7 to a mask clearing
+       *   MAX_ANISO_RATIO if BASE_LEVEL == LAST_LEVEL. The shader must do:
+       *     s_and_b32 samp0, samp0, img7
+       *
+       * GFX8:
+       *   The ANISO_OVERRIDE sampler field enables this fix in TA.
+       */
+      /* TODO: This is unnecessary for combined image+sampler.
+       * We can do this when updating the desc set. */
+      nir_def *comp[4];
+      for (unsigned i = 0; i < 4; i++)
+         comp[i] = nir_channel(b, sampler, i);
+      comp[0] = nir_iand(b, comp[0], nir_channel(b, image, 7));
 
-         sampler = nir_vec(b, comp, 4);
-      }
+      sampler = nir_vec(b, comp, 4);
    }
 
    if (tex->op == nir_texop_descriptor_amd) {
@@ -531,6 +521,9 @@ apply_layout_to_tex(nir_builder *b, apply_layout_state *state, nir_tex_instr *te
          tex->src[i].src_type = nir_tex_src_sampler_handle;
          nir_src_rewrite(&tex->src[i].src, sampler);
          break;
+      case nir_tex_src_sampler_handle:
+         nir_src_rewrite(&tex->src[i].src, sampler);
+         break;
       default:
          break;
       }
@@ -549,9 +542,11 @@ radv_nir_apply_pipeline_layout(nir_shader *shader, struct radv_device *device, c
    apply_layout_state state = {
       .gfx_level = pdev->info.gfx_level,
       .address32_hi = pdev->info.address32_hi,
-      .disable_aniso_single_level = instance->drirc.disable_aniso_single_level,
+      .combined_image_sampler_desc_size = radv_get_combined_image_sampler_desc_size(pdev),
+      .combined_image_sampler_offset = radv_get_combined_image_sampler_offset(pdev),
+      .disable_aniso_single_level = instance->drirc.debug.disable_aniso_single_level,
       .has_image_load_dcc_bug = pdev->info.has_image_load_dcc_bug,
-      .disable_tg4_trunc_coord = !pdev->info.conformant_trunc_coord && !instance->drirc.disable_trunc_coord,
+      .disable_tg4_trunc_coord = !pdev->info.conformant_trunc_coord && !instance->drirc.debug.disable_trunc_coord,
       .args = &stage->args,
       .info = &stage->info,
       .layout = &stage->layout,
