@@ -132,9 +132,8 @@ lower_rt_derefs(nir_shader *shader)
 
    bool progress = false;
 
-   nir_builder b = nir_builder_at(nir_before_impl(impl));
-
-   nir_def *arg_offset = nir_load_rt_arg_scratch_offset_amd(&b);
+   nir_builder b;
+   nir_def *arg_offset = NULL;
 
    nir_foreach_block (block, impl) {
       nir_foreach_instr_safe (instr, block) {
@@ -149,6 +148,11 @@ lower_rt_derefs(nir_shader *shader)
          progress = true;
 
          if (deref->deref_type == nir_deref_type_var) {
+            if (!arg_offset) {
+               b = nir_builder_at(nir_before_impl(impl));
+               arg_offset = nir_load_rt_arg_scratch_offset_amd(&b);
+            }
+
             b.cursor = nir_before_instr(&deref->instr);
             nir_deref_instr *replacement =
                nir_build_deref_cast(&b, arg_offset, nir_var_function_temp, deref->var->type, 0);
@@ -202,6 +206,7 @@ struct rt_variables {
    nir_variable *tmax;
 
    /* Properties of the primitive currently being visited. */
+   nir_variable *primitive_addr;
    nir_variable *primitive_id;
    nir_variable *geometry_id_and_flags;
    nir_variable *instance_addr;
@@ -253,6 +258,7 @@ create_rt_variables(nir_shader *shader, struct radv_device *device, const VkPipe
    vars.direction = nir_variable_create(shader, nir_var_shader_temp, vec3_type, "ray_direction");
    vars.tmax = nir_variable_create(shader, nir_var_shader_temp, glsl_float_type(), "ray_tmax");
 
+   vars.primitive_addr = nir_variable_create(shader, nir_var_shader_temp, glsl_uint64_t_type(), "primitive_addr");
    vars.primitive_id = nir_variable_create(shader, nir_var_shader_temp, glsl_uint_type(), "primitive_id");
    vars.geometry_id_and_flags =
       nir_variable_create(shader, nir_var_shader_temp, glsl_uint_type(), "geometry_id_and_flags");
@@ -299,6 +305,7 @@ map_rt_variables(struct hash_table *var_remap, struct rt_variables *src, const s
    _mesa_hash_table_insert(var_remap, src->direction, dst->direction);
    _mesa_hash_table_insert(var_remap, src->tmax, dst->tmax);
 
+   _mesa_hash_table_insert(var_remap, src->primitive_addr, dst->primitive_addr);
    _mesa_hash_table_insert(var_remap, src->primitive_id, dst->primitive_id);
    _mesa_hash_table_insert(var_remap, src->geometry_id_and_flags, dst->geometry_id_and_flags);
    _mesa_hash_table_insert(var_remap, src->instance_addr, dst->instance_addr);
@@ -322,6 +329,8 @@ create_inner_vars(nir_builder *b, const struct rt_variables *vars)
    inner_vars.idx = nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "inner_idx");
    inner_vars.shader_record_ptr =
       nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint64_t_type(), "inner_shader_record_ptr");
+   inner_vars.primitive_addr =
+      nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint64_t_type(), "inner_primitive_addr");
    inner_vars.primitive_id =
       nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "inner_primitive_id");
    inner_vars.geometry_id_and_flags =
@@ -363,17 +372,19 @@ load_sbt_entry(nir_builder *b, const struct rt_variables *vars, nir_def *idx, en
 {
    nir_def *desc_base_addr = nir_load_sbt_base_amd(b);
 
-   nir_def *desc = nir_pack_64_2x32(b, nir_load_smem_amd(b, 2, desc_base_addr, nir_imm_int(b, binding)));
+   nir_def *desc = nir_pack_64_2x32(b, ac_nir_load_smem(b, 2, desc_base_addr, nir_imm_int(b, binding), 4, 0));
 
    nir_def *stride_offset = nir_imm_int(b, binding + (binding == SBT_RAYGEN ? 8 : 16));
-   nir_def *stride = nir_load_smem_amd(b, 1, desc_base_addr, stride_offset);
+   nir_def *stride = ac_nir_load_smem(b, 1, desc_base_addr, stride_offset, 4, 0);
 
    nir_def *addr = nir_iadd(b, desc, nir_u2u64(b, nir_iadd_imm(b, nir_imul(b, idx, stride), offset)));
 
    if (offset == SBT_RECURSIVE_PTR) {
-      nir_store_var(b, vars->shader_addr, nir_build_load_global(b, 1, 64, addr), 1);
+      nir_store_var(b, vars->shader_addr,
+                    nir_build_load_global(b, 1, 64, addr, .access = ACCESS_CAN_REORDER | ACCESS_NON_WRITEABLE), 1);
    } else {
-      nir_store_var(b, vars->idx, nir_build_load_global(b, 1, 32, addr), 1);
+      nir_store_var(b, vars->idx,
+                    nir_build_load_global(b, 1, 32, addr, .access = ACCESS_CAN_REORDER | ACCESS_NON_WRITEABLE), 1);
    }
 
    nir_def *record_addr = nir_iadd_imm(b, addr, RADV_RT_HANDLE_SIZE - offset);
@@ -649,10 +660,11 @@ radv_lower_rt_instruction(nir_builder *b, nir_instr *instr, void *_data)
    }
    case nir_intrinsic_execute_closest_hit_amd: {
       nir_store_var(b, vars->tmax, intr->src[1].ssa, 0x1);
-      nir_store_var(b, vars->primitive_id, intr->src[2].ssa, 0x1);
-      nir_store_var(b, vars->instance_addr, intr->src[3].ssa, 0x1);
-      nir_store_var(b, vars->geometry_id_and_flags, intr->src[4].ssa, 0x1);
-      nir_store_var(b, vars->hit_kind, intr->src[5].ssa, 0x1);
+      nir_store_var(b, vars->primitive_addr, intr->src[2].ssa, 0x1);
+      nir_store_var(b, vars->primitive_id, intr->src[3].ssa, 0x1);
+      nir_store_var(b, vars->instance_addr, intr->src[4].ssa, 0x1);
+      nir_store_var(b, vars->geometry_id_and_flags, intr->src[5].ssa, 0x1);
+      nir_store_var(b, vars->hit_kind, intr->src[6].ssa, 0x1);
       load_sbt_entry(b, vars, intr->src[0].ssa, SBT_HIT, SBT_RECURSIVE_PTR);
 
       nir_def *should_return =
@@ -689,11 +701,8 @@ radv_lower_rt_instruction(nir_builder *b, nir_instr *instr, void *_data)
       break;
    }
    case nir_intrinsic_load_ray_triangle_vertex_positions: {
-      nir_def *instance_node_addr = nir_load_var(b, vars->instance_addr);
-      nir_def *primitive_id = nir_load_var(b, vars->primitive_id);
-      nir_def *geometry_id = nir_iand_imm(b, nir_load_var(b, vars->geometry_id_and_flags), 0xFFFFFFF);
-      ret = radv_load_vertex_position(vars->device, b, instance_node_addr, geometry_id, primitive_id,
-                                      nir_intrinsic_column(intr));
+      nir_def *primitive_addr = nir_load_var(b, vars->primitive_addr);
+      ret = radv_load_vertex_position(vars->device, b, primitive_addr, nir_intrinsic_column(intr));
       break;
    }
    default:
@@ -924,8 +933,8 @@ radv_build_end_trace_token(nir_builder *b, struct rt_variables *vars, nir_def *t
       dst_addr = nir_iadd_imm(b, dst_addr, 8);
 
       nir_def *dispatch_indices =
-         nir_load_smem_amd(b, 2, nir_imm_int64(b, vars->device->rra_trace.ray_history_addr),
-                           nir_imm_int(b, offsetof(struct radv_ray_history_header, dispatch_index)), .align_mul = 4);
+         ac_nir_load_smem(b, 2, nir_imm_int64(b, vars->device->rra_trace.ray_history_addr),
+                          nir_imm_int(b, offsetof(struct radv_ray_history_header, dispatch_index)), 4, 0);
       nir_def *dispatch_index = nir_iadd(b, nir_channel(b, dispatch_indices, 0), nir_channel(b, dispatch_indices, 1));
       nir_def *dispatch_and_flags = nir_iand_imm(b, nir_load_var(b, vars->cull_mask_and_flags), 0xFFFF);
       dispatch_and_flags = nir_ior(b, dispatch_and_flags, dispatch_index);
@@ -1081,7 +1090,7 @@ lower_any_hit_for_intersection(nir_shader *any_hit)
             case nir_intrinsic_load_rt_arg_scratch_offset_amd:
                b->cursor = nir_after_instr(instr);
                nir_def *arg_offset = nir_isub(b, &intrin->def, scratch_offset);
-               nir_def_rewrite_uses_after(&intrin->def, arg_offset, arg_offset->parent_instr);
+               nir_def_rewrite_uses_after(&intrin->def, arg_offset);
                break;
 
             default:
@@ -1221,8 +1230,10 @@ struct rt_traversal_vars {
    nir_variable *stack_low_watermark;
    nir_variable *current_node;
    nir_variable *previous_node;
+   nir_variable *parent_node;
    nir_variable *instance_top_node;
    nir_variable *instance_bottom_node;
+   nir_variable *second_iteration;
 };
 
 static struct rt_traversal_vars
@@ -1245,9 +1256,11 @@ init_traversal_vars(nir_builder *b)
       nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "traversal_stack_low_watermark");
    ret.current_node = nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "current_node;");
    ret.previous_node = nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "previous_node");
+   ret.parent_node = nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "parent_node");
    ret.instance_top_node = nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "instance_top_node");
    ret.instance_bottom_node =
       nir_variable_create(b->shader, nir_var_shader_temp, glsl_uint_type(), "instance_bottom_node");
+   ret.second_iteration = nir_variable_create(b->shader, nir_var_shader_temp, glsl_bool_type(), "second_iteration");
    return ret;
 }
 
@@ -1385,6 +1398,7 @@ handle_candidate_triangle(nir_builder *b, struct radv_triangle_intersection *int
    {
       struct rt_variables inner_vars = create_inner_vars(b, data->vars);
 
+      nir_store_var(b, inner_vars.primitive_addr, intersection->base.node_addr, 1);
       nir_store_var(b, inner_vars.primitive_id, intersection->base.primitive_id, 1);
       nir_store_var(b, inner_vars.geometry_id_and_flags, intersection->base.geometry_id_and_flags, 1);
       nir_store_var(b, inner_vars.tmax, intersection->t, 0x1);
@@ -1418,6 +1432,7 @@ handle_candidate_triangle(nir_builder *b, struct radv_triangle_intersection *int
 
    nir_push_if(b, nir_load_var(b, data->vars->ahit_accept));
    {
+      nir_store_var(b, data->vars->primitive_addr, intersection->base.node_addr, 1);
       nir_store_var(b, data->vars->primitive_id, intersection->base.primitive_id, 1);
       nir_store_var(b, data->vars->geometry_id_and_flags, intersection->base.geometry_id_and_flags, 1);
       nir_store_var(b, data->vars->tmax, intersection->t, 0x1);
@@ -1452,6 +1467,7 @@ handle_candidate_aabb(nir_builder *b, struct radv_leaf_intersection *intersectio
     * next closest hit candidate. */
    inner_vars.hit_kind = data->vars->hit_kind;
 
+   nir_store_var(b, inner_vars.primitive_addr, intersection->node_addr, 1);
    nir_store_var(b, inner_vars.primitive_id, intersection->primitive_id, 1);
    nir_store_var(b, inner_vars.geometry_id_and_flags, intersection->geometry_id_and_flags, 1);
    nir_store_var(b, inner_vars.tmax, nir_load_var(b, data->vars->tmax), 0x1);
@@ -1481,6 +1497,7 @@ handle_candidate_aabb(nir_builder *b, struct radv_leaf_intersection *intersectio
 
    nir_push_if(b, nir_load_var(b, data->vars->ahit_accept));
    {
+      nir_store_var(b, data->vars->primitive_addr, intersection->node_addr, 1);
       nir_store_var(b, data->vars->primitive_id, intersection->primitive_id, 1);
       nir_store_var(b, data->vars->geometry_id_and_flags, intersection->geometry_id_and_flags, 1);
       nir_store_var(b, data->vars->tmax, nir_load_var(b, inner_vars.tmax), 0x1);
@@ -1523,9 +1540,25 @@ radv_build_traversal(struct radv_device *device, struct radv_ray_tracing_pipelin
    nir_store_var(b, trav_vars.hit, nir_imm_false(b), 1);
 
    nir_def *accel_struct = nir_load_var(b, vars->accel_struct);
-   nir_def *bvh_offset = nir_build_load_global(
-      b, 1, 32, nir_iadd_imm(b, accel_struct, offsetof(struct radv_accel_struct_header, bvh_offset)),
-      .access = ACCESS_NON_WRITEABLE);
+
+   nir_def *zero = nir_imm_int(b, 0);
+   nir_def *bvh_offset;
+   nir_push_if(b, nir_ine_imm(b, accel_struct, 0));
+   {
+      bvh_offset = nir_build_load_global(
+         b, 1, 32, nir_iadd_imm(b, accel_struct, offsetof(struct radv_accel_struct_header, bvh_offset)),
+         .access = ACCESS_NON_WRITEABLE);
+      nir_store_var(b, trav_vars.current_node, nir_imm_int(b, RADV_BVH_ROOT_NODE), 0x1);
+   }
+   nir_push_else(b, NULL);
+   {
+      nir_store_var(b, trav_vars.current_node,
+                    nir_imm_int(b, radv_use_bvh_stack_rtn(pdev) ? RADV_BVH_STACK_TERMINAL_NODE : RADV_BVH_INVALID_NODE),
+                    0x1);
+   }
+   nir_pop_if(b, NULL);
+   bvh_offset = nir_if_phi(b, bvh_offset, zero);
+
    nir_def *root_bvh_base = nir_iadd(b, accel_struct, nir_u2u64(b, bvh_offset));
    root_bvh_base = build_addr_to_node(device, b, root_bvh_base, cull_mask_and_flags);
 
@@ -1552,10 +1585,11 @@ radv_build_traversal(struct radv_device *device, struct radv_ray_tracing_pipelin
 
    nir_store_var(b, trav_vars.stack, stack_idx, 1);
    nir_store_var(b, trav_vars.stack_low_watermark, nir_load_var(b, trav_vars.stack), 1);
-   nir_store_var(b, trav_vars.current_node, nir_imm_int(b, RADV_BVH_ROOT_NODE), 0x1);
    nir_store_var(b, trav_vars.previous_node, nir_imm_int(b, RADV_BVH_INVALID_NODE), 0x1);
+   nir_store_var(b, trav_vars.parent_node, nir_imm_int(b, RADV_BVH_INVALID_NODE), 0x1);
    nir_store_var(b, trav_vars.instance_top_node, nir_imm_int(b, RADV_BVH_INVALID_NODE), 0x1);
    nir_store_var(b, trav_vars.instance_bottom_node, nir_imm_int(b, RADV_BVH_NO_INSTANCE_ROOT), 0x1);
+   nir_store_var(b, trav_vars.second_iteration, nir_imm_false(b), 0x1);
 
    nir_store_var(b, trav_vars.top_stack, nir_imm_int(b, -1), 1);
 
@@ -1570,8 +1604,10 @@ radv_build_traversal(struct radv_device *device, struct radv_ray_tracing_pipelin
       .stack_low_watermark = nir_build_deref_var(b, trav_vars.stack_low_watermark),
       .current_node = nir_build_deref_var(b, trav_vars.current_node),
       .previous_node = nir_build_deref_var(b, trav_vars.previous_node),
+      .parent_node = nir_build_deref_var(b, trav_vars.parent_node),
       .instance_top_node = nir_build_deref_var(b, trav_vars.instance_top_node),
       .instance_bottom_node = nir_build_deref_var(b, trav_vars.instance_bottom_node),
+      .second_iteration = nir_build_deref_var(b, trav_vars.second_iteration),
       .instance_addr = nir_build_deref_var(b, trav_vars.instance_addr),
       .sbt_offset_and_flags = nir_build_deref_var(b, trav_vars.sbt_offset_and_flags),
    };
@@ -1673,7 +1709,14 @@ radv_build_traversal(struct radv_device *device, struct radv_ray_tracing_pipelin
       } else {
          for (int i = 0; i < ARRAY_SIZE(hit_attribs); ++i)
             nir_store_hit_attrib_amd(b, nir_load_var(b, hit_attribs[i]), .base = i);
-         nir_execute_closest_hit_amd(b, nir_load_var(b, vars->idx), nir_load_var(b, vars->tmax),
+
+         nir_def *primitive_addr;
+         if (info->has_position_fetch)
+            primitive_addr = nir_load_var(b, vars->primitive_addr);
+         else
+            primitive_addr = nir_undef(b, 1, 64);
+
+         nir_execute_closest_hit_amd(b, nir_load_var(b, vars->idx), nir_load_var(b, vars->tmax), primitive_addr,
                                      nir_load_var(b, vars->primitive_id), nir_load_var(b, vars->instance_addr),
                                      nir_load_var(b, vars->geometry_id_and_flags), nir_load_var(b, vars->hit_kind));
       }
@@ -1715,6 +1758,9 @@ radv_build_traversal_shader(struct radv_device *device, struct radv_ray_tracing_
    b.shader->info.internal = false;
    b.shader->info.workgroup_size[0] = 8;
    b.shader->info.workgroup_size[1] = pdev->rt_wave_size == 64 ? 8 : 4;
+   b.shader->info.api_subgroup_size = pdev->rt_wave_size;
+   b.shader->info.max_subgroup_size = pdev->rt_wave_size;
+   b.shader->info.min_subgroup_size = pdev->rt_wave_size;
    b.shader->info.shared_size = pdev->rt_wave_size * MAX_STACK_ENTRY_COUNT * sizeof(uint32_t);
    struct rt_variables vars = create_rt_variables(b.shader, device, create_flags, false);
 
@@ -1820,13 +1866,13 @@ lower_rt_instruction_monolithic(nir_builder *b, nir_instr *instr, void *data)
       return true;
    }
    case nir_intrinsic_rt_resume:
-      unreachable("nir_intrinsic_rt_resume");
+      UNREACHABLE("nir_intrinsic_rt_resume");
    case nir_intrinsic_rt_return_amd:
-      unreachable("nir_intrinsic_rt_return_amd");
+      UNREACHABLE("nir_intrinsic_rt_return_amd");
    case nir_intrinsic_execute_closest_hit_amd:
-      unreachable("nir_intrinsic_execute_closest_hit_amd");
+      UNREACHABLE("nir_intrinsic_execute_closest_hit_amd");
    case nir_intrinsic_execute_miss_amd:
-      unreachable("nir_intrinsic_execute_miss_amd");
+      UNREACHABLE("nir_intrinsic_execute_miss_amd");
    default:
       return false;
    }
@@ -1890,7 +1936,7 @@ lower_rt_instructions_monolithic(nir_shader *shader, struct radv_device *device,
 static nir_def *
 select_next_shader(nir_builder *b, nir_def *shader_addr, unsigned wave_size)
 {
-   gl_shader_stage stage = b->shader->info.stage;
+   mesa_shader_stage stage = b->shader->info.stage;
    nir_def *prio = nir_iand_imm(b, shader_addr, radv_rt_priority_mask);
    nir_def *ballot = nir_ballot(b, 1, wave_size, nir_imm_bool(b, true));
    nir_def *ballot_traversal = nir_ballot(b, 1, wave_size, nir_ieq_imm(b, prio, radv_rt_priority_traversal));
@@ -1922,8 +1968,11 @@ void
 radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKHR *pCreateInfo,
                       const struct radv_shader_args *args, const struct radv_shader_info *info, uint32_t *stack_size,
                       bool resume_shader, struct radv_device *device, struct radv_ray_tracing_pipeline *pipeline,
-                      bool monolithic, const struct radv_ray_tracing_stage_info *traversal_info)
+                      bool monolithic, bool has_position_fetch,
+                      const struct radv_ray_tracing_stage_info *traversal_info)
 {
+   const struct radv_physical_device *pdev = radv_device_physical(device);
+
    nir_function_impl *impl = nir_shader_get_entrypoint(shader);
 
    const VkPipelineCreateFlagBits2 create_flags = vk_rt_pipeline_create_flags(pCreateInfo);
@@ -1951,8 +2000,9 @@ radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKH
    /* initialize variables */
    nir_builder b = nir_builder_at(nir_before_impl(impl));
 
-   nir_def *descriptor_sets = ac_nir_load_arg(&b, &args->ac, args->descriptor_sets[0]);
+   nir_def *descriptors = ac_nir_load_arg(&b, &args->ac, args->descriptors[0]);
    nir_def *push_constants = ac_nir_load_arg(&b, &args->ac, args->ac.push_constants);
+   nir_def *dynamic_descriptors = ac_nir_load_arg(&b, &args->ac, args->ac.dynamic_descriptors);
    nir_def *sbt_descriptors = ac_nir_load_arg(&b, &args->ac, args->ac.rt.sbt_descriptors);
 
    nir_def *launch_sizes[3];
@@ -1975,7 +2025,8 @@ radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKH
    }
 
    nir_def *traversal_addr = ac_nir_load_arg(&b, &args->ac, args->ac.rt.traversal_shader_addr);
-   nir_store_var(&b, vars.traversal_addr, nir_pack_64_2x32(&b, traversal_addr), 1);
+   nir_store_var(&b, vars.traversal_addr,
+                 nir_pack_64_2x32_split(&b, traversal_addr, nir_imm_int(&b, pdev->info.address32_hi)), 1);
 
    nir_def *shader_addr = ac_nir_load_arg(&b, &args->ac, args->ac.rt.shader_addr);
    shader_addr = nir_pack_64_2x32(&b, shader_addr);
@@ -2001,6 +2052,8 @@ radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKH
    else
       nir_store_var(&b, vars.miss_index, ac_nir_load_arg(&b, &args->ac, args->ac.rt.miss_index), 0x1);
 
+   nir_def *primitive_addr = ac_nir_load_arg(&b, &args->ac, args->ac.rt.primitive_addr);
+   nir_store_var(&b, vars.primitive_addr, nir_pack_64_2x32(&b, primitive_addr), 1);
    nir_store_var(&b, vars.primitive_id, ac_nir_load_arg(&b, &args->ac, args->ac.rt.primitive_id), 1);
    nir_def *instance_addr = ac_nir_load_arg(&b, &args->ac, args->ac.rt.instance_addr);
    nir_store_var(&b, vars.instance_addr, nir_pack_64_2x32(&b, instance_addr), 1);
@@ -2025,16 +2078,15 @@ radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKH
 
    b.cursor = nir_after_impl(impl);
 
-   if (monolithic) {
-      nir_terminate(&b);
-   } else {
+   if (!monolithic) {
       /* select next shader */
       shader_addr = nir_load_var(&b, vars.shader_addr);
       nir_def *next = select_next_shader(&b, shader_addr, info->wave_size);
       ac_nir_store_arg(&b, &args->ac, args->ac.rt.uniform_shader_addr, next);
 
-      ac_nir_store_arg(&b, &args->ac, args->descriptor_sets[0], descriptor_sets);
+      ac_nir_store_arg(&b, &args->ac, args->descriptors[0], descriptors);
       ac_nir_store_arg(&b, &args->ac, args->ac.push_constants, push_constants);
+      ac_nir_store_arg(&b, &args->ac, args->ac.dynamic_descriptors, dynamic_descriptors);
       ac_nir_store_arg(&b, &args->ac, args->ac.rt.sbt_descriptors, sbt_descriptors);
       ac_nir_store_arg(&b, &args->ac, args->ac.rt.traversal_shader_addr, traversal_addr);
 
@@ -2073,6 +2125,9 @@ radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKH
       radv_store_arg(&b, args, traversal_info, args->ac.rt.ray_direction, nir_load_var(&b, vars.direction));
       radv_store_arg(&b, args, traversal_info, args->ac.rt.ray_tmax, nir_load_var(&b, vars.tmax));
 
+      if (has_position_fetch)
+         radv_store_arg(&b, args, traversal_info, args->ac.rt.primitive_addr, nir_load_var(&b, vars.primitive_addr));
+
       radv_store_arg(&b, args, traversal_info, args->ac.rt.primitive_id, nir_load_var(&b, vars.primitive_id));
       radv_store_arg(&b, args, traversal_info, args->ac.rt.instance_addr, nir_load_var(&b, vars.instance_addr));
       radv_store_arg(&b, args, traversal_info, args->ac.rt.geometry_id_and_flags,
