@@ -8,6 +8,7 @@
 #include "radv_cs.h"
 #include "radv_entrypoints.h"
 
+#include "radix_sort/common/vk/barrier.h"
 #include "radix_sort/radix_sort_u64.h"
 
 #include "bvh/build_interface.h"
@@ -48,9 +49,10 @@ static const uint32_t leaf_spv[] = {
 #include "bvh/radv_leaf.spv.h"
 };
 
+#define RADV_OFFSET_UNUSED 0xffffffff
+
 struct acceleration_structure_layout {
    uint32_t geometry_info_offset;
-   uint32_t primitive_base_indices_offset;
    uint32_t leaf_node_offsets_offset;
    uint32_t bvh_offset;
    uint32_t leaf_nodes_offset;
@@ -66,7 +68,9 @@ struct update_scratch_layout {
 };
 
 enum radv_encode_key_bits {
-   RADV_ENCODE_KEY_COMPACT = 1,
+   RADV_ENCODE_KEY_COMPACT = (1 << 0),
+   RADV_ENCODE_KEY_WRITE_LEAF_NODE_OFFSETS = (1 << 1),
+   RADV_ENCODE_KEY_PAIR_COMPRESS_GFX12 = (1 << 2),
 };
 
 static void
@@ -81,7 +85,6 @@ radv_get_acceleration_structure_layout(struct radv_device *device,
    VkGeometryTypeKHR geometry_type = vk_get_as_geometry_type(state->build_info);
 
    uint32_t bvh_leaf_size;
-   uint32_t bvh_node_size_gcd;
    if (radv_use_bvh8(pdev)) {
       switch (geometry_type) {
       case VK_GEOMETRY_TYPE_TRIANGLES_KHR:
@@ -94,9 +97,8 @@ radv_get_acceleration_structure_layout(struct radv_device *device,
          bvh_leaf_size = sizeof(struct radv_gfx12_instance_node) + sizeof(struct radv_gfx12_instance_node_user_data);
          break;
       default:
-         unreachable("Unknown VkGeometryTypeKHR");
+         UNREACHABLE("Unknown VkGeometryTypeKHR");
       }
-      bvh_node_size_gcd = RADV_GFX12_BVH_NODE_SIZE;
    } else {
       switch (geometry_type) {
       case VK_GEOMETRY_TYPE_TRIANGLES_KHR:
@@ -109,9 +111,8 @@ radv_get_acceleration_structure_layout(struct radv_device *device,
          bvh_leaf_size = sizeof(struct radv_bvh_instance_node);
          break;
       default:
-         unreachable("Unknown VkGeometryTypeKHR");
+         UNREACHABLE("Unknown VkGeometryTypeKHR");
       }
-      bvh_node_size_gcd = 64;
    }
 
    uint32_t internal_node_size =
@@ -124,23 +125,21 @@ radv_get_acceleration_structure_layout(struct radv_device *device,
    if (device->rra_trace.accel_structs) {
       accel_struct->geometry_info_offset = offset;
       offset += sizeof(struct radv_accel_struct_geometry_info) * state->build_info->geometryCount;
-   }
-
-   if (device->vk.enabled_features.rayTracingPositionFetch && geometry_type == VK_GEOMETRY_TYPE_TRIANGLES_KHR) {
-      accel_struct->primitive_base_indices_offset = offset;
-      offset += sizeof(uint32_t) * state->build_info->geometryCount;
+   } else {
+      accel_struct->geometry_info_offset = RADV_OFFSET_UNUSED;
    }
 
    /* On GFX12, we need additional space for leaf node offsets since they do not have the same
     * order as the application provided data.
     */
    accel_struct->leaf_node_offsets_offset = offset;
-   if (radv_use_bvh8(pdev))
+   if (state->config.encode_key[0] & RADV_ENCODE_KEY_WRITE_LEAF_NODE_OFFSETS)
       offset += state->leaf_node_count * 4;
 
    /* Parent links, which have to go directly before bvh_offset as we index them using negative
     * offsets from there. */
-   offset += bvh_size / bvh_node_size_gcd * 4;
+   if (!radv_use_bvh8(pdev))
+      offset += bvh_size / 64 * 4;
 
    /* The BVH and hence bvh_offset needs 64 byte alignment for RT nodes. */
    offset = ALIGN(offset, 64);
@@ -219,161 +218,6 @@ radv_device_finish_accel_struct_build_state(struct radv_device *device)
 
    if (state->accel_struct_build.radix_sort)
       radix_sort_vk_destroy(state->accel_struct_build.radix_sort, _device, &state->alloc);
-
-   radv_DestroyBuffer(_device, state->accel_struct_build.null.buffer, &state->alloc);
-   radv_FreeMemory(_device, state->accel_struct_build.null.memory, &state->alloc);
-   vk_common_DestroyAccelerationStructureKHR(_device, state->accel_struct_build.null.accel_struct, &state->alloc);
-}
-
-VkResult
-radv_device_init_null_accel_struct(struct radv_device *device)
-{
-   const struct radv_physical_device *pdev = radv_device_physical(device);
-
-   if (pdev->memory_properties.memoryTypeCount == 0)
-      return VK_SUCCESS; /* Exit in the case of null winsys. */
-
-   VkDevice _device = radv_device_to_handle(device);
-
-   uint32_t bvh_offset = ALIGN(sizeof(struct radv_accel_struct_header), 64);
-   uint32_t size = bvh_offset;
-   if (radv_use_bvh8(pdev))
-      size += sizeof(struct radv_gfx12_box_node);
-   else
-      size += sizeof(struct radv_bvh_box32_node);
-
-   VkResult result;
-
-   VkBuffer buffer = VK_NULL_HANDLE;
-   VkDeviceMemory memory = VK_NULL_HANDLE;
-   VkAccelerationStructureKHR accel_struct = VK_NULL_HANDLE;
-
-   VkBufferCreateInfo buffer_create_info = {
-      .sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO,
-      .pNext =
-         &(VkBufferUsageFlags2CreateInfo){
-            .sType = VK_STRUCTURE_TYPE_BUFFER_USAGE_FLAGS_2_CREATE_INFO,
-            .usage = VK_BUFFER_USAGE_2_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR,
-         },
-      .size = size,
-      .sharingMode = VK_SHARING_MODE_EXCLUSIVE,
-   };
-
-   result = radv_CreateBuffer(_device, &buffer_create_info, &device->meta_state.alloc, &buffer);
-   if (result != VK_SUCCESS)
-      return result;
-
-   VkMemoryRequirements2 mem_req = {
-      .sType = VK_STRUCTURE_TYPE_MEMORY_REQUIREMENTS_2,
-   };
-
-   VkDeviceBufferMemoryRequirements buffer_mem_req_info = {
-      .sType = VK_STRUCTURE_TYPE_DEVICE_BUFFER_MEMORY_REQUIREMENTS,
-      .pCreateInfo = &buffer_create_info,
-   };
-
-   radv_GetDeviceBufferMemoryRequirements(radv_device_to_handle(device), &buffer_mem_req_info, &mem_req);
-
-   VkMemoryAllocateInfo alloc_info = {
-      .sType = VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO,
-      .allocationSize = mem_req.memoryRequirements.size,
-      .memoryTypeIndex =
-         radv_find_memory_index(pdev, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT |
-                                         VK_MEMORY_PROPERTY_HOST_COHERENT_BIT),
-   };
-
-   result = radv_AllocateMemory(_device, &alloc_info, &device->meta_state.alloc, &memory);
-   if (result != VK_SUCCESS)
-      return result;
-
-   VkBindBufferMemoryInfo bind_info = {
-      .sType = VK_STRUCTURE_TYPE_BIND_BUFFER_MEMORY_INFO,
-      .buffer = buffer,
-      .memory = memory,
-   };
-
-   result = radv_BindBufferMemory2(_device, 1, &bind_info);
-   if (result != VK_SUCCESS)
-      return result;
-
-   VkMemoryMapInfo memory_map_info = {
-      .sType = VK_STRUCTURE_TYPE_MEMORY_MAP_INFO,
-      .memory = memory,
-      .size = size,
-   };
-   void *data;
-
-   result = radv_MapMemory2(_device, &memory_map_info, &data);
-   if (result != VK_SUCCESS)
-      return result;
-
-   struct radv_accel_struct_header header = {
-      .bvh_offset = bvh_offset,
-   };
-   memcpy(data, &header, sizeof(struct radv_accel_struct_header));
-
-   if (radv_use_bvh8(pdev)) {
-      struct radv_gfx12_box_node root = {
-         .obb_matrix_index = 0x7f,
-      };
-
-      for (uint32_t child = 0; child < 8; child++) {
-         root.children[child] = (struct radv_gfx12_box_child){
-            .dword0 = 0xffffffff,
-            .dword1 = 0xfff,
-            .dword2 = 0,
-         };
-      }
-
-      memcpy((uint8_t *)data + bvh_offset, &root, sizeof(struct radv_gfx12_box_node));
-   } else {
-      struct radv_bvh_box32_node root = {
-         .children =
-            {
-               RADV_BVH_INVALID_NODE,
-               RADV_BVH_INVALID_NODE,
-               RADV_BVH_INVALID_NODE,
-               RADV_BVH_INVALID_NODE,
-            },
-      };
-
-      for (uint32_t child = 0; child < 4; child++) {
-         root.coords[child] = (vk_aabb){
-            .min.x = NAN,
-            .min.y = NAN,
-            .min.z = NAN,
-            .max.x = NAN,
-            .max.y = NAN,
-            .max.z = NAN,
-         };
-      }
-
-      memcpy((uint8_t *)data + bvh_offset, &root, sizeof(struct radv_bvh_box32_node));
-   }
-
-   VkMemoryUnmapInfo unmap_info = {
-      .sType = VK_STRUCTURE_TYPE_MEMORY_UNMAP_INFO,
-      .memory = memory,
-   };
-
-   radv_UnmapMemory2(_device, &unmap_info);
-
-   VkAccelerationStructureCreateInfoKHR create_info = {
-      .sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CREATE_INFO_KHR,
-      .buffer = buffer,
-      .size = size,
-      .type = VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR,
-   };
-
-   result = vk_common_CreateAccelerationStructureKHR(_device, &create_info, &device->meta_state.alloc, &accel_struct);
-   if (result != VK_SUCCESS)
-      return result;
-
-   device->meta_state.accel_struct_build.null.buffer = buffer;
-   device->meta_state.accel_struct_build.null.memory = memory;
-   device->meta_state.accel_struct_build.null.accel_struct = accel_struct;
-
-   return VK_SUCCESS;
 }
 
 static VkDeviceSize
@@ -403,9 +247,29 @@ radv_get_build_config(VkDevice _device, struct vk_acceleration_structure_build_s
    struct radv_physical_device *pdev = radv_device_physical(device);
 
    uint32_t encode_key = 0;
-   if (radv_use_bvh8(pdev))
+   if (radv_use_bvh8(pdev)) {
       encode_key |= RADV_ENCODE_KEY_COMPACT;
 
+      /*
+       * Leaf nodes are not written in the order provided by the application when BVH8 encoding is used.
+       * The proper order leaf nodes is used...
+       *    1. When fetching vertex positions for triangles.
+       *    2. When applying/writing BLAS pointers during TLAS deserialization/serialization. The type
+       *       compared to VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR to handle the
+       *       VK_ACCELERATION_STRUCTURE_TYPE_GENERIC_KHR case when the application queries the
+       *       acceleration structure size.
+       */
+      if ((state->build_info->flags & VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DATA_ACCESS_KHR) ||
+          state->build_info->type != VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR)
+         encode_key |= RADV_ENCODE_KEY_WRITE_LEAF_NODE_OFFSETS;
+
+      VkGeometryTypeKHR geometry_type = vk_get_as_geometry_type(state->build_info);
+      if (!(state->build_info->flags & (VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_UPDATE_BIT_KHR |
+                                        VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DATA_ACCESS_KHR)) &&
+          geometry_type == VK_GEOMETRY_TYPE_TRIANGLES_KHR)
+         encode_key |= RADV_ENCODE_KEY_PAIR_COMPRESS_GFX12;
+   }
+
    if (state->build_info->flags & VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_COMPACTION_BIT_KHR)
       encode_key |= RADV_ENCODE_KEY_COMPACT;
 
@@ -416,6 +280,9 @@ radv_get_build_config(VkDevice _device, struct vk_acceleration_structure_build_s
    if (state->build_info->srcAccelerationStructure == state->build_info->dstAccelerationStructure)
       update_key |= RADV_BUILD_FLAG_UPDATE_IN_PLACE;
 
+   if (state->build_info->geometryCount == 1)
+      update_key |= RADV_BUILD_FLAG_UPDATE_SINGLE_GEOMETRY;
+
    if (device->meta_state.accel_struct_build.build_args.propagate_cull_flags)
       update_key |= VK_BUILD_FLAG_PROPAGATE_CULL_FLAGS;
 
@@ -432,7 +299,7 @@ radv_bvh_build_bind_pipeline(VkCommandBuffer commandBuffer, enum radv_meta_objec
    VkPipeline pipeline;
    VkResult result = vk_get_bvh_build_pipeline_spv(
       &device->vk, &device->meta_state.device, (enum vk_meta_object_key_type)type, spirv, spirv_size,
-      push_constants_size, &device->meta_state.accel_struct_build.build_args, flags, &pipeline);
+      push_constants_size, &device->meta_state.accel_struct_build.build_args, flags, &pipeline, false);
 
    if (result != VK_SUCCESS) {
       vk_command_buffer_set_error(&cmd_buffer->vk, result);
@@ -476,11 +343,17 @@ radv_build_flags(VkCommandBuffer commandBuffer, uint32_t key)
       flags |= RADV_BUILD_FLAG_COMPACT;
    if (radv_use_bvh8(pdev))
       flags |= RADV_BUILD_FLAG_BVH8;
-   /* gfx11 box intersection tests can return garbage with infs and non-standard box sorting */
-   if (pdev->info.gfx_level == GFX11)
-      flags |= RADV_BUILD_FLAG_NO_INFS;
-   if (pdev->info.gfx_level >= GFX11)
-      flags |= VK_BUILD_FLAG_PROPAGATE_CULL_FLAGS;
+   if (!radv_emulate_rt(pdev)) {
+      /* gfx11 box intersection tests can return garbage with infs and non-standard box sorting */
+      if (pdev->info.gfx_level == GFX11)
+         flags |= RADV_BUILD_FLAG_NO_INFS;
+      if (pdev->info.gfx_level >= GFX11)
+         flags |= VK_BUILD_FLAG_PROPAGATE_CULL_FLAGS;
+   }
+   if (key & RADV_ENCODE_KEY_WRITE_LEAF_NODE_OFFSETS)
+      flags |= RADV_BUILD_FLAG_WRITE_LEAF_NODE_OFFSETS;
+   if (key & RADV_ENCODE_KEY_PAIR_COMPRESS_GFX12)
+      flags |= RADV_BUILD_FLAG_PAIR_COMPRESS_TRIANGLES;
 
    return flags;
 }
@@ -500,7 +373,8 @@ radv_encode_bind_pipeline_gfx12(VkCommandBuffer commandBuffer,
                                 const struct vk_acceleration_structure_build_state *state)
 {
    radv_bvh_build_bind_pipeline(commandBuffer, RADV_META_OBJECT_KEY_BVH_ENCODE, encode_gfx12_spv,
-                                sizeof(encode_gfx12_spv), sizeof(struct encode_gfx12_args), 0);
+                                sizeof(encode_gfx12_spv), sizeof(struct encode_gfx12_args),
+                                radv_build_flags(commandBuffer, state->config.encode_key[0]));
 
    return VK_SUCCESS;
 }
@@ -599,17 +473,11 @@ radv_encode_as_gfx12(VkCommandBuffer commandBuffer, const struct vk_acceleration
 static VkResult
 radv_init_header_bind_pipeline(VkCommandBuffer commandBuffer, const struct vk_acceleration_structure_build_state *state)
 {
-   VK_FROM_HANDLE(radv_cmd_buffer, cmd_buffer, commandBuffer);
-
    if (!(state->config.encode_key[1] & RADV_ENCODE_KEY_COMPACT))
       return VK_SUCCESS;
 
    /* Wait for encoding to finish. */
-   cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_CS_PARTIAL_FLUSH |
-                                   radv_src_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                         VK_ACCESS_2_SHADER_WRITE_BIT, 0, NULL, NULL) |
-                                   radv_dst_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                         VK_ACCESS_2_SHADER_READ_BIT, 0, NULL, NULL);
+   vk_barrier_compute_w_to_compute_r(commandBuffer);
 
    radv_bvh_build_bind_pipeline(commandBuffer, RADV_META_OBJECT_KEY_BVH_HEADER, header_spv, sizeof(header_spv),
                                 sizeof(struct header_args), 0);
@@ -670,12 +538,11 @@ radv_init_header(VkCommandBuffer commandBuffer, const struct vk_acceleration_str
    header.build_flags = state->build_info->flags;
    header.geometry_type = vk_get_as_geometry_type(state->build_info);
    header.geometry_count = state->build_info->geometryCount;
-   header.primitive_base_indices_offset = layout.primitive_base_indices_offset;
 
    radv_update_memory_cp(cmd_buffer, vk_acceleration_structure_get_va(dst) + base, (const char *)&header + base,
                          sizeof(header) - base);
 
-   if (device->rra_trace.accel_structs) {
+   if (layout.geometry_info_offset != RADV_OFFSET_UNUSED) {
       uint64_t geometry_infos_size = state->build_info->geometryCount * sizeof(struct radv_accel_struct_geometry_info);
 
       struct radv_accel_struct_geometry_info *geometry_infos = malloc(geometry_infos_size);
@@ -695,27 +562,6 @@ radv_init_header(VkCommandBuffer commandBuffer, const struct vk_acceleration_str
 
       free(geometry_infos);
    }
-
-   VkGeometryTypeKHR geometry_type = vk_get_as_geometry_type(state->build_info);
-   if (device->vk.enabled_features.rayTracingPositionFetch && geometry_type == VK_GEOMETRY_TYPE_TRIANGLES_KHR) {
-      uint32_t base_indices_size = sizeof(uint32_t) * state->build_info->geometryCount;
-      uint32_t *base_indices = malloc(base_indices_size);
-      if (!base_indices) {
-         vk_command_buffer_set_error(&cmd_buffer->vk, VK_ERROR_OUT_OF_HOST_MEMORY);
-         return;
-      }
-
-      uint32_t base_index = 0;
-      for (uint32_t i = 0; i < state->build_info->geometryCount; i++) {
-         base_indices[i] = base_index;
-         base_index += state->build_range_infos[i].primitiveCount;
-      }
-
-      radv_CmdUpdateBuffer(commandBuffer, vk_buffer_to_handle(dst->buffer),
-                           dst->offset + layout.primitive_base_indices_offset, base_indices_size, base_indices);
-
-      free(base_indices);
-   }
 }
 
 static void
@@ -734,7 +580,8 @@ radv_init_update_scratch(VkCommandBuffer commandBuffer, const struct vk_accelera
    radv_fill_memory(cmd_buffer, scratch + layout.internal_ready_count_offset,
                     layout.size - layout.internal_ready_count_offset, 0x0, RADV_COPY_FLAGS_DEVICE_LOCAL);
 
-   if (radv_use_bvh8(pdev)) {
+   /* geometryCount == 1 passes the data as push constant. */
+   if (radv_use_bvh8(pdev) && !(state->config.update_key[0] & RADV_BUILD_FLAG_UPDATE_SINGLE_GEOMETRY)) {
       uint32_t data_size = sizeof(struct vk_bvh_geometry_data) * state->build_info->geometryCount;
       struct vk_bvh_geometry_data *data = malloc(data_size);
       if (!data) {
@@ -762,24 +609,23 @@ radv_init_update_scratch(VkCommandBuffer commandBuffer, const struct vk_accelera
 }
 
 static void
-radv_update_bind_pipeline(VkCommandBuffer commandBuffer, const struct vk_acceleration_structure_build_state *state)
+radv_update_bind_pipeline(VkCommandBuffer commandBuffer, const struct vk_acceleration_structure_build_state *state,
+                          bool flushed_cp_after_init_update_scratch, bool flushed_compute_after_init_update_scratch)
 {
    VK_FROM_HANDLE(radv_cmd_buffer, cmd_buffer, commandBuffer);
    struct radv_device *device = radv_cmd_buffer_device(cmd_buffer);
    const struct radv_physical_device *pdev = radv_device_physical(device);
 
    /* Wait for update scratch initialization to finish.. */
-   cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_CS_PARTIAL_FLUSH |
-                                   radv_src_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                         VK_ACCESS_2_SHADER_WRITE_BIT, 0, NULL, NULL) |
-                                   radv_dst_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                         VK_ACCESS_2_SHADER_READ_BIT, 0, NULL, NULL);
+   if (!flushed_compute_after_init_update_scratch)
+      vk_barrier_compute_w_to_compute_r(commandBuffer);
 
-   if (radv_device_physical(device)->info.cp_sdma_ge_use_system_memory_scope)
-      cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_INV_L2;
+   if (!flushed_cp_after_init_update_scratch) {
+      if (radv_device_physical(device)->info.cp_sdma_ge_use_system_memory_scope)
+         cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_INV_L2;
+   }
 
-   bool in_place = state->config.update_key[0] & RADV_BUILD_FLAG_UPDATE_IN_PLACE;
-   uint32_t flags = in_place ? RADV_BUILD_FLAG_UPDATE_IN_PLACE : 0;
+   uint32_t flags = state->config.update_key[0];
 
    if (radv_use_bvh8(pdev)) {
       radv_bvh_build_bind_pipeline(commandBuffer, RADV_META_OBJECT_KEY_BVH_UPDATE, update_gfx12_spv,
@@ -880,6 +726,12 @@ radv_update_as_gfx12(VkCommandBuffer commandBuffer, const struct vk_acceleration
       .leaf_node_count = state->leaf_node_count,
    };
 
+   if (state->config.update_key[0] & RADV_BUILD_FLAG_UPDATE_SINGLE_GEOMETRY) {
+      const VkAccelerationStructureGeometryKHR *geom =
+         state->build_info->pGeometries ? &state->build_info->pGeometries[0] : state->build_info->ppGeometries[0];
+      update_consts.geom_data0 = vk_fill_geometry_data(state->build_info->type, 0, 0, geom, state->build_range_infos);
+   }
+
    radv_bvh_build_set_args(commandBuffer, &update_consts, sizeof(update_consts));
 
    struct radv_dispatch_info dispatch = {
@@ -1090,11 +942,7 @@ radv_CmdCopyMemoryToAccelerationStructureKHR(VkCommandBuffer commandBuffer,
 
    if (radv_use_bvh8(pdev)) {
       /* Wait for the main copy dispatch to finish. */
-      cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_CS_PARTIAL_FLUSH |
-                                      radv_src_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                            VK_ACCESS_2_SHADER_WRITE_BIT, 0, NULL, NULL) |
-                                      radv_dst_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                            VK_ACCESS_2_SHADER_READ_BIT, 0, NULL, NULL);
+      vk_barrier_compute_w_to_compute_r(commandBuffer);
 
       radv_bvh_build_bind_pipeline(commandBuffer, RADV_META_OBJECT_KEY_BVH_COPY_BLAS_ADDRS_GFX12,
                                    copy_blas_addrs_gfx12_spv, sizeof(copy_blas_addrs_gfx12_spv),
@@ -1137,11 +985,7 @@ radv_CmdCopyAccelerationStructureToMemoryKHR(VkCommandBuffer commandBuffer,
 
    if (radv_use_bvh8(pdev)) {
       /* Wait for the main copy dispatch to finish. */
-      cmd_buffer->state.flush_bits |= RADV_CMD_FLAG_CS_PARTIAL_FLUSH |
-                                      radv_src_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                            VK_ACCESS_2_SHADER_WRITE_BIT, 0, NULL, NULL) |
-                                      radv_dst_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT,
-                                                            VK_ACCESS_2_SHADER_READ_BIT, 0, NULL, NULL);
+      vk_barrier_compute_w_to_compute_r(commandBuffer);
 
       radv_bvh_build_bind_pipeline(commandBuffer, RADV_META_OBJECT_KEY_BVH_COPY_BLAS_ADDRS_GFX12,
                                    copy_blas_addrs_gfx12_spv, sizeof(copy_blas_addrs_gfx12_spv),
