@@ -19,13 +19,14 @@ radv_create_shadow_regs_preamble(struct radv_device *device, struct radv_queue_s
    struct radeon_winsys *ws = device->ws;
    const struct radeon_info *gpu_info = &pdev->info;
    struct ac_pm4_state *pm4 = NULL;
+   struct radv_cmd_stream *cs;
    VkResult result;
 
-   struct radeon_cmdbuf *cs = ws->cs_create(ws, AMD_IP_GFX, false);
-   if (!cs)
-      return VK_ERROR_OUT_OF_DEVICE_MEMORY;
+   result = radv_create_cmd_stream(device, AMD_IP_GFX, false, &cs);
+   if (result != VK_SUCCESS)
+      return result;
 
-   radeon_check_space(ws, cs, 256);
+   radeon_check_space(ws, cs->b, 256);
 
    /* allocate memory for queue_state->shadowed_regs where register states are saved */
    result = radv_bo_create(device, NULL, SI_SHADOWED_REG_BUFFER_SIZE, 4096, RADEON_DOMAIN_VRAM,
@@ -43,10 +44,10 @@ radv_create_shadow_regs_preamble(struct radv_device *device, struct radv_queue_s
    radeon_emit_array(pm4->pm4, pm4->ndw);
    radeon_end();
 
-   ws->cs_pad(cs, 0);
+   ws->cs_pad(cs->b, 0);
 
    result = radv_bo_create(
-      device, NULL, cs->cdw * 4, 4096, ws->cs_domain(ws),
+      device, NULL, cs->b->cdw * 4, 4096, ws->cs_domain(ws),
       RADEON_FLAG_CPU_ACCESS | RADEON_FLAG_NO_INTERPROCESS_SHARING | RADEON_FLAG_READ_ONLY | RADEON_FLAG_GTT_WC,
       RADV_BO_PRIORITY_CS, 0, true, &queue_state->shadow_regs_ib);
    if (result != VK_SUCCESS)
@@ -60,13 +61,13 @@ radv_create_shadow_regs_preamble(struct radv_device *device, struct radv_queue_s
       result = VK_ERROR_MEMORY_MAP_FAILED;
       goto fail_map;
    }
-   memcpy(map, cs->buf, cs->cdw * 4);
-   queue_state->shadow_regs_ib_size_dw = cs->cdw;
+   memcpy(map, cs->b->buf, cs->b->cdw * 4);
+   queue_state->shadow_regs_ib_size_dw = cs->b->cdw;
 
    ws->buffer_unmap(ws, queue_state->shadow_regs_ib, false);
 
    ac_pm4_free_state(pm4);
-   ws->cs_destroy(cs);
+   radv_destroy_cmd_stream(device, cs);
    return VK_SUCCESS;
 fail_map:
    radv_bo_destroy(device, NULL, queue_state->shadow_regs_ib);
@@ -77,7 +78,7 @@ fail_create:
    radv_bo_destroy(device, NULL, queue_state->shadowed_regs);
    queue_state->shadowed_regs = NULL;
 fail:
-   ws->cs_destroy(cs);
+   radv_destroy_cmd_stream(device, cs);
    return result;
 }
 
@@ -92,15 +93,15 @@ radv_destroy_shadow_regs_preamble(struct radv_device *device, struct radv_queue_
 }
 
 void
-radv_emit_shadow_regs_preamble(struct radeon_cmdbuf *cs, const struct radv_device *device,
+radv_emit_shadow_regs_preamble(struct radv_cmd_stream *cs, const struct radv_device *device,
                                struct radv_queue_state *queue_state)
 {
    struct radeon_winsys *ws = device->ws;
 
-   ws->cs_execute_ib(cs, queue_state->shadow_regs_ib, 0, queue_state->shadow_regs_ib_size_dw & 0xffff, false);
+   ws->cs_execute_ib(cs->b, queue_state->shadow_regs_ib, 0, queue_state->shadow_regs_ib_size_dw & 0xffff, false);
 
-   radv_cs_add_buffer(device->ws, cs, queue_state->shadowed_regs);
-   radv_cs_add_buffer(device->ws, cs, queue_state->shadow_regs_ib);
+   radv_cs_add_buffer(device->ws, cs->b, queue_state->shadowed_regs);
+   radv_cs_add_buffer(device->ws, cs->b, queue_state->shadow_regs_ib);
 }
 
 /* radv_init_shadowed_regs_buffer_state() will be called once from radv_queue_init(). This
@@ -111,14 +112,14 @@ radv_init_shadowed_regs_buffer_state(const struct radv_device *device, struct ra
    const struct radv_physical_device *pdev = radv_device_physical(device);
    const struct radeon_info *gpu_info = &pdev->info;
    struct radeon_winsys *ws = device->ws;
-   struct radeon_cmdbuf *cs;
+   struct radv_cmd_stream *cs;
    VkResult result;
 
-   cs = ws->cs_create(ws, AMD_IP_GFX, false);
-   if (!cs)
-      return VK_ERROR_OUT_OF_DEVICE_MEMORY;
+   result = radv_create_cmd_stream(device, AMD_IP_GFX, false, &cs);
+   if (result != VK_SUCCESS)
+      return result;
 
-   radeon_check_space(ws, cs, 768);
+   radeon_check_space(ws, cs->b, 768);
 
    radv_emit_shadow_regs_preamble(cs, device, &queue->state);
 
@@ -136,13 +137,13 @@ radv_init_shadowed_regs_buffer_state(const struct radv_device *device, struct ra
       ac_pm4_free_state(pm4);
    }
 
-   result = ws->cs_finalize(cs);
+   result = radv_finalize_cmd_stream(device, cs);
    if (result == VK_SUCCESS) {
-      if (!radv_queue_internal_submit(queue, cs))
+      if (!radv_queue_internal_submit(queue, cs->b))
          result = VK_ERROR_UNKNOWN;
    }
 
 fail:
-   ws->cs_destroy(cs);
+   radv_destroy_cmd_stream(device, cs);
    return result;
 }
