@@ -13,9 +13,9 @@
 #include "radv_cp_reg_shadowing.h"
 #include "radv_cs.h"
 #include "radv_debug.h"
+#include "radv_debug_nir.h"
 #include "radv_device_memory.h"
 #include "radv_image.h"
-#include "radv_printf.h"
 #include "radv_rmv.h"
 #include "vk_semaphore.h"
 #include "vk_sync.h"
@@ -31,19 +31,7 @@ radv_get_queue_global_priority(const VkDeviceQueueGlobalPriorityCreateInfo *pObj
    if (!pObj)
       return RADEON_CTX_PRIORITY_MEDIUM;
 
-   switch (pObj->globalPriority) {
-   case VK_QUEUE_GLOBAL_PRIORITY_REALTIME:
-      return RADEON_CTX_PRIORITY_REALTIME;
-   case VK_QUEUE_GLOBAL_PRIORITY_HIGH:
-      return RADEON_CTX_PRIORITY_HIGH;
-   case VK_QUEUE_GLOBAL_PRIORITY_MEDIUM:
-      return RADEON_CTX_PRIORITY_MEDIUM;
-   case VK_QUEUE_GLOBAL_PRIORITY_LOW:
-      return RADEON_CTX_PRIORITY_LOW;
-   default:
-      unreachable("Illegal global priority value");
-      return RADEON_CTX_PRIORITY_INVALID;
-   }
+   return vk_to_radeon_priority(pObj->globalPriority);
 }
 
 static VkResult
@@ -360,7 +348,7 @@ radv_fill_shader_rings(struct radv_device *device, uint32_t *desc, struct radeon
 }
 
 static void
-radv_emit_gs_ring_sizes(struct radv_device *device, struct radeon_cmdbuf *cs, struct radeon_winsys_bo *esgs_ring_bo,
+radv_emit_gs_ring_sizes(struct radv_device *device, struct radv_cmd_stream *cs, struct radeon_winsys_bo *esgs_ring_bo,
                         uint32_t esgs_ring_size, struct radeon_winsys_bo *gsvs_ring_bo, uint32_t gsvs_ring_size)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
@@ -369,10 +357,10 @@ radv_emit_gs_ring_sizes(struct radv_device *device, struct radeon_cmdbuf *cs, st
       return;
 
    if (esgs_ring_bo)
-      radv_cs_add_buffer(device->ws, cs, esgs_ring_bo);
+      radv_cs_add_buffer(device->ws, cs->b, esgs_ring_bo);
 
    if (gsvs_ring_bo)
-      radv_cs_add_buffer(device->ws, cs, gsvs_ring_bo);
+      radv_cs_add_buffer(device->ws, cs->b, gsvs_ring_bo);
 
    radeon_begin(cs);
 
@@ -390,7 +378,8 @@ radv_emit_gs_ring_sizes(struct radv_device *device, struct radeon_cmdbuf *cs, st
 }
 
 static void
-radv_emit_tess_factor_ring(struct radv_device *device, struct radeon_cmdbuf *cs, struct radeon_winsys_bo *tess_rings_bo)
+radv_emit_tess_factor_ring(struct radv_device *device, struct radv_cmd_stream *cs,
+                           struct radeon_winsys_bo *tess_rings_bo)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
    uint64_t tf_va;
@@ -401,7 +390,7 @@ radv_emit_tess_factor_ring(struct radv_device *device, struct radeon_cmdbuf *cs,
    tf_ring_size = pdev->info.tess_factor_ring_size / 4;
    tf_va = radv_buffer_get_va(tess_rings_bo) + pdev->info.tess_offchip_ring_size;
 
-   radv_cs_add_buffer(device->ws, cs, tess_rings_bo);
+   radv_cs_add_buffer(device->ws, cs->b, tess_rings_bo);
 
    radeon_begin(cs);
 
@@ -465,7 +454,7 @@ radv_initialise_task_control_buffer(struct radv_device *device, struct radeon_wi
 }
 
 static void
-radv_emit_task_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struct radeon_winsys_bo *task_rings_bo,
+radv_emit_task_rings(struct radv_device *device, struct radv_cmd_stream *cs, struct radeon_winsys_bo *task_rings_bo,
                      bool compute)
 {
    if (!task_rings_bo)
@@ -473,7 +462,7 @@ radv_emit_task_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struc
 
    const uint64_t task_ctrlbuf_va = radv_buffer_get_va(task_rings_bo);
    assert(util_is_aligned(task_ctrlbuf_va, 256));
-   radv_cs_add_buffer(device->ws, cs, task_rings_bo);
+   radv_cs_add_buffer(device->ws, cs->b, task_rings_bo);
 
    radeon_begin(cs);
 
@@ -488,8 +477,8 @@ radv_emit_task_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struc
 }
 
 static void
-radv_emit_graphics_scratch(struct radv_device *device, struct radeon_cmdbuf *cs, uint32_t size_per_wave, uint32_t waves,
-                           struct radeon_winsys_bo *scratch_bo)
+radv_emit_graphics_scratch(struct radv_device *device, struct radv_cmd_stream *cs, uint32_t size_per_wave,
+                           uint32_t waves, struct radeon_winsys_bo *scratch_bo)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
    const struct radeon_info *gpu_info = &pdev->info;
@@ -500,7 +489,7 @@ radv_emit_graphics_scratch(struct radv_device *device, struct radeon_cmdbuf *cs,
 
    ac_get_scratch_tmpring_size(gpu_info, waves, size_per_wave, &tmpring_size);
 
-   radv_cs_add_buffer(device->ws, cs, scratch_bo);
+   radv_cs_add_buffer(device->ws, cs->b, scratch_bo);
 
    radeon_begin(cs);
 
@@ -519,8 +508,8 @@ radv_emit_graphics_scratch(struct radv_device *device, struct radeon_cmdbuf *cs,
 }
 
 static void
-radv_emit_compute_scratch(struct radv_device *device, struct radeon_cmdbuf *cs, uint32_t size_per_wave, uint32_t waves,
-                          struct radeon_winsys_bo *compute_scratch_bo)
+radv_emit_compute_scratch(struct radv_device *device, struct radv_cmd_stream *cs, uint32_t size_per_wave,
+                          uint32_t waves, struct radeon_winsys_bo *compute_scratch_bo)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
    const struct radeon_info *gpu_info = &pdev->info;
@@ -541,7 +530,7 @@ radv_emit_compute_scratch(struct radv_device *device, struct radeon_cmdbuf *cs,
 
    ac_get_scratch_tmpring_size(gpu_info, waves, size_per_wave, &tmpring_size);
 
-   radv_cs_add_buffer(device->ws, cs, compute_scratch_bo);
+   radv_cs_add_buffer(device->ws, cs->b, compute_scratch_bo);
 
    radeon_begin(cs);
 
@@ -563,14 +552,14 @@ radv_emit_compute_scratch(struct radv_device *device, struct radeon_cmdbuf *cs,
 }
 
 static void
-radv_emit_compute_shader_pointers(struct radv_device *device, struct radeon_cmdbuf *cs,
+radv_emit_compute_shader_pointers(struct radv_device *device, struct radv_cmd_stream *cs,
                                   struct radeon_winsys_bo *descriptor_bo)
 {
    if (!descriptor_bo)
       return;
 
    uint64_t va = radv_buffer_get_va(descriptor_bo);
-   radv_cs_add_buffer(device->ws, cs, descriptor_bo);
+   radv_cs_add_buffer(device->ws, cs->b, descriptor_bo);
 
    /* Compute shader user data 0-1 have the scratch pointer (unlike GFX shaders),
     * so emit the descriptor pointer to user data 2-3 instead (task_ring_offsets arg).
@@ -581,7 +570,7 @@ radv_emit_compute_shader_pointers(struct radv_device *device, struct radeon_cmdb
 }
 
 static void
-radv_emit_graphics_shader_pointers(struct radv_device *device, struct radeon_cmdbuf *cs,
+radv_emit_graphics_shader_pointers(struct radv_device *device, struct radv_cmd_stream *cs,
                                    struct radeon_winsys_bo *descriptor_bo)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
@@ -592,7 +581,7 @@ radv_emit_graphics_shader_pointers(struct radv_device *device, struct radeon_cmd
 
    va = radv_buffer_get_va(descriptor_bo);
 
-   radv_cs_add_buffer(device->ws, cs, descriptor_bo);
+   radv_cs_add_buffer(device->ws, cs->b, descriptor_bo);
 
    radeon_begin(cs);
 
@@ -638,7 +627,7 @@ radv_emit_graphics_shader_pointers(struct radv_device *device, struct radeon_cmd
 }
 
 static void
-radv_emit_ge_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struct radeon_winsys_bo *ge_rings_bo)
+radv_emit_ge_rings(struct radv_device *device, struct radv_cmd_stream *cs, struct radeon_winsys_bo *ge_rings_bo)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
    uint64_t va;
@@ -646,37 +635,20 @@ radv_emit_ge_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struct
    if (!ge_rings_bo)
       return;
 
-   assert(pdev->info.gfx_level >= GFX11);
-
    va = radv_buffer_get_va(ge_rings_bo);
    assert((va >> 32) == pdev->info.address32_hi);
 
-   radv_cs_add_buffer(device->ws, cs, ge_rings_bo);
-
-   radeon_begin(cs);
+   radv_cs_add_buffer(device->ws, cs->b, ge_rings_bo);
 
    /* We must wait for idle using an EOP event before changing the attribute ring registers. Use the
     * bottom-of-pipe EOP event, but increment the PWS counter instead of writing memory.
     */
-   radeon_emit(PKT3(PKT3_RELEASE_MEM, 6, 0));
-   radeon_emit(S_490_EVENT_TYPE(V_028A90_BOTTOM_OF_PIPE_TS) | S_490_EVENT_INDEX(5) | S_490_PWS_ENABLE(1));
-   radeon_emit(0); /* DST_SEL, INT_SEL, DATA_SEL */
-   radeon_emit(0); /* ADDRESS_LO */
-   radeon_emit(0); /* ADDRESS_HI */
-   radeon_emit(0); /* DATA_LO */
-   radeon_emit(0); /* DATA_HI */
-   radeon_emit(0); /* INT_CTXID */
+   ac_emit_cp_release_mem_pws(cs->b, pdev->info.gfx_level, AMD_IP_GFX, V_028A90_BOTTOM_OF_PIPE_TS, 0);
 
    /* Wait for the PWS counter. */
-   radeon_emit(PKT3(PKT3_ACQUIRE_MEM, 6, 0));
-   radeon_emit(S_580_PWS_STAGE_SEL(V_580_CP_ME) | S_580_PWS_COUNTER_SEL(V_580_TS_SELECT) | S_580_PWS_ENA2(1) |
-               S_580_PWS_COUNT(0));
-   radeon_emit(0xffffffff); /* GCR_SIZE */
-   radeon_emit(0x01ffffff); /* GCR_SIZE_HI */
-   radeon_emit(0);          /* GCR_BASE_LO */
-   radeon_emit(0);          /* GCR_BASE_HI */
-   radeon_emit(S_585_PWS_ENA(1));
-   radeon_emit(0); /* GCR_CNTL */
+   ac_emit_cp_acquire_mem_pws(cs->b, pdev->info.gfx_level, AMD_IP_GFX, V_028A90_BOTTOM_OF_PIPE_TS, V_580_CP_ME, 0, 0);
+
+   radeon_begin(cs);
 
    /* The PS will read inputs from this address. */
    radeon_set_uconfig_reg_seq(R_031110_SPI_GS_THROTTLE_CNTL1, 4);
@@ -705,7 +677,7 @@ radv_emit_ge_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struct
          /* Mitigate the HiZ GPU hang by increasing a timeout when BOTTOM_OF_PIPE_TS is used as the
           * workaround. This must be emitted when the gfx queue is idle.
           */
-         const uint32_t timeout = pdev->use_gfx12_hiz_his_event_wa ? 0xfff : 0;
+         const uint32_t timeout = pdev->gfx12_hiz_wa == RADV_GFX12_HIZ_WA_PARTIAL ? 0xfff : 0;
 
          radeon_emit(PKT3(PKT3_UPDATE_DB_SUMMARIZER_TIMEOUT, 0, 0));
          radeon_emit(S_EF1_SUMM_CNTL_EVICT_TIMEOUT(timeout));
@@ -716,7 +688,7 @@ radv_emit_ge_rings(struct radv_device *device, struct radeon_cmdbuf *cs, struct
 }
 
 static void
-radv_emit_compute(struct radv_device *device, struct radeon_cmdbuf *cs, bool is_compute_queue)
+radv_emit_compute(struct radv_device *device, struct radv_cmd_stream *cs, bool is_compute_queue)
 {
    const struct radv_physical_device *pdev = radv_device_physical(device);
    const uint64_t border_color_va = device->border_color_data.bo ? radv_buffer_get_va(device->border_color_data.bo) : 0;
@@ -768,7 +740,7 @@ radv_pack_float_12p4(float x)
 }
 
 void
-radv_emit_graphics(struct radv_device *device, struct radeon_cmdbuf *cs)
+radv_emit_graphics(struct radv_device *device, struct radv_cmd_stream *cs)
 {
    struct radv_physical_device *pdev = radv_device_physical(device);
    const uint64_t border_color_va = device->border_color_data.bo ? radv_buffer_get_va(device->border_color_data.bo) : 0;
@@ -941,14 +913,14 @@ radv_emit_graphics(struct radv_device *device, struct radeon_cmdbuf *cs)
 }
 
 static void
-radv_init_graphics_state(struct radeon_cmdbuf *cs, struct radv_device *device)
+radv_init_graphics_state(struct radv_cmd_stream *cs, struct radv_device *device)
 {
    if (device->gfx_init) {
       struct radeon_winsys *ws = device->ws;
 
-      ws->cs_execute_ib(cs, device->gfx_init, 0, device->gfx_init_size_dw & 0xffff, false);
+      ws->cs_execute_ib(cs->b, device->gfx_init, 0, device->gfx_init_size_dw & 0xffff, false);
 
-      radv_cs_add_buffer(device->ws, cs, device->gfx_init);
+      radv_cs_add_buffer(device->ws, cs->b, device->gfx_init);
    } else {
       radv_emit_graphics(device, cs);
    }
@@ -971,7 +943,7 @@ radv_update_preamble_cs(struct radv_queue_state *queue, struct radv_device *devi
    struct radeon_winsys_bo *ge_rings_bo = queue->ge_rings_bo;
    struct radeon_winsys_bo *gds_bo = queue->gds_bo;
    struct radeon_winsys_bo *gds_oa_bo = queue->gds_oa_bo;
-   struct radeon_cmdbuf *dest_cs[3] = {0};
+   struct radv_cmd_stream *dest_cs[3] = {0};
    const uint32_t ring_bo_flags = RADEON_FLAG_NO_CPU_ACCESS | RADEON_FLAG_NO_INTERPROCESS_SHARING;
    VkResult result = VK_SUCCESS;
 
@@ -1132,20 +1104,21 @@ radv_update_preamble_cs(struct radv_queue_state *queue, struct radv_device *devi
       ws->buffer_unmap(ws, descriptor_bo, false);
    }
 
+   const enum amd_ip_type hw_ip = radv_queue_family_to_ring(pdev, queue->qf);
+
    for (int i = 0; i < 3; ++i) {
       enum rgp_flush_bits sqtt_flush_bits = 0;
-      struct radeon_cmdbuf *cs = NULL;
-      cs = ws->cs_create(ws, radv_queue_family_to_ring(pdev, queue->qf), false);
-      if (!cs) {
-         result = VK_ERROR_OUT_OF_DEVICE_MEMORY;
-         goto fail;
-      }
+      struct radv_cmd_stream *cs = NULL;
 
-      radeon_check_space(ws, cs, 512);
+      result = radv_create_cmd_stream(device, hw_ip, false, &cs);
+      if (result != VK_SUCCESS)
+         goto fail;
+
+      radeon_check_space(ws, cs->b, 512);
       dest_cs[i] = cs;
 
       if (scratch_bo)
-         radv_cs_add_buffer(ws, cs, scratch_bo);
+         radv_cs_add_buffer(ws, cs->b, scratch_bo);
 
       /* Emit initial configuration. */
       switch (queue->qf) {
@@ -1202,22 +1175,22 @@ radv_update_preamble_cs(struct radv_queue_state *queue, struct radv_device *devi
                flush_bits |= RADV_CMD_FLAG_PS_PARTIAL_FLUSH;
          }
 
-         radv_cs_emit_cache_flush(ws, cs, gfx_level, NULL, 0, queue->qf, flush_bits, &sqtt_flush_bits, 0);
+         radv_cs_emit_cache_flush(ws, cs, gfx_level, NULL, 0, flush_bits, &sqtt_flush_bits, 0);
       }
 
-      result = ws->cs_finalize(cs);
+      result = radv_finalize_cmd_stream(device, cs);
       if (result != VK_SUCCESS)
          goto fail;
    }
 
    if (queue->initial_full_flush_preamble_cs)
-      ws->cs_destroy(queue->initial_full_flush_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->initial_full_flush_preamble_cs);
 
    if (queue->initial_preamble_cs)
-      ws->cs_destroy(queue->initial_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->initial_preamble_cs);
 
    if (queue->continue_preamble_cs)
-      ws->cs_destroy(queue->continue_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->continue_preamble_cs);
 
    queue->initial_full_flush_preamble_cs = dest_cs[0];
    queue->initial_preamble_cs = dest_cs[1];
@@ -1272,7 +1245,7 @@ radv_update_preamble_cs(struct radv_queue_state *queue, struct radv_device *devi
 fail:
    for (int i = 0; i < ARRAY_SIZE(dest_cs); ++i)
       if (dest_cs[i])
-         ws->cs_destroy(dest_cs[i]);
+         radv_destroy_cmd_stream(device, dest_cs[i]);
    if (descriptor_bo && descriptor_bo != queue->descriptor_bo)
       radv_bo_destroy(device, NULL, descriptor_bo);
    if (scratch_bo && scratch_bo != queue->scratch_bo)
@@ -1394,12 +1367,14 @@ radv_create_flush_postamble(struct radv_queue *queue)
    const struct radv_physical_device *pdev = radv_device_physical(device);
    const enum amd_ip_type ip = radv_queue_family_to_ring(pdev, queue->state.qf);
    struct radeon_winsys *ws = device->ws;
+   struct radv_cmd_stream *cs;
+   VkResult result;
 
-   struct radeon_cmdbuf *cs = ws->cs_create(ws, ip, false);
-   if (!cs)
-      return VK_ERROR_OUT_OF_DEVICE_MEMORY;
+   result = radv_create_cmd_stream(device, ip, false, &cs);
+   if (result != VK_SUCCESS)
+      return result;
 
-   radeon_check_space(ws, cs, 256);
+   radeon_check_space(ws, cs->b, 256);
 
    const enum amd_gfx_level gfx_level = pdev->info.gfx_level;
    enum radv_cmd_flush_bits flush_bits = 0;
@@ -1416,12 +1391,12 @@ radv_create_flush_postamble(struct radv_queue *queue)
    }
 
    enum rgp_flush_bits sqtt_flush_bits = 0;
-   radv_cs_emit_cache_flush(ws, cs, gfx_level, NULL, 0, queue->state.qf, flush_bits, &sqtt_flush_bits, 0);
+   radv_cs_emit_cache_flush(ws, cs, gfx_level, NULL, 0, flush_bits, &sqtt_flush_bits, 0);
 
-   VkResult r = ws->cs_finalize(cs);
-   if (r != VK_SUCCESS) {
-      ws->cs_destroy(cs);
-      return r;
+   result = radv_finalize_cmd_stream(device, cs);
+   if (result != VK_SUCCESS) {
+      radv_destroy_cmd_stream(device, cs);
+      return result;
    }
 
    queue->state.flush_postamble_cs = cs;
@@ -1433,13 +1408,13 @@ radv_create_gang_wait_preambles_postambles(struct radv_queue *queue)
 {
    struct radv_device *device = radv_queue_device(queue);
    const struct radv_physical_device *pdev = radv_device_physical(device);
+   const enum amd_ip_type ip = radv_queue_family_to_ring(pdev, queue->state.qf);
 
    if (queue->gang_sem_bo)
       return VK_SUCCESS;
 
    VkResult r = VK_SUCCESS;
    struct radeon_winsys *ws = device->ws;
-   const enum amd_ip_type leader_ip = radv_queue_family_to_ring(pdev, queue->state.qf);
    struct radeon_winsys_bo *gang_sem_bo = NULL;
 
    /* Gang semaphores BO.
@@ -1452,25 +1427,34 @@ radv_create_gang_wait_preambles_postambles(struct radv_queue *queue)
    if (r != VK_SUCCESS)
       return r;
 
-   struct radeon_cmdbuf *leader_pre_cs = ws->cs_create(ws, leader_ip, false);
-   struct radeon_cmdbuf *leader_post_cs = ws->cs_create(ws, leader_ip, false);
-   struct radeon_cmdbuf *ace_pre_cs = ws->cs_create(ws, AMD_IP_COMPUTE, false);
-   struct radeon_cmdbuf *ace_post_cs = ws->cs_create(ws, AMD_IP_COMPUTE, false);
+   struct radv_cmd_stream *leader_pre_cs = NULL, *leader_post_cs = NULL;
+   struct radv_cmd_stream *ace_pre_cs = NULL, *ace_post_cs = NULL;
 
-   if (!leader_pre_cs || !leader_post_cs || !ace_pre_cs || !ace_post_cs) {
-      r = VK_ERROR_OUT_OF_DEVICE_MEMORY;
+   r = radv_create_cmd_stream(device, ip, false, &leader_pre_cs);
+   if (r != VK_SUCCESS)
       goto fail;
-   }
 
-   radeon_check_space(ws, leader_pre_cs, 256);
-   radeon_check_space(ws, leader_post_cs, 256);
-   radeon_check_space(ws, ace_pre_cs, 256);
-   radeon_check_space(ws, ace_post_cs, 256);
+   radv_create_cmd_stream(device, ip, false, &leader_post_cs);
+   if (r != VK_SUCCESS)
+      goto fail;
 
-   radv_cs_add_buffer(ws, leader_pre_cs, gang_sem_bo);
-   radv_cs_add_buffer(ws, leader_post_cs, gang_sem_bo);
-   radv_cs_add_buffer(ws, ace_pre_cs, gang_sem_bo);
-   radv_cs_add_buffer(ws, ace_post_cs, gang_sem_bo);
+   radv_create_cmd_stream(device, AMD_IP_COMPUTE, false, &ace_pre_cs);
+   if (r != VK_SUCCESS)
+      goto fail;
+
+   radv_create_cmd_stream(device, AMD_IP_COMPUTE, false, &ace_post_cs);
+   if (r != VK_SUCCESS)
+      goto fail;
+
+   radeon_check_space(ws, leader_pre_cs->b, 256);
+   radeon_check_space(ws, leader_post_cs->b, 256);
+   radeon_check_space(ws, ace_pre_cs->b, 256);
+   radeon_check_space(ws, ace_post_cs->b, 256);
+
+   radv_cs_add_buffer(ws, leader_pre_cs->b, gang_sem_bo);
+   radv_cs_add_buffer(ws, leader_post_cs->b, gang_sem_bo);
+   radv_cs_add_buffer(ws, ace_pre_cs->b, gang_sem_bo);
+   radv_cs_add_buffer(ws, ace_post_cs->b, gang_sem_bo);
 
    const uint64_t ace_wait_va = radv_buffer_get_va(gang_sem_bo);
    const uint64_t leader_wait_va = ace_wait_va + 4;
@@ -1483,31 +1467,30 @@ radv_create_gang_wait_preambles_postambles(struct radv_queue *queue)
     * in a multi-process environment, because task shader dispatches are not
     * meant to be executed on multiple compute engines at the same time.
     */
-   radv_cp_wait_mem(ace_pre_cs, RADV_QUEUE_COMPUTE, WAIT_REG_MEM_GREATER_OR_EQUAL, ace_wait_va, 1, 0xffffffff);
-   radv_cs_write_data(device, ace_pre_cs, RADV_QUEUE_COMPUTE, V_370_ME, ace_wait_va, 1, &zero, false);
-   radv_cs_write_data(device, leader_pre_cs, queue->state.qf, V_370_ME, ace_wait_va, 1, &one, false);
-
+   radv_cp_wait_mem(ace_pre_cs, WAIT_REG_MEM_GREATER_OR_EQUAL, ace_wait_va, 1, 0xffffffff);
+   radv_cs_write_data(device, ace_pre_cs, V_370_ME, ace_wait_va, 1, &zero, false);
+   radv_cs_write_data(device, leader_pre_cs, V_370_ME, ace_wait_va, 1, &one, false);
    /* Create postambles for gang submission.
     * This ensures that the gang leader waits for the whole gang,
     * which is necessary because the kernel signals the userspace fence
     * as soon as the gang leader is done, which may lead to bugs because the
     * same command buffers could be submitted again while still being executed.
     */
-   radv_cp_wait_mem(leader_post_cs, queue->state.qf, WAIT_REG_MEM_GREATER_OR_EQUAL, leader_wait_va, 1, 0xffffffff);
-   radv_cs_write_data(device, leader_post_cs, queue->state.qf, V_370_ME, leader_wait_va, 1, &zero, false);
-   radv_cs_emit_write_event_eop(ace_post_cs, pdev->info.gfx_level, RADV_QUEUE_COMPUTE, V_028A90_BOTTOM_OF_PIPE_TS, 0,
-                                EOP_DST_SEL_MEM, EOP_DATA_SEL_VALUE_32BIT, leader_wait_va, 1, 0);
+   radv_cp_wait_mem(leader_post_cs, WAIT_REG_MEM_GREATER_OR_EQUAL, leader_wait_va, 1, 0xffffffff);
+   radv_cs_write_data(device, leader_post_cs, V_370_ME, leader_wait_va, 1, &zero, false);
+   radv_cs_emit_write_event_eop(ace_post_cs, pdev->info.gfx_level, V_028A90_BOTTOM_OF_PIPE_TS, 0, EOP_DST_SEL_MEM,
+                                EOP_DATA_SEL_VALUE_32BIT, leader_wait_va, 1, 0);
 
-   r = ws->cs_finalize(leader_pre_cs);
+   r = radv_finalize_cmd_stream(device, leader_pre_cs);
    if (r != VK_SUCCESS)
       goto fail;
-   r = ws->cs_finalize(leader_post_cs);
+   r = radv_finalize_cmd_stream(device, leader_post_cs);
    if (r != VK_SUCCESS)
       goto fail;
-   r = ws->cs_finalize(ace_pre_cs);
+   r = radv_finalize_cmd_stream(device, ace_pre_cs);
    if (r != VK_SUCCESS)
       goto fail;
-   r = ws->cs_finalize(ace_post_cs);
+   r = radv_finalize_cmd_stream(device, ace_post_cs);
    if (r != VK_SUCCESS)
       goto fail;
 
@@ -1521,13 +1504,13 @@ radv_create_gang_wait_preambles_postambles(struct radv_queue *queue)
 
 fail:
    if (leader_pre_cs)
-      ws->cs_destroy(leader_pre_cs);
+      radv_destroy_cmd_stream(device, leader_pre_cs);
    if (leader_post_cs)
-      ws->cs_destroy(leader_post_cs);
+      radv_destroy_cmd_stream(device, leader_post_cs);
    if (ace_pre_cs)
-      ws->cs_destroy(ace_pre_cs);
+      radv_destroy_cmd_stream(device, ace_pre_cs);
    if (ace_post_cs)
-      ws->cs_destroy(ace_post_cs);
+      radv_destroy_cmd_stream(device, ace_post_cs);
    if (gang_sem_bo)
       radv_bo_destroy(device, &queue->vk.base, gang_sem_bo);
 
@@ -1585,22 +1568,23 @@ radv_update_gang_preambles(struct radv_queue *queue)
    return VK_SUCCESS;
 }
 
-static struct radeon_cmdbuf *
+static struct radv_cmd_stream *
 radv_create_perf_counter_lock_cs(struct radv_device *device, unsigned pass, bool unlock)
 {
-   struct radeon_cmdbuf **cs_ref = &device->perf_counter_lock_cs[pass * 2 + (unlock ? 1 : 0)];
-   struct radeon_cmdbuf *cs;
+   struct radv_cmd_stream **cs_ref = &device->perf_counter_lock_cs[pass * 2 + (unlock ? 1 : 0)];
+   struct radv_cmd_stream *cs;
+   VkResult result;
 
    if (*cs_ref)
       return *cs_ref;
 
-   cs = device->ws->cs_create(device->ws, AMD_IP_GFX, false);
-   if (!cs)
+   result = radv_create_cmd_stream(device, AMD_IP_GFX, false, &cs);
+   if (result != VK_SUCCESS)
       return NULL;
 
-   ASSERTED unsigned cdw = radeon_check_space(device->ws, cs, 21);
+   ASSERTED unsigned cdw = radeon_check_space(device->ws, cs->b, 21);
 
-   radv_cs_add_buffer(device->ws, cs, device->perf_counter_bo);
+   radv_cs_add_buffer(device->ws, cs->b, device->perf_counter_bo);
 
    radeon_begin(cs);
 
@@ -1650,11 +1634,11 @@ radv_create_perf_counter_lock_cs(struct radv_device *device, unsigned pass, bool
    }
 
    radeon_end();
-   assert(cs->cdw <= cdw);
+   assert(cs->b->cdw <= cdw);
 
-   VkResult result = device->ws->cs_finalize(cs);
+   result = radv_finalize_cmd_stream(device, cs);
    if (result != VK_SUCCESS) {
-      device->ws->cs_destroy(cs);
+      radv_destroy_cmd_stream(device, cs);
       return NULL;
    }
 
@@ -1662,7 +1646,7 @@ radv_create_perf_counter_lock_cs(struct radv_device *device, unsigned pass, bool
     * alternative.
     */
    if (p_atomic_cmpxchg((uintptr_t *)cs_ref, 0, (uintptr_t)cs) != 0) {
-      device->ws->cs_destroy(cs);
+      radv_destroy_cmd_stream(device, cs);
    }
 
    return *cs_ref;
@@ -1708,7 +1692,7 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
    const unsigned max_cs_submission = radv_device_fault_detection_enabled(device) ? 1 : cmd_buffer_count;
    const unsigned cs_array_size = (use_ace ? 2 : 1) * MIN2(max_cs_submission, cmd_buffer_count);
 
-   struct radeon_cmdbuf **cs_array = malloc(sizeof(struct radeon_cmdbuf *) * cs_array_size);
+   struct ac_cmdbuf **cs_array = malloc(sizeof(struct ac_cmdbuf *) * cs_array_size);
    if (!cs_array)
       return VK_ERROR_OUT_OF_HOST_MEMORY;
 
@@ -1742,24 +1726,24 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
    unsigned num_initial_preambles = 0;
    unsigned num_continue_preambles = 0;
    unsigned num_postambles = 0;
-   struct radeon_cmdbuf *initial_preambles[5] = {0};
-   struct radeon_cmdbuf *continue_preambles[5] = {0};
-   struct radeon_cmdbuf *postambles[4] = {0};
+   struct ac_cmdbuf *initial_preambles[5] = {0};
+   struct ac_cmdbuf *continue_preambles[5] = {0};
+   struct ac_cmdbuf *postambles[4] = {0};
 
    if (queue->state.qf == RADV_QUEUE_GENERAL || queue->state.qf == RADV_QUEUE_COMPUTE) {
       initial_preambles[num_initial_preambles++] =
-         need_wait ? queue->state.initial_full_flush_preamble_cs : queue->state.initial_preamble_cs;
+         need_wait ? queue->state.initial_full_flush_preamble_cs->b : queue->state.initial_preamble_cs->b;
 
-      continue_preambles[num_continue_preambles++] = queue->state.continue_preamble_cs;
+      continue_preambles[num_continue_preambles++] = queue->state.continue_preamble_cs->b;
 
       if (use_perf_counters) {
          /* RADV only supports perf counters on the GFX queue currently. */
          assert(queue->state.qf == RADV_QUEUE_GENERAL);
 
          /* Create the lock/unlock CS. */
-         struct radeon_cmdbuf *perf_ctr_lock_cs =
+         struct radv_cmd_stream *perf_ctr_lock_cs =
             radv_create_perf_counter_lock_cs(device, submission->perf_pass_index, false);
-         struct radeon_cmdbuf *perf_ctr_unlock_cs =
+         struct radv_cmd_stream *perf_ctr_unlock_cs =
             radv_create_perf_counter_lock_cs(device, submission->perf_pass_index, true);
 
          if (!perf_ctr_lock_cs || !perf_ctr_unlock_cs) {
@@ -1767,14 +1751,14 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
             goto fail;
          }
 
-         initial_preambles[num_initial_preambles++] = perf_ctr_lock_cs;
-         continue_preambles[num_continue_preambles++] = perf_ctr_lock_cs;
-         postambles[num_postambles++] = perf_ctr_unlock_cs;
+         initial_preambles[num_initial_preambles++] = perf_ctr_lock_cs->b;
+         continue_preambles[num_continue_preambles++] = perf_ctr_lock_cs->b;
+         postambles[num_postambles++] = perf_ctr_unlock_cs->b;
       }
    }
 
    if (queue->state.flush_postamble_cs) {
-      postambles[num_postambles++] = queue->state.flush_postamble_cs;
+      postambles[num_postambles++] = queue->state.flush_postamble_cs->b;
    }
 
    const unsigned num_1q_initial_preambles = num_initial_preambles;
@@ -1782,17 +1766,17 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
    const unsigned num_1q_postambles = num_postambles;
 
    if (use_ace) {
-      initial_preambles[num_initial_preambles++] = queue->state.gang_wait_preamble_cs;
-      initial_preambles[num_initial_preambles++] = queue->follower_state->gang_wait_preamble_cs;
-      initial_preambles[num_initial_preambles++] =
-         need_wait ? queue->follower_state->initial_full_flush_preamble_cs : queue->follower_state->initial_preamble_cs;
+      initial_preambles[num_initial_preambles++] = queue->state.gang_wait_preamble_cs->b;
+      initial_preambles[num_initial_preambles++] = queue->follower_state->gang_wait_preamble_cs->b;
+      initial_preambles[num_initial_preambles++] = need_wait ? queue->follower_state->initial_full_flush_preamble_cs->b
+                                                             : queue->follower_state->initial_preamble_cs->b;
 
-      continue_preambles[num_continue_preambles++] = queue->state.gang_wait_preamble_cs;
-      continue_preambles[num_continue_preambles++] = queue->follower_state->gang_wait_preamble_cs;
-      continue_preambles[num_continue_preambles++] = queue->follower_state->continue_preamble_cs;
+      continue_preambles[num_continue_preambles++] = queue->state.gang_wait_preamble_cs->b;
+      continue_preambles[num_continue_preambles++] = queue->follower_state->gang_wait_preamble_cs->b;
+      continue_preambles[num_continue_preambles++] = queue->follower_state->continue_preamble_cs->b;
 
-      postambles[num_postambles++] = queue->follower_state->gang_wait_postamble_cs;
-      postambles[num_postambles++] = queue->state.gang_wait_postamble_cs;
+      postambles[num_postambles++] = queue->follower_state->gang_wait_postamble_cs->b;
+      postambles[num_postambles++] = queue->state.gang_wait_postamble_cs->b;
    }
 
    struct radv_winsys_submit_info submit = {
@@ -1818,20 +1802,22 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
       if (radv_device_fault_detection_enabled(device))
          device->trace_data->primary_id = 0;
 
-      struct radeon_cmdbuf *chainable = NULL;
-      struct radeon_cmdbuf *chainable_ace = NULL;
+      struct ac_cmdbuf *chainable = NULL;
+      struct ac_cmdbuf *chainable_ace = NULL;
 
       /* Add CS from submitted command buffers. */
       for (unsigned c = 0; c < advance; ++c) {
          struct radv_cmd_buffer *cmd_buffer = (struct radv_cmd_buffer *)submission->command_buffers[j + c];
          assert(cmd_buffer->vk.level == VK_COMMAND_BUFFER_LEVEL_PRIMARY);
          const bool can_chain_next = !(cmd_buffer->usage_flags & VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT);
+         struct radv_cmd_stream *cs = cmd_buffer->cs;
+         struct radv_cmd_stream *ace_cs = cmd_buffer->gang.cs;
 
          /* Follower needs to be before the gang leader because the last CS must match the queue's IP type. */
-         if (cmd_buffer->gang.cs) {
-            device->ws->cs_unchain(cmd_buffer->gang.cs);
-            if (!chainable_ace || !device->ws->cs_chain(chainable_ace, cmd_buffer->gang.cs, false)) {
-               cs_array[num_submitted_cs++] = cmd_buffer->gang.cs;
+         if (ace_cs) {
+            device->ws->cs_unchain(ace_cs->b);
+            if (!chainable_ace || !device->ws->cs_chain(chainable_ace, ace_cs->b, false)) {
+               cs_array[num_submitted_cs++] = ace_cs->b;
 
                /* Prevent chaining the gang leader when the follower couldn't be chained.
                 * Otherwise, they would be in the wrong order.
@@ -1839,19 +1825,18 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
                chainable = NULL;
             }
 
-            chainable_ace = can_chain_next ? cmd_buffer->gang.cs : NULL;
+            chainable_ace = can_chain_next ? ace_cs->b : NULL;
             submit_ace = true;
          }
 
-         device->ws->cs_unchain(cmd_buffer->cs);
-         if (!chainable || !device->ws->cs_chain(chainable, cmd_buffer->cs, queue->state.uses_shadow_regs)) {
+         device->ws->cs_unchain(cs->b);
+         if (!chainable || !device->ws->cs_chain(chainable, cs->b, queue->state.uses_shadow_regs)) {
             /* don't submit empty command buffers to the kernel. */
-            if ((radv_queue_ring(queue) != AMD_IP_VCN_ENC && radv_queue_ring(queue) != AMD_IP_UVD) ||
-                cmd_buffer->cs->cdw != 0)
-               cs_array[num_submitted_cs++] = cmd_buffer->cs;
+            if ((radv_queue_ring(queue) != AMD_IP_VCN_ENC && radv_queue_ring(queue) != AMD_IP_UVD) || cs->b->cdw != 0)
+               cs_array[num_submitted_cs++] = cs->b;
          }
 
-         chainable = can_chain_next ? cmd_buffer->cs : NULL;
+         chainable = can_chain_next ? cs->b : NULL;
       }
 
       submit.cs_count = num_submitted_cs;
@@ -1873,8 +1858,8 @@ radv_queue_submit_normal(struct radv_queue *queue, struct vk_queue_submit *submi
          radv_check_trap_handler(queue);
       }
 
-      initial_preambles[0] = queue->state.initial_preamble_cs;
-      initial_preambles[1] = !use_ace ? NULL : queue->follower_state->initial_preamble_cs;
+      initial_preambles[0] = queue->state.initial_preamble_cs ? queue->state.initial_preamble_cs->b : NULL;
+      initial_preambles[1] = !use_ace ? NULL : queue->follower_state->initial_preamble_cs->b;
    }
 
    queue->last_shader_upload_seq = MAX2(queue->last_shader_upload_seq, shader_upload_seq);
@@ -1980,7 +1965,7 @@ fail:
 }
 
 bool
-radv_queue_internal_submit(struct radv_queue *queue, struct radeon_cmdbuf *cs)
+radv_queue_internal_submit(struct radv_queue *queue, struct ac_cmdbuf *cs)
 {
    struct radv_device *device = radv_queue_device(queue);
    struct radeon_winsys_ctx *ctx = queue->hw_ctx;
@@ -2006,9 +1991,13 @@ radv_queue_init(struct radv_device *device, struct radv_queue *queue, int idx,
    const struct radv_physical_device *pdev = radv_device_physical(device);
 
    queue->priority = radv_get_queue_global_priority(global_priority);
-   queue->hw_ctx = device->hw_ctx[queue->priority];
    queue->state.qf = vk_queue_to_radv(pdev, create_info->queueFamilyIndex);
 
+   if (queue->state.qf == RADV_QUEUE_VIDEO_ENC && device->hw_vcn_enc_ctx)
+      queue->hw_ctx = device->hw_vcn_enc_ctx;
+   else
+      queue->hw_ctx = device->hw_ctx[queue->priority];
+
    VkResult result = vk_queue_init(&queue->vk, &device->vk, create_info, idx);
    if (result != VK_SUCCESS)
       return result;
@@ -2047,17 +2036,17 @@ radv_queue_state_finish(struct radv_queue_state *queue, struct radv_device *devi
 {
    radv_destroy_shadow_regs_preamble(device, queue, device->ws);
    if (queue->initial_full_flush_preamble_cs)
-      device->ws->cs_destroy(queue->initial_full_flush_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->initial_full_flush_preamble_cs);
    if (queue->initial_preamble_cs)
-      device->ws->cs_destroy(queue->initial_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->initial_preamble_cs);
    if (queue->continue_preamble_cs)
-      device->ws->cs_destroy(queue->continue_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->continue_preamble_cs);
    if (queue->gang_wait_preamble_cs)
-      device->ws->cs_destroy(queue->gang_wait_preamble_cs);
+      radv_destroy_cmd_stream(device, queue->gang_wait_preamble_cs);
    if (queue->gang_wait_postamble_cs)
-      device->ws->cs_destroy(queue->gang_wait_postamble_cs);
+      radv_destroy_cmd_stream(device, queue->gang_wait_postamble_cs);
    if (queue->flush_postamble_cs)
-      device->ws->cs_destroy(queue->flush_postamble_cs);
+      radv_destroy_cmd_stream(device, queue->flush_postamble_cs);
    if (queue->descriptor_bo)
       radv_bo_destroy(device, NULL, queue->descriptor_bo);
    if (queue->scratch_bo) {
@@ -2146,6 +2135,6 @@ radv_queue_family_to_ring(const struct radv_physical_device *pdev, enum radv_que
    case RADV_QUEUE_VIDEO_ENC:
       return AMD_IP_VCN_ENC;
    default:
-      unreachable("Unknown queue family");
+      UNREACHABLE("Unknown queue family");
    }
 }
