@@ -73,7 +73,7 @@ radv_amdgpu_winsys_query_value(struct radeon_winsys *rws, enum radeon_value_id v
       ac_drm_query_sensor_info(ws->dev, AMDGPU_INFO_SENSOR_GFX_MCLK, 4, &retval);
       return retval;
    default:
-      unreachable("invalid query value");
+      UNREACHABLE("invalid query value");
    }
 
    return 0;
@@ -148,6 +148,9 @@ radv_amdgpu_winsys_destroy(struct radeon_winsys *rws)
    u_rwlock_destroy(&ws->global_bo_list.lock);
    free(ws->global_bo_list.bos);
 
+   if (ws->bo_history_logfile)
+      fclose(ws->bo_history_logfile);
+
    u_rwlock_destroy(&ws->log_bo_list_lock);
    ac_drm_device_deinitialize(ws->dev);
    FREE(rws);
@@ -193,8 +196,8 @@ radv_amdgpu_winsys_unreserve_vmid(struct radeon_winsys *rws)
 static uint64_t
 radv_amdgpu_winsys_filter_perftest_flags(uint64_t perftest_flags)
 {
-   return perftest_flags & (RADV_PERFTEST_NO_GTT_SPILL | RADV_PERFTEST_LOCAL_BOS | RADV_PERFTEST_NO_SAM |
-                            RADV_PERFTEST_SAM | RADV_PERFTEST_BO_LIST);
+   return perftest_flags &
+          (RADV_PERFTEST_NO_GTT_SPILL | RADV_PERFTEST_LOCAL_BOS | RADV_PERFTEST_NO_SAM | RADV_PERFTEST_SAM);
 }
 
 VkResult
@@ -231,9 +234,9 @@ radv_amdgpu_winsys_create(int fd, uint64_t debug_flags, uint64_t perftest_flags,
       ++ws->refcount;
    }
 
-   if (is_virtio && (perftest_flags & (RADV_PERFTEST_BO_LIST | RADV_PERFTEST_LOCAL_BOS))) {
+   if (is_virtio && (perftest_flags & RADV_PERFTEST_LOCAL_BOS)) {
       /* virtio doesn't support VM_ALWAYS_VALID, so disable options that requires it. */
-      fprintf(stderr, "localbos and bolist options are not supported values for RADV_PERFTEST with virtio.\n");
+      fprintf(stderr, "RADV_PERFTEST=localbos is not supported with virtio.\n");
       return VK_ERROR_INITIALIZATION_FAILED;
    }
 
@@ -244,7 +247,7 @@ radv_amdgpu_winsys_create(int fd, uint64_t debug_flags, uint64_t perftest_flags,
       /* Check that options don't differ from the existing winsys. */
       if (((debug_flags & RADV_DEBUG_ALL_BOS) && !ws->debug_all_bos) ||
           ((debug_flags & RADV_DEBUG_HANG) && !ws->debug_log_bos) ||
-          ((debug_flags & RADV_DEBUG_NO_IBS) && ws->use_ib_bos) || (perftest_flags != ws->perftest)) {
+          ((debug_flags & RADV_DEBUG_NO_IB_CHAINING) && ws->chain_ib) || (perftest_flags != ws->perftest)) {
          fprintf(stderr, "radv/amdgpu: Found options that differ from the existing winsys.\n");
          return VK_ERROR_INITIALIZATION_FAILED;
       }
@@ -291,20 +294,20 @@ radv_amdgpu_winsys_create(int fd, uint64_t debug_flags, uint64_t perftest_flags,
    ws->info.ip[AMD_IP_SDMA].num_queues = MIN2(ws->info.ip[AMD_IP_SDMA].num_queues, MAX_RINGS_PER_TYPE);
    ws->info.ip[AMD_IP_COMPUTE].num_queues = MIN2(ws->info.ip[AMD_IP_COMPUTE].num_queues, MAX_RINGS_PER_TYPE);
 
-   ws->use_ib_bos = true;
-
+   ws->chain_ib = !(debug_flags & RADV_DEBUG_NO_IB_CHAINING);
    ws->debug_all_bos = !!(debug_flags & RADV_DEBUG_ALL_BOS);
    ws->debug_log_bos = debug_flags & RADV_DEBUG_HANG;
-   if (debug_flags & RADV_DEBUG_NO_IBS)
-      ws->use_ib_bos = false;
+
+   if (debug_flags & RADV_DEBUG_DUMP_BO_HISTORY) {
+      ws->bo_history_logfile = fopen("/tmp/radv_bo_history.log", "w+");
+      if (!ws->bo_history_logfile)
+         fprintf(stderr, "radv/amdgpu: Failed to create /tmp/radv_bo_history.log.\n");
+   }
 
    int num_sync_types = 0;
 
    ws->syncobj_sync_type = vk_drm_syncobj_get_type_from_provider(ac_drm_device_get_sync_provider(dev));
    if (ws->syncobj_sync_type.features) {
-      /* multi wait is always supported */
-      ws->syncobj_sync_type.features |= VK_SYNC_FEATURE_GPU_MULTI_WAIT;
-
       if (!ws->info.has_timeline_syncobj && ws->syncobj_sync_type.features & VK_SYNC_FEATURE_TIMELINE) {
          /* Disable timeline feature if it was disabled in the driver. */
          assert(is_virtio);
@@ -336,6 +339,7 @@ radv_amdgpu_winsys_create(int fd, uint64_t debug_flags, uint64_t perftest_flags,
    ws->base.get_fd = radv_amdgpu_winsys_get_fd;
    ws->base.get_sync_types = radv_amdgpu_winsys_get_sync_types;
    ws->base.get_sync_provider = radv_amdgpu_winsys_get_sync_provider;
+   ws->base.copy_sync_payloads = vk_drm_syncobj_copy_payloads;
    ws->base.reserve_vmid = radv_amdgpu_winsys_reserve_vmid;
    ws->base.unreserve_vmid = radv_amdgpu_winsys_unreserve_vmid;
    radv_amdgpu_bo_init_functions(ws);
