@@ -50,7 +50,7 @@ assert_register_is_aligned(const agx_instr *I, agx_index reg)
       return;
    }
 
-   unreachable("Invalid register size");
+   UNREACHABLE("Invalid register size");
 }
 
 /* Texturing has its own operands */
@@ -58,11 +58,10 @@ static unsigned
 agx_pack_sample_coords(const agx_instr *I, agx_index index, bool *flag,
                        bool *is_16)
 {
-   /* TODO: Do we have a use case for 16-bit coords? */
-   pack_assert_msg(I, index.size == AGX_SIZE_32, "32-bit coordinates");
+   pack_assert_msg(I, index.size <= AGX_SIZE_32, "32-bit coordinates");
    pack_assert_msg(I, index.value < 0x100, "coordinate register bound");
 
-   *is_16 = false;
+   *is_16 = index.size == AGX_SIZE_16;
    *flag = index.discard;
    return index.value;
 }
@@ -88,7 +87,7 @@ agx_pack_texture(const agx_instr *I, agx_index base, agx_index index,
       pack_assert(I, base.type == AGX_INDEX_UNIFORM);
       pack_assert(I, base.size == AGX_SIZE_64);
       pack_assert(I, (base.value & 3) == 0);
-      pack_assert(I, index.size == AGX_SIZE_32);
+      pack_assert(I, index.size <= AGX_SIZE_32);
 
       /* Bindless */
       *packed_base = base.value >> 2;
@@ -169,7 +168,7 @@ agx_pack_pbe_lod(const agx_instr *I, agx_index index, bool *flag)
    else if (index.type == AGX_INDEX_REGISTER)
       *flag = false;
    else
-      unreachable("Invalid PBE LOD type");
+      UNREACHABLE("Invalid PBE LOD type");
 
    return index.value;
 }
@@ -579,8 +578,8 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups,
       unsigned C = I->explicit_coords ? coords.value : 0;
 
       uint64_t raw = agx_opcodes_info[I->op].encoding.exact |
-                     ((uint64_t)(D & BITFIELD_MASK(8)) << 7) | (St << 22) |
-                     ((uint64_t)(I->format) << 24) |
+                     ((uint64_t)((D >> 1) & BITFIELD_MASK(7)) << 8) |
+                     (St << 22) | ((uint64_t)(I->format) << 24) |
                      ((uint64_t)(C & BITFIELD_MASK(6)) << 16) |
                      ((uint64_t)(I->pixel_offset & BITFIELD_MASK(7)) << 28) |
                      (load || I->explicit_coords ? (1ull << 35) : 0) |
@@ -628,6 +627,9 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups,
       unsigned D = agx_pack_alu_dst(I, I->dest[0]);
       unsigned channels = (I->channels & 0x3);
 
+      /* Destination cache not supported, mask off */
+      D &= ~1;
+
       agx_index src_I = I->src[0];
       pack_assert(I, src_I.type == AGX_INDEX_IMMEDIATE ||
                         src_I.type == AGX_INDEX_REGISTER);
@@ -910,8 +912,8 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups,
          (q2 << 30) | (((uint64_t)(T & BITFIELD_MASK(6))) << 32) |
          (((uint64_t)Tt) << 38) |
          (((uint64_t)(I->dim & BITFIELD_MASK(3))) << 40) |
-         (((uint64_t)q3) << 43) | (((uint64_t)I->mask) << 48) |
-         (((uint64_t)lod_mode) << 52) |
+         (((uint64_t)q3) << 43) | (Cs ? BITFIELD64_BIT(47) : 0) |
+         (((uint64_t)I->mask) << 48) | (((uint64_t)lod_mode) << 52) |
          (((uint64_t)(S & BITFIELD_MASK(6))) << 56) | (((uint64_t)St) << 62) |
          (((uint64_t)I->scoreboard) << 63);
 
@@ -928,12 +930,14 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups,
 
       pack_assert(I, U < (1 << 4));
       pack_assert(I, (T & 1) == 0);
-      pack_assert(I, T < (1 << 8));
+      pack_assert(I, T < (1 << 24));
       pack_assert(I, I->imm < (1 << 7));
 
-      uint64_t raw = agx_opcodes_info[I->op].encoding.exact | (I->imm << 8) |
-                     (I->scoreboard << 16) | ((uint64_t)(T >> 1) << 27) |
-                     ((uint64_t)(U << 1) << 58);
+      uint64_t raw =
+         agx_opcodes_info[I->op].encoding.exact | (I->imm << 8) |
+         (I->scoreboard << 16) | ((uint64_t)T << 26) |
+         ((I->src[1].type == AGX_INDEX_IMMEDIATE) ? (1ul << 58) : 0) |
+         ((uint64_t)U << 59);
 
       memcpy(util_dynarray_grow_bytes(emission, 1, 8), &raw, 8);
       break;
@@ -1169,6 +1173,12 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups,
       break;
    }
 
+   case AGX_OPCODE_EXPORT:
+      /* Zero-byte pseudo-op. This remains until late for the benefit of
+       * register cache optimizations.
+       */
+      return;
+
    default:
       agx_pack_alu(emission, I);
       return;
