@@ -5,6 +5,9 @@ import sys
 
 a = 'a'
 
+has_unpack_sat = 'c && v3d_device_has_unpack_sat(c->devinfo)'
+has_unpack_max0 = 'c && v3d_device_has_unpack_max0(c->devinfo)'
+
 lower_alu = [
     (('f2i8', a), ('i2i8', ('f2i32', a))),
     (('f2i16', a), ('i2i16', ('f2i32', a))),
@@ -17,6 +20,10 @@ lower_alu = [
 
     (('u2f32', 'a@8'), ('u2f32', ('u2u32', a))),
     (('u2f32', 'a@16'), ('u2f32', ('u2u32', a))),
+
+    (('fmin', ('fmax', a, -1.0), 1.0), ('fsat_signed', a), has_unpack_sat),
+    (('fmax', ('fmin', a, 1.0), -1.0), ('fsat_signed', a), has_unpack_sat),
+    (('fmax', a, 0.0), ('fclamp_pos', a), has_unpack_max0),
 ]
 
 def main():
@@ -32,7 +39,10 @@ def run():
     print('#include "v3d_compiler.h"')
 
     print(nir_algebraic.AlgebraicPass("v3d_nir_lower_algebraic",
-                                      lower_alu).render())
+                                      lower_alu,
+                                      [
+                                          ("const struct v3d_compile *", "c")
+                                      ]).render())
 
 if __name__ == '__main__':
     main()
