@@ -45,6 +45,39 @@
          TEXTURE_DATA_FORMAT_##tex,                                 \
          swiz,                                                      \
          return_size,                                               \
+         false,                                                     \
+         false,                                                     \
+      }},                                                           \
+      supports_filtering,                                           \
+   }
+
+/* Format requiring software unorm packing */
+#define FORMAT_UNORM(vk, rt, tex, swiz, return_size, supports_filtering)  \
+   [VK_ENUM_OFFSET(VK_FORMAT_##vk)] = {                             \
+      1,                                                            \
+      {{                                                            \
+         V3D_OUTPUT_IMAGE_FORMAT_##rt,                              \
+         TEXTURE_DATA_FORMAT_##tex,                                 \
+         swiz,                                                      \
+         return_size,                                               \
+         true,                                                      \
+         false,                                                     \
+      }},                                                           \
+      supports_filtering,                                           \
+   }
+
+
+/* Format requiring software snorm packing */
+#define FORMAT_SNORM(vk, rt, tex, swiz, return_size, supports_filtering)  \
+   [VK_ENUM_OFFSET(VK_FORMAT_##vk)] = {                             \
+      1,                                                            \
+      {{                                                            \
+         V3D_OUTPUT_IMAGE_FORMAT_##rt,                              \
+         TEXTURE_DATA_FORMAT_##tex,                                 \
+         swiz,                                                      \
+         return_size,                                               \
+         false,                                                     \
+         true,                                                      \
       }},                                                           \
       supports_filtering,                                           \
    }
@@ -96,8 +129,8 @@ static const struct v3dv_format format_table[] = {
    FORMAT(R8G8B8A8_UINT,           RGBA8UI,      RGBA8UI,       SWIZ_XYZW, 16, false),
 
    FORMAT(R16G16B16A16_SFLOAT,     RGBA16F,      RGBA16F,       SWIZ_XYZW, 16, true),
-   FORMAT(R16G16B16A16_UNORM,      NO,           RGBA16,        SWIZ_XYZW, 32, true),
-   FORMAT(R16G16B16A16_SNORM,      NO,           RGBA16_SNORM,  SWIZ_XYZW, 32, true),
+   FORMAT_UNORM(R16G16B16A16_UNORM,RGBA16UI,     RGBA16,        SWIZ_XYZW, 32, true),
+   FORMAT_SNORM(R16G16B16A16_SNORM,RGBA16I,      RGBA16_SNORM,  SWIZ_XYZW, 32, true),
    FORMAT(R16G16B16A16_SINT,       RGBA16I,      RGBA16I,       SWIZ_XYZW, 16, false),
    FORMAT(R16G16B16A16_UINT,       RGBA16UI,     RGBA16UI,      SWIZ_XYZW, 16, false),
 
@@ -116,8 +149,8 @@ static const struct v3dv_format format_table[] = {
    FORMAT(R8G8_SINT,               RG8I,         RG8I,          SWIZ_XY01, 16, false),
    FORMAT(R8G8_UINT,               RG8UI,        RG8UI,         SWIZ_XY01, 16, false),
 
-   FORMAT(R16G16_UNORM,            NO,           RG16,          SWIZ_XY01, 32, true),
-   FORMAT(R16G16_SNORM,            NO,           RG16_SNORM,    SWIZ_XY01, 32, true),
+   FORMAT_UNORM(R16G16_UNORM,      RG16UI,       RG16,          SWIZ_XY01, 32, true),
+   FORMAT_SNORM(R16G16_SNORM,      RG16I,        RG16_SNORM,    SWIZ_XY01, 32, true),
    FORMAT(R16G16_SFLOAT,           RG16F,        RG16F,         SWIZ_XY01, 16, true),
    FORMAT(R16G16_SINT,             RG16I,        RG16I,         SWIZ_XY01, 16, false),
    FORMAT(R16G16_UINT,             RG16UI,       RG16UI,        SWIZ_XY01, 16, false),
@@ -132,8 +165,8 @@ static const struct v3dv_format format_table[] = {
    FORMAT(R8_SINT,                 R8I,          R8I,           SWIZ_X001, 16, false),
    FORMAT(R8_UINT,                 R8UI,         R8UI,          SWIZ_X001, 16, false),
 
-   FORMAT(R16_UNORM,               NO,           R16,           SWIZ_X001, 32, true),
-   FORMAT(R16_SNORM,               NO,           R16_SNORM,     SWIZ_X001, 32, true),
+   FORMAT_UNORM(R16_UNORM,         R16UI,        R16,           SWIZ_X001, 32, true),
+   FORMAT_SNORM(R16_SNORM,         R16I,         R16_SNORM,     SWIZ_X001, 32, true),
    FORMAT(R16_SFLOAT,              R16F,         R16F,          SWIZ_X001, 16, true),
    FORMAT(R16_SINT,                R16I,         R16I,          SWIZ_X001, 16, false),
    FORMAT(R16_UINT,                R16UI,        R16UI,         SWIZ_X001, 16, false),
@@ -242,6 +275,14 @@ static const struct v3dv_format format_table_4444[] = {
    FORMAT(A4R4G4B4_UNORM_PACK16, ABGR4444, RGBA4, SWIZ_YZWX, 16, true), /* Reverse + RB swap */
 };
 
+/* VK_KHR_maintenance5 introduces A1B5G5R5 and A8 but we only support the
+ * former. It might be possible to support A8 as R8 with special casing
+ * in a number of places but it would probably take some effort.
+ */
+static const struct v3dv_format format_table_maintenance5[] = {
+   FORMAT(A1B5G5R5_UNORM_PACK16_KHR, RGBA5551, A1_RGB5, SWIZ_XYZW, 16, true),
+};
+
 static const struct v3dv_format format_table_ycbcr[] = {
    YCBCR_FORMAT(G8_B8R8_2PLANE_420_UNORM, false, 2,
        PLANE(R8, R8, SWIZ(X, 0, 0, 1), 16),
@@ -275,6 +316,11 @@ v3dX(get_format)(VkFormat format)
          return &format_table_ycbcr[enum_offset];
       else
          return NULL;
+   case _VK_KHR_maintenance5_number:
+      if (enum_offset < ARRAY_SIZE(format_table_maintenance5))
+         return &format_table_maintenance5[enum_offset];
+      else
+         return NULL;
    default:
       return NULL;
    }
@@ -504,7 +550,7 @@ v3dX(get_internal_depth_type)(VkFormat format)
    case VK_FORMAT_D24_UNORM_S8_UINT:
       return V3D_INTERNAL_TYPE_DEPTH_24;
    default:
-      unreachable("Invalid depth format");
+      UNREACHABLE("Invalid depth format");
       break;
    }
 }
