@@ -93,10 +93,9 @@ public:
       if (lowering_op == LOWER_PACK_UNPACK_NONE)
          return;
 
-      setup_factory(ralloc_parent(expr));
+      setup_factory(expr->node_linalloc);
 
       ir_rvalue *op0 = expr->operands[0];
-      ralloc_steal(factory.mem_ctx, op0);
 
       switch (lowering_op) {
       case LOWER_PACK_SNORM_2x16:
@@ -144,7 +143,7 @@ private:
    const int op_mask;
    bool progress;
    ir_factory factory;
-   exec_list factory_instructions;
+   ir_exec_list factory_instructions;
 
    /**
     * Determine the needed lowering operation by filtering \a expr_op
@@ -199,12 +198,12 @@ private:
    }
 
    void
-   setup_factory(void *mem_ctx)
+   setup_factory(linear_ctx *linalloc)
    {
-      assert(factory.mem_ctx == NULL);
+      assert(factory.linalloc == NULL);
       assert(factory.instructions->is_empty());
 
-      factory.mem_ctx = mem_ctx;
+      factory.linalloc = linalloc;
    }
 
    void
@@ -212,7 +211,7 @@ private:
    {
       base_ir->insert_before(factory.instructions);
       assert(factory.instructions->is_empty());
-      factory.mem_ctx = NULL;
+      factory.linalloc = NULL;
    }
 
    template <typename T>
@@ -1326,7 +1325,7 @@ private:
  * \brief Lower the builtin packing functions.
  */
 bool
-lower_packing_builtins(exec_list *instructions,
+lower_packing_builtins(ir_exec_list *instructions,
                        bool has_shading_language_packing,
                        bool has_gpu_shader5,
                        bool has_half_float_packing)
