@@ -39,22 +39,13 @@
  */
 
 struct divergence_state {
-   const gl_shader_stage stage;
+   const mesa_shader_stage stage;
    nir_shader *shader;
    nir_function_impl *impl;
    nir_divergence_options options;
    nir_loop *loop;
    bool loop_all_invariant;
 
-   /* Whether the caller requested vertex divergence (meaning between vertices
-    * of the same primitive) instead of subgroup invocation divergence
-    * (between invocations of the same subgroup). For example, patch input
-    * loads are always convergent, while subgroup intrinsics are divergent
-    * because vertices of the same primitive can be processed by different
-    * subgroups.
-    */
-   bool vertex_divergence;
-
    /** current control flow state */
    /* True if some loop-active invocations might take a different control-flow path.
     * A divergent break does not cause subsequent control-flow to be considered
@@ -83,7 +74,7 @@ nir_src_is_divergent(nir_src *src)
       return true;
 
    nir_cf_node *use_node = nir_src_get_block(src)->cf_node.parent;
-   nir_cf_node *def_node = src->ssa->parent_instr->block->cf_node.parent;
+   nir_cf_node *def_node = nir_def_block(src->ssa)->cf_node.parent;
 
    /* Short-cut the common case. */
    if (def_node == use_node)
@@ -130,14 +121,14 @@ src_invariant(nir_src *src, void *loop)
    nir_block *first_block = nir_loop_first_block(loop);
 
    /* Invariant if SSA is defined before the current loop. */
-   if (src->ssa->parent_instr->block->index < first_block->index)
+   if (nir_def_block(src->ssa)->index < first_block->index)
       return true;
 
    if (!src->ssa->loop_invariant)
       return false;
 
    /* The value might be defined in a nested loop. */
-   nir_cf_node *cf_node = src->ssa->parent_instr->block->cf_node.parent;
+   nir_cf_node *cf_node = nir_def_block(src->ssa)->cf_node.parent;
    while (cf_node->type != nir_cf_node_loop)
       cf_node = cf_node->parent;
 
@@ -166,17 +157,15 @@ visit_alu(nir_alu_instr *instr, struct divergence_state *state)
  * wave can "tear" so that different invocations see the pre-store value and
  * the post-store value even though they are loading from the same location.
  * This means we have to assume it's not uniform unless it's readonly.
- *
- * TODO The Vulkan memory model is much more strict here and requires an
- * atomic or volatile load for the data race to be valid, which could allow us
- * to do better if it's in use, however we currently don't have that
- * information plumbed through.
  */
 static bool
 load_may_tear(struct divergence_state *state, nir_intrinsic_instr *instr)
 {
+   uint32_t access = nir_intrinsic_access(instr);
+   bool atomic_volatile = access & (ACCESS_ATOMIC | ACCESS_VOLATILE);
    return (state->options & nir_divergence_uniform_load_tears) &&
-          !(nir_intrinsic_access(instr) & ACCESS_NON_WRITEABLE);
+          !(access & ACCESS_NON_WRITEABLE) &&
+          (!state->shader->info.assume_no_data_races || atomic_volatile);
 }
 
 static bool
@@ -189,7 +178,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       return false;
 
    nir_divergence_options options = state->options;
-   gl_shader_stage stage = state->stage;
+   mesa_shader_stage stage = state->stage;
    bool is_divergent = false;
    switch (instr->intrinsic) {
    case nir_intrinsic_shader_clock:
@@ -207,15 +196,18 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_first_invocation:
    case nir_intrinsic_last_invocation:
    case nir_intrinsic_load_subgroup_id:
+   case nir_intrinsic_load_core_id:
    case nir_intrinsic_shared_append_amd:
    case nir_intrinsic_shared_consume_amd:
    case nir_intrinsic_load_sm_id_nv:
    case nir_intrinsic_load_warp_id_nv:
+   case nir_intrinsic_load_warp_id_arm:
       /* VS/TES/GS invocations of the same primitive can be in different
        * subgroups, so subgroup ops are always divergent between vertices of
        * the same primitive.
        */
-      is_divergent = state->vertex_divergence;
+      is_divergent = (state->options & nir_divergence_vertex) ||
+                     (state->options & nir_divergence_across_subgroups);
       break;
 
    /* Intrinsics which are always uniform */
@@ -232,6 +224,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_subgroup_id_shift_ir3:
    case nir_intrinsic_load_base_instance:
    case nir_intrinsic_load_base_vertex:
+   case nir_intrinsic_load_raw_vertex_offset_pan:
    case nir_intrinsic_load_first_vertex:
    case nir_intrinsic_load_draw_id:
    case nir_intrinsic_load_is_indexed_draw:
@@ -274,7 +267,6 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_ring_es2gs_offset_amd:
    case nir_intrinsic_load_ring_task_draw_amd:
    case nir_intrinsic_load_ring_task_payload_amd:
-   case nir_intrinsic_load_sample_positions_amd:
    case nir_intrinsic_load_rasterization_samples_amd:
    case nir_intrinsic_load_ring_gsvs_amd:
    case nir_intrinsic_load_ring_gs2vs_offset_amd:
@@ -307,7 +299,6 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_tess_level_inner_default:
    case nir_intrinsic_load_tess_level_outer_default:
    case nir_intrinsic_load_scalar_arg_amd:
-   case nir_intrinsic_load_smem_amd:
    case nir_intrinsic_load_resume_shader_address_amd:
    case nir_intrinsic_load_reloc_const_intel:
    case nir_intrinsic_load_btd_global_arg_addr_intel:
@@ -329,14 +320,10 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_base_global_invocation_id:
    case nir_intrinsic_load_base_workgroup_id:
    case nir_intrinsic_load_alpha_reference_amd:
-   case nir_intrinsic_load_ubo_uniform_block_intel:
-   case nir_intrinsic_load_ssbo_uniform_block_intel:
-   case nir_intrinsic_load_shared_uniform_block_intel:
    case nir_intrinsic_load_barycentric_optimize_amd:
    case nir_intrinsic_load_poly_line_smooth_enabled:
    case nir_intrinsic_load_rasterization_primitive_amd:
    case nir_intrinsic_unit_test_uniform_amd:
-   case nir_intrinsic_load_global_constant_uniform_block_intel:
    case nir_intrinsic_load_debug_log_desc_amd:
    case nir_intrinsic_load_xfb_state_address_gfx12_amd:
    case nir_intrinsic_cmat_length:
@@ -354,10 +341,10 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_tcs_mem_attrib_stride:
    case nir_intrinsic_load_printf_buffer_address:
    case nir_intrinsic_load_printf_buffer_size:
-   case nir_intrinsic_load_core_id_agx:
    case nir_intrinsic_load_samples_log2_agx:
    case nir_intrinsic_load_active_subgroup_count_agx:
    case nir_intrinsic_load_root_agx:
+   case nir_intrinsic_load_descriptor_set_agx:
    case nir_intrinsic_load_sm_count_nv:
    case nir_intrinsic_load_warps_per_sm_nv:
    case nir_intrinsic_load_fs_msaa_intel:
@@ -367,9 +354,31 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_ray_base_mem_addr_intel:
    case nir_intrinsic_load_ray_hw_stack_size_intel:
    case nir_intrinsic_load_per_primitive_remap_intel:
+   case nir_intrinsic_load_core_count_arm:
+   case nir_intrinsic_load_core_max_id_arm:
+   case nir_intrinsic_load_warp_max_id_arm:
+   case nir_intrinsic_load_tess_config_intel:
       is_divergent = false;
       break;
 
+   case nir_intrinsic_load_ubo_uniform_block_intel:
+   case nir_intrinsic_load_ssbo_uniform_block_intel:
+   case nir_intrinsic_load_shared_uniform_block_intel:
+   case nir_intrinsic_load_global_constant_uniform_block_intel:
+      if (options & (nir_divergence_across_subgroups |
+                     nir_divergence_multiple_workgroup_per_compute_subgroup)) {
+         unsigned num_srcs = nir_intrinsic_infos[instr->intrinsic].num_srcs;
+         for (unsigned i = 0; i < num_srcs; i++) {
+            if (src_divergent(instr->src[i], state)) {
+               is_divergent = true;
+               break;
+            }
+         }
+      } else {
+         is_divergent = false;
+      }
+      break;
+
    /* This is divergent because it specifically loads sequential values into
     * successive SIMD lanes.
     */
@@ -397,13 +406,15 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
          is_divergent |= !(options & nir_divergence_single_prim_per_subgroup);
       } else if (stage == MESA_SHADER_TESS_EVAL) {
          /* Patch input loads are uniform between vertices of the same primitive. */
-         if (state->vertex_divergence)
+         if (state->options & nir_divergence_vertex)
             is_divergent = false;
          else
             is_divergent |= !(options & nir_divergence_single_patch_per_tes_subgroup);
       } else {
          is_divergent = true;
       }
+      if (options & nir_divergence_across_subgroups)
+         is_divergent = true;
       break;
    case nir_intrinsic_load_attribute_pan:
       assert(stage == MESA_SHADER_VERTEX);
@@ -420,6 +431,8 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
          is_divergent |= !(options & nir_divergence_single_patch_per_tes_subgroup);
       else
          is_divergent = true;
+      if (options & nir_divergence_across_subgroups)
+         is_divergent = true;
       break;
    case nir_intrinsic_load_input_vertex:
       is_divergent = src_divergent(instr->src[1], state);
@@ -442,7 +455,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
           */
          break;
       default:
-         unreachable("Invalid stage for load_output");
+         UNREACHABLE("Invalid stage for load_output");
       }
       break;
    case nir_intrinsic_load_per_view_output:
@@ -487,25 +500,25 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       is_divergent |= !(options & nir_divergence_single_prim_per_subgroup);
       break;
    case nir_intrinsic_load_instance_id:
-      is_divergent = !state->vertex_divergence;
+      is_divergent = !(state->options & nir_divergence_vertex);
       break;
    case nir_intrinsic_load_primitive_id:
       if (stage == MESA_SHADER_FRAGMENT)
          is_divergent = !(options & nir_divergence_single_prim_per_subgroup);
       else if (stage == MESA_SHADER_TESS_CTRL)
-         is_divergent = !state->vertex_divergence &&
+         is_divergent = !(state->options & nir_divergence_vertex) &&
                         !(options & nir_divergence_single_patch_per_tcs_subgroup);
       else if (stage == MESA_SHADER_TESS_EVAL)
-         is_divergent = !state->vertex_divergence &&
+         is_divergent = !(state->options & nir_divergence_vertex) &&
                         !(options & nir_divergence_single_patch_per_tes_subgroup);
       else if (stage == MESA_SHADER_GEOMETRY || stage == MESA_SHADER_VERTEX)
-         is_divergent = !state->vertex_divergence;
+         is_divergent = !(state->options & nir_divergence_vertex);
       else if (stage == MESA_SHADER_ANY_HIT ||
                stage == MESA_SHADER_CLOSEST_HIT ||
                stage == MESA_SHADER_INTERSECTION)
          is_divergent = true;
       else
-         unreachable("Invalid stage for load_primitive_id");
+         UNREACHABLE("Invalid stage for load_primitive_id");
       break;
    case nir_intrinsic_load_tess_level_inner:
    case nir_intrinsic_load_tess_level_outer:
@@ -514,12 +527,12 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       else if (stage == MESA_SHADER_TESS_EVAL)
          is_divergent = !(options & nir_divergence_single_patch_per_tes_subgroup);
       else
-         unreachable("Invalid stage for load_primitive_tess_level_*");
+         UNREACHABLE("Invalid stage for load_primitive_tess_level_*");
       break;
 
    case nir_intrinsic_load_workgroup_index:
    case nir_intrinsic_load_workgroup_id:
-      assert(gl_shader_stage_uses_workgroup(stage) || stage == MESA_SHADER_TESS_CTRL);
+      assert(mesa_shader_stage_uses_workgroup(stage) || stage == MESA_SHADER_TESS_CTRL);
       if (stage == MESA_SHADER_COMPUTE)
          is_divergent |= (options & nir_divergence_multiple_workgroup_per_compute_subgroup);
       break;
@@ -536,7 +549,8 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
           * vertices of the same primitive because they may be in
           * different subgroups.
           */
-         is_divergent = state->vertex_divergence;
+         is_divergent = (state->options & nir_divergence_vertex) ||
+                        (state->options & nir_divergence_across_subgroups);
          break;
       }
       FALLTHROUGH;
@@ -544,7 +558,8 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_inclusive_scan_clusters_ir3: {
       nir_op op = nir_intrinsic_reduction_op(instr);
       is_divergent = src_divergent(instr->src[0], state) ||
-                     state->vertex_divergence;
+                     (state->options & nir_divergence_vertex) ||
+                     (state->options & nir_divergence_across_subgroups);
       if (op != nir_op_umin && op != nir_op_imin && op != nir_op_fmin &&
           op != nir_op_umax && op != nir_op_imax && op != nir_op_fmax &&
           op != nir_op_iand && op != nir_op_ior)
@@ -556,7 +571,8 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       /* This reduces the last invocations in all 8-wide clusters. It should
        * behave the same as reduce with cluster_size == subgroup_size.
        */
-      is_divergent = state->vertex_divergence;
+      is_divergent = (state->options & nir_divergence_vertex) ||
+                     (state->options & nir_divergence_across_subgroups);
       break;
 
    case nir_intrinsic_load_ubo:
@@ -656,6 +672,13 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       is_divergent = src_divergent(instr->src[0], state);
       break;
 
+   case nir_intrinsic_quad_swizzle_amd:
+   case nir_intrinsic_masked_swizzle_amd:
+      /* Without fetch inactive, reads for inactive lanes have to return 0. */
+      is_divergent = !nir_intrinsic_fetch_inactive(instr) ||
+                     src_divergent(instr->src[0], state);
+      break;
+
    /* Intrinsics with divergence depending on sources */
    case nir_intrinsic_convert_alu_types:
    case nir_intrinsic_ddx:
@@ -712,13 +735,12 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_input_attachment_conv_pan:
    case nir_intrinsic_atomic_counter_read:
    case nir_intrinsic_atomic_counter_read_deref:
-   case nir_intrinsic_quad_swizzle_amd:
-   case nir_intrinsic_masked_swizzle_amd:
    case nir_intrinsic_is_sparse_texels_resident:
    case nir_intrinsic_is_sparse_resident_zink:
    case nir_intrinsic_sparse_residency_code_and:
    case nir_intrinsic_bvh64_intersect_ray_amd:
    case nir_intrinsic_bvh8_intersect_ray_amd:
+   case nir_intrinsic_load_sample_positions_amd:
    case nir_intrinsic_image_deref_load_param_intel:
    case nir_intrinsic_image_load_raw_intel:
    case nir_intrinsic_get_ubo_size:
@@ -729,7 +751,11 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_reg:
    case nir_intrinsic_load_constant_agx:
    case nir_intrinsic_load_texture_handle_agx:
+   case nir_intrinsic_load_from_texture_handle_agx:
+   case nir_intrinsic_load_vbo_base_agx:
+   case nir_intrinsic_load_attrib_clamp_agx:
    case nir_intrinsic_bindless_image_agx:
+   case nir_intrinsic_bindless_sampler_agx:
    case nir_intrinsic_load_reg_indirect:
    case nir_intrinsic_load_const_ir3:
    case nir_intrinsic_load_frag_size_ir3:
@@ -751,9 +777,13 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       /* Not having the non_uniform flag with divergent sources is undefined
        * behavior. The Intel driver defines it pick the lowest numbered live
        * SIMD lane (via emit_uniformize).
+       *
+       * When gather the divergence across subgroups, we need propagate the
+       * divergence from the sources.
        */
       if ((nir_intrinsic_resource_access_intel(instr) &
-           nir_resource_intel_non_uniform) != 0) {
+           nir_resource_intel_non_uniform) != 0 ||
+          (state->options & nir_divergence_across_subgroups)) {
          unsigned num_srcs = nir_intrinsic_infos[instr->intrinsic].num_srcs;
          for (unsigned i = 0; i < num_srcs; i++) {
             if (src_divergent(instr->src[i], state)) {
@@ -779,7 +809,6 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_color0:
    case nir_intrinsic_load_color1:
    case nir_intrinsic_load_sample_id:
-   case nir_intrinsic_load_sample_id_no_per_sample:
    case nir_intrinsic_load_sample_mask_in:
    case nir_intrinsic_load_interpolated_input:
    case nir_intrinsic_load_point_coord_maybe_flipped:
@@ -816,6 +845,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_sample_pos_or_center:
    case nir_intrinsic_load_vertex_id_zero_base:
    case nir_intrinsic_load_vertex_id:
+   case nir_intrinsic_load_raw_vertex_id_pan:
    case nir_intrinsic_load_invocation_id:
    case nir_intrinsic_load_local_invocation_id:
    case nir_intrinsic_load_local_invocation_index:
@@ -855,6 +885,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_global_atomic_swap_agx:
    case nir_intrinsic_global_atomic_2x32:
    case nir_intrinsic_global_atomic_swap_2x32:
+   case nir_intrinsic_global_atomic_pco:
    case nir_intrinsic_atomic_counter_add:
    case nir_intrinsic_atomic_counter_min:
    case nir_intrinsic_atomic_counter_max:
@@ -962,6 +993,7 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
    case nir_intrinsic_load_shared_lock_nv:
    case nir_intrinsic_store_shared_unlock_nv:
    case nir_intrinsic_bvh_stack_rtn_amd:
+   case nir_intrinsic_cmat_load_shared_nv:
       is_divergent = true;
       break;
 
@@ -971,10 +1003,18 @@ visit_intrinsic(nir_intrinsic_instr *instr, struct divergence_state *state)
       break;
 #else
       nir_print_instr(&instr->instr, stderr);
-      unreachable("\nNIR divergence analysis: Unhandled intrinsic.");
+      UNREACHABLE("\nNIR divergence analysis: Unhandled intrinsic.");
 #endif
    }
 
+   if (nir_intrinsic_has_access(instr) &&
+       (nir_intrinsic_access(instr) & ACCESS_SKIP_HELPERS))
+      is_divergent = true;
+
+   if (nir_intrinsic_has_access(instr) &&
+       (nir_intrinsic_access(instr) & ACCESS_SMEM_AMD))
+      is_divergent = false;
+
    instr->def.divergent = is_divergent;
    return is_divergent;
 }
@@ -1065,7 +1105,7 @@ nir_variable_is_uniform(nir_shader *shader, nir_variable *var,
    }
 
    nir_divergence_options options = state->options;
-   gl_shader_stage stage = shader->info.stage;
+   mesa_shader_stage stage = shader->info.stage;
 
    if (stage == MESA_SHADER_FRAGMENT &&
        (options & nir_divergence_single_prim_per_subgroup) &&
@@ -1136,11 +1176,11 @@ visit_jump(nir_jump_instr *jump, struct divergence_state *state)
       /* This totally kills invocations so it doesn't add divergence */
       break;
    case nir_jump_return:
-      unreachable("NIR divergence analysis: Unsupported return instruction.");
+      UNREACHABLE("NIR divergence analysis: Unsupported return instruction.");
       break;
    case nir_jump_goto:
    case nir_jump_goto_if:
-      unreachable("NIR divergence analysis: Unsupported goto_if instruction.");
+      UNREACHABLE("NIR divergence analysis: Unsupported goto_if instruction.");
       break;
    }
    return false;
@@ -1178,7 +1218,7 @@ instr_is_loop_invariant(nir_instr *instr, struct divergence_state *state)
    case nir_instr_type_phi:
    case nir_instr_type_parallel_copy:
    default:
-      unreachable("NIR divergence analysis: Unsupported instruction type.");
+      UNREACHABLE("NIR divergence analysis: Unsupported instruction type.");
    }
 }
 
@@ -1204,7 +1244,7 @@ update_instr_divergence(nir_instr *instr, struct divergence_state *state)
    case nir_instr_type_phi:
    case nir_instr_type_parallel_copy:
    default:
-      unreachable("NIR divergence analysis: Unsupported instruction type.");
+      UNREACHABLE("NIR divergence analysis: Unsupported instruction type.");
    }
 }
 
@@ -1424,7 +1464,7 @@ visit_loop(nir_loop *loop, struct divergence_state *state)
    /* setup loop state */
    struct divergence_state loop_state = *state;
    loop_state.loop = loop;
-   loop_state.loop_all_invariant = loop_header->predecessors->entries == 1;
+   loop_state.loop_all_invariant = loop_header->predecessors.entries == 1;
    loop_state.divergent_loop_cf = false;
    loop_state.divergent_loop_continue = false;
    loop_state.divergent_loop_break = false;
@@ -1479,7 +1519,7 @@ visit_cf_list(struct exec_list *list, struct divergence_state *state)
          has_changed |= visit_loop(nir_cf_node_as_loop(node), state);
          break;
       case nir_cf_node_function:
-         unreachable("NIR divergence analysis: Unsupported cf_node type.");
+         UNREACHABLE("NIR divergence analysis: Unsupported cf_node type.");
       }
    }
 
@@ -1527,21 +1567,11 @@ nir_divergence_analysis(nir_shader *shader)
 void
 nir_vertex_divergence_analysis(nir_shader *shader)
 {
-   struct divergence_state state = {
-      .stage = shader->info.stage,
-      .shader = shader,
-      .options = shader->options->divergence_analysis_options,
-      .loop = NULL,
-      .loop_all_invariant = false,
-      .vertex_divergence = true,
-      .first_visit = true,
-   };
+   nir_divergence_options options =
+      shader->options->divergence_analysis_options | nir_divergence_vertex;
 
    nir_foreach_function_impl(impl, shader) {
-      nir_metadata_require(impl, nir_metadata_block_index);
-      state.impl = impl;
-      visit_cf_list(&impl->body, &state);
-      nir_progress(true, impl, nir_metadata_all & ~nir_metadata_divergence);
+      nir_divergence_analysis_impl(impl, options);
    }
 }
 
