@@ -118,7 +118,7 @@ get_loop_instr_count(struct exec_list *cf_list)
          break;
       }
       default:
-         unreachable("Invalid CF node type");
+         UNREACHABLE("Invalid CF node type");
       }
    }
 
@@ -157,7 +157,7 @@ gcm_build_block_info(struct exec_list *cf_list, struct gcm_state *state,
          break;
       }
       default:
-         unreachable("Invalid CF node type");
+         UNREACHABLE("Invalid CF node type");
       }
    }
 }
@@ -212,6 +212,7 @@ is_src_scalarizable(nir_src *src)
       case nir_intrinsic_load_global_constant:
       case nir_intrinsic_load_input:
       case nir_intrinsic_load_per_primitive_input:
+      case nir_intrinsic_load_ssbo_intel:
          return true;
       default:
          break;
@@ -262,6 +263,7 @@ pin_intrinsic(nir_intrinsic_instr *intrin)
    if (!non_uniform &&
        (intrin->intrinsic == nir_intrinsic_load_ubo ||
         intrin->intrinsic == nir_intrinsic_load_ssbo ||
+        intrin->intrinsic == nir_intrinsic_load_ssbo_intel ||
         intrin->intrinsic == nir_intrinsic_get_ubo_size ||
         intrin->intrinsic == nir_intrinsic_get_ssbo_size ||
         nir_intrinsic_has_image_dim(intrin) ||
@@ -376,7 +378,7 @@ gcm_pin_instructions(nir_function_impl *impl, struct gcm_state *state)
             break;
 
          default:
-            unreachable("Invalid instruction type in GCM");
+            UNREACHABLE("Invalid instruction type in GCM");
          }
 
          if (!(instr->pass_flags & GCM_INSTR_PLACED)) {
@@ -666,9 +668,9 @@ gcm_schedule_late_def(nir_def *def, void *void_state)
    }
 
    if (def->parent_instr->pass_flags & GCM_INSTR_SCHEDULE_EARLIER_ONLY &&
-       lca != def->parent_instr->block &&
-       nir_block_dominates(def->parent_instr->block, lca)) {
-      lca = def->parent_instr->block;
+       lca != nir_def_block(def) &&
+       nir_block_dominates(nir_def_block(def), lca)) {
+      lca = nir_def_block(def);
    }
 
    /* We now have the LCA of all of the uses.  If our invariants hold,
@@ -679,7 +681,7 @@ gcm_schedule_late_def(nir_def *def, void *void_state)
    nir_block *best_block =
       gcm_choose_block_for_instr(def->parent_instr, early_block, lca, state);
 
-   if (def->parent_instr->block != best_block)
+   if (nir_def_block(def) != best_block)
       state->progress = true;
 
    def->parent_instr->block = best_block;
@@ -758,6 +760,7 @@ gcm_place_instr(nir_instr *instr, struct gcm_state *state)
    if (instr->block == NULL) {
       nir_foreach_def(instr, gcm_replace_def_with_undef, state);
       nir_instr_remove(instr);
+      state->progress = true;
       return;
    }
 
@@ -794,7 +797,9 @@ weak_gvn(const nir_instr *a, const nir_instr *b)
 static bool
 opt_gcm_impl(nir_shader *shader, nir_function_impl *impl, bool value_number)
 {
-   nir_metadata_require(impl, nir_metadata_control_flow);
+   nir_metadata_require(impl, nir_metadata_block_index |
+                              nir_metadata_dominance |
+                              nir_metadata_dominance_lca);
    nir_metadata_require(impl, nir_metadata_loop_analysis,
                         shader->options->force_indirect_unrolling,
                         shader->options->force_indirect_unrolling_sampler);
@@ -829,18 +834,20 @@ opt_gcm_impl(nir_shader *shader, nir_function_impl *impl, bool value_number)
     * on both sides of the same if/else block, we allow them to be moved.
     * This cleans up a lot of mess without being -too- aggressive.
     */
-   struct set *gvn_set = nir_instr_set_create(NULL);
+   struct set gvn_set;
+   nir_instr_set_init(&gvn_set, NULL);
+
    foreach_list_typed_safe(nir_instr, instr, node, &state.instrs) {
       if (instr->pass_flags & GCM_INSTR_PINNED)
          continue;
 
-      if (nir_instr_set_add_or_rewrite(gvn_set, instr,
+      if (nir_instr_set_add_or_rewrite(&gvn_set, instr,
                                        value_number ? NULL : weak_gvn)) {
          state.progress = true;
          nir_instr_remove(instr);
       }
    }
-   nir_instr_set_destroy(gvn_set);
+   nir_instr_set_fini(&gvn_set);
 
    foreach_list_typed(nir_instr, instr, node, &state.instrs)
       gcm_schedule_early_instr(instr, &state);
@@ -858,10 +865,10 @@ opt_gcm_impl(nir_shader *shader, nir_function_impl *impl, bool value_number)
    ralloc_free(state.instr_infos);
 
    if (state.progress) {
-      nir_metadata_preserve(impl, nir_metadata_control_flow);
+      nir_progress(true, impl, nir_metadata_control_flow);
    } else {
-      nir_metadata_preserve(impl, nir_metadata_control_flow |
-                                     nir_metadata_loop_analysis);
+      nir_progress(true, impl,
+                   nir_metadata_control_flow | nir_metadata_loop_analysis);
    }
 
    return state.progress;
