@@ -371,7 +371,7 @@ TEST_UNI_EXPR_PROP(MESH, FRAGMENT, CLIP_DIST0, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, CLIP_DIST1, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, CULL_DIST0, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, CULL_DIST1, 0, float, 32)
-TEST_UNI_EXPR_KEPT(MESH, FRAGMENT, LAYER, 0, float, 32) /* RADV mesh multiview workaround */
+TEST_UNI_EXPR_PROP(MESH, FRAGMENT, LAYER, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, VIEWPORT, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, VAR0, 0, float, 32)
 TEST_UNI_EXPR_PROP(MESH, FRAGMENT, VAR0, 0, float, 16)
