@@ -2,8 +2,10 @@
 // SPDX-License-Identifier: MIT
 
 use crate::bitset::BitSet;
+use crate::depth_first_search::{dfs, DepthFirstSearch};
 use std::collections::HashMap;
 use std::hash::{BuildHasher, Hash};
+use std::iter::{Cloned, Rev};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 use std::slice;
 
@@ -32,49 +34,61 @@ impl<N> DerefMut for CFGNode<N> {
     }
 }
 
-fn graph_post_dfs<N>(
-    nodes: &Vec<CFGNode<N>>,
-    id: usize,
-    seen: &mut BitSet,
-    post_idx: &mut Vec<usize>,
-    count: &mut usize,
-) {
-    if seen.contains(id) {
-        return;
-    }
-    seen.insert(id);
+struct PostOrderSort {
+    post_idx: Vec<usize>,
+    count: usize,
+}
 
-    // Reverse the order of the successors so that any successors which are
-    // forward edges get descending indices.  This ensures that, in the reverse
-    // post order, successors (and their dominated children) come in-order.
-    // In particular, as long as fall-through edges are only ever used for
-    // forward edges and the fall-through edge comes first, we guarantee that
-    // the fallthrough block comes immediately after its predecessor.
-    for s in nodes[id].succ.iter().rev() {
-        graph_post_dfs(nodes, *s, seen, post_idx, count);
+struct PostOrderSortDFS<'a, N> {
+    nodes: &'a [CFGNode<N>],
+    sort: PostOrderSort,
+}
+
+impl<'a, N> DepthFirstSearch for PostOrderSortDFS<'a, N> {
+    type ChildIter = Cloned<Rev<std::slice::Iter<'a, usize>>>;
+
+    fn pre(&mut self, id: usize) -> Self::ChildIter {
+        // Reverse the order of the successors so that any successors which are
+        // forward edges get descending indices.  This ensures that, in the
+        // reverse post order, successors (and their dominated children) come
+        // in-order.  In particular, as long as fall-through edges are only ever
+        // used for forward edges and the fall-through edge comes first, we
+        // guarantee that the fallthrough block comes immediately after its
+        // predecessor.
+        self.nodes[id].succ.iter().rev().cloned()
     }
 
-    post_idx[id] = *count;
-    *count += 1;
+    fn post(&mut self, id: usize) {
+        self.sort.post_idx[id] = self.sort.count;
+        self.sort.count += 1;
+    }
+}
+
+impl PostOrderSort {
+    fn new<N>(nodes: &[CFGNode<N>]) -> Self {
+        let mut post_idx: Vec<usize> = Vec::new();
+        post_idx.resize(nodes.len(), usize::MAX);
+
+        let mut sort_dfs = PostOrderSortDFS {
+            nodes,
+            sort: PostOrderSort { post_idx, count: 0 },
+        };
+        dfs(&mut sort_dfs, 0);
+
+        sort_dfs.sort
+    }
 }
 
 fn rev_post_order_sort<N>(nodes: &mut Vec<CFGNode<N>>) {
-    let mut seen = BitSet::new();
-    let mut post_idx = Vec::new();
-    post_idx.resize(nodes.len(), usize::MAX);
-    let mut count = 0;
-
-    graph_post_dfs(nodes, 0, &mut seen, &mut post_idx, &mut count);
-
-    assert!(count <= nodes.len());
+    let sort = PostOrderSort::new(nodes);
 
     let remap_idx = |i: usize| {
-        let pid = post_idx[i];
+        let pid = sort.post_idx[i];
         if pid == usize::MAX {
             None
         } else {
-            assert!(pid < count);
-            Some((count - 1) - pid)
+            assert!(pid < sort.count);
+            Some((sort.count - 1) - pid)
         }
     };
     assert!(remap_idx(0) == Some(0));
@@ -95,19 +109,19 @@ fn rev_post_order_sort<N>(nodes: &mut Vec<CFGNode<N>>) {
 
     // We know a priori that each non-MAX post_idx is unique so we can sort the
     // nodes by inserting them into a new array by index.
-    let mut sorted: Vec<CFGNode<N>> = Vec::with_capacity(count);
+    let mut sorted: Vec<CFGNode<N>> = Vec::with_capacity(sort.count);
     for (i, n) in nodes.drain(..).enumerate() {
         if let Some(r) = remap_idx(i) {
             unsafe { sorted.as_mut_ptr().add(r).write(n) };
         }
     }
-    unsafe { sorted.set_len(count) };
+    unsafe { sorted.set_len(sort.count) };
 
     std::mem::swap(nodes, &mut sorted);
 }
 
 fn find_common_dom<N>(
-    nodes: &Vec<CFGNode<N>>,
+    nodes: &[CFGNode<N>],
     mut a: usize,
     mut b: usize,
 ) -> usize {
@@ -122,21 +136,26 @@ fn find_common_dom<N>(
     a
 }
 
-fn dom_idx_dfs<N>(
-    nodes: &mut Vec<CFGNode<N>>,
-    dom_children: &Vec<Vec<usize>>,
-    id: usize,
-    count: &mut usize,
-) {
-    nodes[id].dom_pre_idx = *count;
-    *count += 1;
+struct DominanceDFS<'a, N> {
+    nodes: &'a mut [CFGNode<N>],
+    dom_children: &'a [Vec<usize>],
+    count: usize,
+}
 
-    for c in dom_children[id].iter() {
-        dom_idx_dfs(nodes, dom_children, *c, count);
+impl<'a, N> DepthFirstSearch for DominanceDFS<'a, N> {
+    type ChildIter = Cloned<std::slice::Iter<'a, usize>>;
+
+    fn pre(&mut self, id: usize) -> Self::ChildIter {
+        self.nodes[id].dom_pre_idx = self.count;
+        self.count += 1;
+
+        self.dom_children[id].iter().cloned()
     }
 
-    nodes[id].dom_post_idx = *count;
-    *count += 1;
+    fn post(&mut self, id: usize) {
+        self.nodes[id].dom_post_idx = self.count;
+        self.count += 1;
+    }
 }
 
 fn calc_dominance<N>(nodes: &mut Vec<CFGNode<N>>) {
@@ -172,61 +191,90 @@ fn calc_dominance<N>(nodes: &mut Vec<CFGNode<N>>) {
         }
     }
 
-    let mut count = 0_usize;
-    dom_idx_dfs(nodes, &dom_children, 0, &mut count);
-    debug_assert!(count == nodes.len() * 2);
+    let mut dom_dfs = DominanceDFS {
+        nodes,
+        dom_children: &dom_children,
+        count: 0,
+    };
+    dfs(&mut dom_dfs, 0);
+    debug_assert!(dom_dfs.count == nodes.len() * 2);
 }
 
-fn loop_detect_dfs<N>(
-    nodes: &Vec<CFGNode<N>>,
-    id: usize,
-    pre: &mut BitSet,
-    post: &mut BitSet,
-    back_edges: &mut Vec<(usize, usize)>,
-) {
-    if pre.contains(id) {
-        return;
+struct BackEdgesDFS<'a, N> {
+    nodes: &'a [CFGNode<N>],
+    pre: BitSet,
+    post: BitSet,
+    back_edges: Vec<(usize, usize)>,
+}
+
+impl<'a, N> DepthFirstSearch for BackEdgesDFS<'a, N> {
+    type ChildIter = Cloned<std::slice::Iter<'a, usize>>;
+
+    fn pre(&mut self, id: usize) -> Self::ChildIter {
+        self.pre.insert(id);
+
+        self.nodes[id].succ.iter().cloned()
     }
 
-    pre.insert(id);
-
-    for &s in nodes[id].succ.iter() {
-        if pre.contains(s) && !post.contains(s) {
-            back_edges.push((id, s));
+    fn edge(&mut self, parent: usize, child: usize) {
+        if self.pre.contains(child) && !self.post.contains(child) {
+            self.back_edges.push((parent, child));
         }
-        loop_detect_dfs(nodes, s, pre, post, back_edges);
     }
 
-    post.insert(id);
+    fn post(&mut self, id: usize) {
+        self.post.insert(id);
+    }
 }
 
-/// Computes the set of nodes that reach the given node without going through
-/// stop
-fn reaches_dfs<N>(
-    nodes: &Vec<CFGNode<N>>,
-    id: usize,
+fn find_back_edges<N>(nodes: &[CFGNode<N>]) -> Vec<(usize, usize)> {
+    let mut be_dfs = BackEdgesDFS {
+        nodes,
+        pre: Default::default(),
+        post: Default::default(),
+        back_edges: Default::default(),
+    };
+    dfs(&mut be_dfs, 0);
+    be_dfs.back_edges
+}
+
+struct ReachesDFS<'a, N> {
+    nodes: &'a [CFGNode<N>],
     stop: usize,
-    reaches: &mut BitSet,
-) {
-    if id == stop || reaches.contains(id) {
-        return;
-    }
+    reaches: BitSet,
+}
 
-    reaches.insert(id);
+impl<'a, N> DepthFirstSearch for ReachesDFS<'a, N> {
+    type ChildIter = Cloned<std::slice::Iter<'a, usize>>;
 
-    // Since we're trying to find the set of things that reach the start node,
-    // not the set of things reachable from the start node, walk predecessors.
-    for &s in nodes[id].pred.iter() {
-        reaches_dfs(nodes, s, stop, reaches);
+    fn pre(&mut self, id: usize) -> Self::ChildIter {
+        if id == self.stop || self.reaches.contains(id) {
+            return (&[]).iter().cloned();
+        }
+
+        self.reaches.insert(id);
+
+        // Since we're trying to find the set of things that reach the start
+        // node, not the set of things reachable from the start node, walk
+        // predecessors.
+        self.nodes[id].pred.iter().cloned()
     }
 }
 
+/// Computes the set of nodes that reach the given edge without going through
+/// the edge
+fn reaches<N>(nodes: &Vec<CFGNode<N>>, edge: (usize, usize)) -> BitSet {
+    let mut r_dfs = ReachesDFS {
+        nodes,
+        stop: edge.1,
+        reaches: Default::default(),
+    };
+    dfs(&mut r_dfs, edge.0);
+    r_dfs.reaches
+}
+
 fn detect_loops<N>(nodes: &mut Vec<CFGNode<N>>) -> bool {
-    let mut dfs_pre = BitSet::new();
-    let mut dfs_post = BitSet::new();
-    let mut back_edges = Vec::new();
-    loop_detect_dfs(nodes, 0, &mut dfs_pre, &mut dfs_post, &mut back_edges);
-
+    let back_edges = find_back_edges(nodes);
     if back_edges.is_empty() {
         return false;
     }
@@ -242,12 +290,7 @@ fn detect_loops<N>(nodes: &mut Vec<CFGNode<N>>) -> bool {
         // Stash the loop headers while we're here
         loops.insert(h);
 
-        // re-use dfs_pre for our reaches set
-        dfs_pre.clear();
-        let reaches = &mut dfs_pre;
-        reaches_dfs(nodes, c, h, reaches);
-
-        for n in reaches.iter() {
+        for n in reaches(nodes, (c, h)).iter() {
             node_loops[n].insert(h);
         }
     }
@@ -408,6 +451,26 @@ impl<N> CFG<N> {
         }
     }
 
+    /// Returns the loop depth of the given node.  Nodes not in any loops have
+    /// a loop depth of zero.  For nodes inside a loop, this is the count of
+    /// number of loop headers above them in the dominance tree.
+    pub fn loop_depth(&self, idx: usize) -> usize {
+        let mut idx = idx;
+        let mut depth = 0;
+        loop {
+            let lph = self.nodes[idx].lph;
+            if lph == usize::MAX {
+                return depth;
+            }
+
+            depth += 1;
+
+            // Loop headers have themselves as the lph so we need to skip to
+            // the dominator of the loop header for the next iteration.
+            idx = self.nodes[lph].dom;
+        }
+    }
+
     /// Returns the indices of the successors of this node in the CFG.
     pub fn succ_indices(&self, idx: usize) -> &[usize] {
         &self.nodes[idx].succ[..]
