@@ -5,13 +5,14 @@
 /* alphabetical order */
 EGL_ENTRYPOINT(eglBindAPI)
 EGL_ENTRYPOINT(eglBindTexImage)
+#ifdef HAVE_BIND_WL_DISPLAY
 EGL_ENTRYPOINT(eglBindWaylandDisplayWL)
+#endif
 EGL_ENTRYPOINT(eglChooseConfig)
 EGL_ENTRYPOINT(eglClientWaitSync)
 EGL_ENTRYPOINT(eglClientWaitSyncKHR)
 EGL_ENTRYPOINT(eglCopyBuffers)
 EGL_ENTRYPOINT(eglCreateContext)
-EGL_ENTRYPOINT(eglCreateDRMImageMESA)
 EGL_ENTRYPOINT(eglCreateImage)
 EGL_ENTRYPOINT(eglCreateImageKHR)
 EGL_ENTRYPOINT(eglCreatePbufferFromClientBuffer)
@@ -24,7 +25,9 @@ EGL_ENTRYPOINT(eglCreatePlatformWindowSurfaceEXT)
 EGL_ENTRYPOINT(eglCreateSync)
 EGL_ENTRYPOINT(eglCreateSync64KHR)
 EGL_ENTRYPOINT(eglCreateSyncKHR)
+#ifdef HAVE_BIND_WL_DISPLAY
 EGL_ENTRYPOINT(eglCreateWaylandBufferFromImageWL)
+#endif
 EGL_ENTRYPOINT(eglCreateWindowSurface)
 EGL_ENTRYPOINT(eglDebugMessageControlKHR)
 EGL_ENTRYPOINT(eglDestroyContext)
@@ -36,7 +39,6 @@ EGL_ENTRYPOINT(eglDestroySyncKHR)
 EGL_ENTRYPOINT(eglDupNativeFenceFDANDROID)
 EGL_ENTRYPOINT(eglExportDMABUFImageMESA)
 EGL_ENTRYPOINT(eglExportDMABUFImageQueryMESA)
-EGL_ENTRYPOINT(eglExportDRMImageMESA)
 EGL_ENTRYPOINT2(eglGLInteropExportObjectMESA, MesaGLInteropEGLExportObject)
 EGL_ENTRYPOINT2(eglGLInteropFlushObjectsMESA, MesaGLInteropEGLFlushObjects)
 EGL_ENTRYPOINT2(eglGLInteropQueryDeviceInfoMESA, MesaGLInteropEGLQueryDeviceInfo)
@@ -59,11 +61,11 @@ EGL_ENTRYPOINT(eglGetSyncValuesCHROMIUM)
 EGL_ENTRYPOINT(eglInitialize)
 EGL_ENTRYPOINT(eglLabelObjectKHR)
 EGL_ENTRYPOINT(eglMakeCurrent)
-EGL_ENTRYPOINT(eglPostSubBufferNV)
 EGL_ENTRYPOINT(eglQueryAPI)
 EGL_ENTRYPOINT(eglQueryContext)
 EGL_ENTRYPOINT(eglQueryDebugKHR)
 EGL_ENTRYPOINT(eglQueryDeviceAttribEXT)
+EGL_ENTRYPOINT(eglQueryDeviceBinaryEXT)
 EGL_ENTRYPOINT(eglQueryDeviceStringEXT)
 EGL_ENTRYPOINT(eglQueryDevicesEXT)
 EGL_ENTRYPOINT(eglQueryDisplayAttribEXT)
@@ -72,7 +74,9 @@ EGL_ENTRYPOINT(eglQueryDmaBufModifiersEXT)
 EGL_ENTRYPOINT(eglQueryString)
 EGL_ENTRYPOINT(eglQuerySupportedCompressionRatesEXT)
 EGL_ENTRYPOINT(eglQuerySurface)
+#ifdef HAVE_BIND_WL_DISPLAY
 EGL_ENTRYPOINT(eglQueryWaylandBufferWL)
+#endif
 EGL_ENTRYPOINT(eglReleaseTexImage)
 EGL_ENTRYPOINT(eglReleaseThread)
 EGL_ENTRYPOINT(eglSetBlobCacheFuncsANDROID)
@@ -80,12 +84,13 @@ EGL_ENTRYPOINT(eglSetDamageRegionKHR)
 EGL_ENTRYPOINT(eglSignalSyncKHR)
 EGL_ENTRYPOINT(eglSurfaceAttrib)
 EGL_ENTRYPOINT(eglSwapBuffers)
-EGL_ENTRYPOINT(eglSwapBuffersRegionNOK)
 EGL_ENTRYPOINT(eglSwapBuffersWithDamageEXT)
 EGL_ENTRYPOINT(eglSwapBuffersWithDamageKHR)
 EGL_ENTRYPOINT(eglSwapInterval)
 EGL_ENTRYPOINT(eglTerminate)
+#ifdef HAVE_BIND_WL_DISPLAY
 EGL_ENTRYPOINT(eglUnbindWaylandDisplayWL)
+#endif
 EGL_ENTRYPOINT(eglWaitClient)
 EGL_ENTRYPOINT(eglWaitGL)
 EGL_ENTRYPOINT(eglWaitNative)
