@@ -34,10 +34,10 @@ etna_query_feature_db(struct etna_core_info *info)
    if (!db)
       return false;
 
+   etna_core_enable_feature(info, ETNA_FEATURE_CORE_GPU);
+
    if (db->NNCoreCount)
-      info->type = ETNA_CORE_NPU;
-   else
-      info->type = ETNA_CORE_GPU;
+      etna_core_enable_feature(info, ETNA_FEATURE_CORE_NPU);
 
    /* Features: */
    ETNA_FEATURE(REG_FastClear, FAST_CLEAR);
@@ -109,8 +109,12 @@ etna_query_feature_db(struct etna_core_info *info)
    ETNA_FEATURE(VIP_V7, VIP_V7);
    ETNA_FEATURE(NN_XYDP0, NN_XYDP0);
 
+   ETNA_FEATURE(MSAA_FRAGMENT_OPERATION, MSAA_FRAGMENT_OPERATION);
+   ETNA_FEATURE(REG_RSS8, S8);
+   ETNA_FEATURE(HWTFB, HWTFB);
+
    /* Limits: */
-   if (info->type == ETNA_CORE_GPU) {
+   if (etna_core_has_feature(info, ETNA_FEATURE_CORE_GPU)) {
       info->gpu.max_instructions = db->InstructionCount;
       info->gpu.vertex_output_buffer_size = db->VertexOutputBufferSize;
       info->gpu.vertex_cache_size = db->VertexCacheSize;
@@ -120,7 +124,9 @@ etna_query_feature_db(struct etna_core_info *info)
       info->gpu.pixel_pipes = db->NumPixelPipes;
       info->gpu.max_varyings = db->VaryingCount;
       info->gpu.num_constants = db->NumberOfConstants;
-   } else {
+   }
+
+   if (etna_core_has_feature(info, ETNA_FEATURE_CORE_NPU)) {
       info->npu.nn_core_count = db->NNCoreCount;
       info->npu.nn_mad_per_core = db->NNMadPerCore;
       info->npu.tp_core_count = db->TPEngine_CoreCount;
