@@ -23,6 +23,33 @@
  * src iterators work.
  */
 
+/* Return the number of cycles from the start of the instruction until src_n is
+ * read.
+ */
+unsigned
+ir3_src_read_delay(struct ir3_compiler *compiler, struct ir3_instruction *instr,
+                   unsigned src_n)
+{
+   /* gat and swz have scalar sources and each source is read in a subsequent
+    * cycle.
+    */
+   if (instr->opc == OPC_GAT || instr->opc == OPC_SWZ) {
+      return src_n;
+   }
+
+   /* cat3 instructions consume their last source one or two cycles later. Note
+    * that not all cat3 instructions seem to do this pre-a7xx.
+    */
+   bool cat3_reads_later = compiler->gen >= 7
+                              ? (opc_cat(instr->opc) == 3)
+                              : (is_mad(instr->opc) || is_madsh(instr->opc));
+   if (cat3_reads_later && src_n == 2) {
+      return compiler->delay_slots.cat3_src2_read;
+   }
+
+   return 0;
+}
+
 /* calculate required # of delay slots between the instruction that
  * assigns a value and the one that consumes
  */
@@ -46,7 +73,7 @@ ir3_delayslots(struct ir3_compiler *compiler,
       return 0;
 
    if (writes_addr0(assigner) || writes_addr1(assigner))
-      return 6;
+      return compiler->delay_slots.non_alu;
 
    if (soft && needs_ss(compiler, assigner, consumer))
       return soft_ss_delay(assigner);
@@ -76,7 +103,7 @@ ir3_delayslots(struct ir3_compiler *compiler,
    /* assigner must be alu: */
    if (is_flow(consumer) || is_sfu(consumer) || is_tex(consumer) ||
        is_mem(consumer)) {
-      return 6;
+      return compiler->delay_slots.non_alu;
    } else {
       /* In mergedregs mode, there is an extra 2-cycle penalty when half of
        * a full-reg is read as a half-reg or when a half-reg is read as a
@@ -85,12 +112,8 @@ ir3_delayslots(struct ir3_compiler *compiler,
       bool mismatched_half = (assigner->dsts[0]->flags & IR3_REG_HALF) !=
                              (consumer->srcs[n]->flags & IR3_REG_HALF);
       unsigned penalty = mismatched_half ? 3 : 0;
-      if ((is_mad(consumer->opc) || is_madsh(consumer->opc)) && (n == 2)) {
-         /* special case, 3rd src to cat3 not required on first cycle */
-         return 1 + penalty;
-      } else {
-         return 3 + penalty;
-      }
+      return compiler->delay_slots.alu_to_alu + penalty -
+             ir3_src_read_delay(compiler, consumer, n);
    }
 }
 
