@@ -30,7 +30,7 @@
 struct ir3_legalize_ctx {
    struct ir3_compiler *compiler;
    struct ir3_shader_variant *so;
-   gl_shader_stage type;
+   mesa_shader_stage type;
    int max_bary;
    bool early_input_release;
    bool has_inputs;
@@ -105,6 +105,9 @@ ir3_required_sync_flags(struct ir3_legalize_state *state,
       else
          reg = n->srcs[i - n->dsts_count];
 
+      if (reg->flags & IR3_REG_DUMMY)
+         continue;
+
       if (is_reg_gpr(reg)) {
 
          /* TODO: we probably only need (ss) for alu
@@ -152,7 +155,7 @@ ir3_required_sync_flags(struct ir3_legalize_state *state,
          if (state->needs_sy_for_const) {
             flags |= IR3_INSTR_SY;
          }
-      } else if (reg_is_addr1(reg) && n->block->in_early_preamble) {
+      } else if (!(reg->flags & (IR3_REG_IMMED | IR3_REG_RT))) {
          if (regmask_get(&state->needs_ss, reg)) {
             flags |= IR3_INSTR_SS;
          }
@@ -160,7 +163,7 @@ ir3_required_sync_flags(struct ir3_legalize_state *state,
    }
 
    foreach_dst (reg, n) {
-      if (reg->flags & IR3_REG_RT)
+      if (reg->flags & (IR3_REG_RT | IR3_REG_DUMMY))
          continue;
       if (needs_sy_war(state, reg)) {
          flags |= IR3_INSTR_SY;
@@ -232,12 +235,14 @@ sync_update(struct ir3_legalize_state *state, struct ir3_compiler *compiler,
          }
       } else if (reg_is_addr1(dst) && n->block->in_early_preamble) {
          regmask_set(&state->needs_ss, dst);
+      } else if (dst->flags & IR3_REG_UNIFORM) {
+         regmask_set(&state->needs_ss, dst);
       }
    }
 
-   if (is_tex_or_prefetch(n) && n->dsts_count > 0) {
+   if (is_tex_or_prefetch(n) && !has_dummy_dst(n)) {
       regmask_set(&state->needs_sy, n->dsts[0]);
-   } else if (n->opc == OPC_RESINFO && n->dsts_count > 0) {
+   } else if (n->opc == OPC_RESINFO && !has_dummy_dst(n)) {
       regmask_set(&state->needs_ss, n->dsts[0]);
    } else if (is_load(n)) {
       if (is_local_mem_load(n))
@@ -424,7 +429,7 @@ ir3_merge_pred_legalize_states(struct ir3_legalize_state *state,
                         &pstate->needs_ss_or_sy_scalar_war);
    }
 
-   gl_shader_stage stage = block->shader->type;
+   mesa_shader_stage stage = block->shader->type;
 
    if (stage == MESA_SHADER_TESS_CTRL || stage == MESA_SHADER_GEOMETRY) {
       if (block == ir3_start_block(block->shader)) {
@@ -464,13 +469,18 @@ get_ready_slot(struct ir3_legalize_state *state,
       consumer_alu ? &state->alu_nop : &state->non_alu_nop;
    assert(!(reg->flags & IR3_REG_SHARED));
    if (reg->flags & IR3_REG_HALF) {
-      if (matching_size)
+      if (matching_size) {
+         assert(num < ARRAY_SIZE(nop->half_ready));
          return &nop->half_ready[num];
-      else
+      } else {
+         assert(num / 2 < ARRAY_SIZE(nop->full_ready));
          return &nop->full_ready[num / 2];
+      }
    } else {
-      if (matching_size)
+      if (matching_size) {
+         assert(num < ARRAY_SIZE(nop->full_ready));
          return &nop->full_ready[num];
+      }
       /* If "num" is large enough, then it can't alias a half-reg because only
        * the first half of the full reg speace aliases half regs. Return NULL in
        * this case.
@@ -525,7 +535,7 @@ delay_update(struct ir3_compiler *compiler,
       return;
 
    foreach_dst_n (dst, n, instr) {
-      if (dst->flags & IR3_REG_RT)
+      if (dst->flags & (IR3_REG_RT | IR3_REG_DUMMY))
          continue;
 
       unsigned elems = post_ra_reg_elems(dst);
@@ -545,7 +555,7 @@ delay_update(struct ir3_compiler *compiler,
       if (dst->flags & IR3_REG_RELATIV)
          dst_cycle += instr->repeat;
 
-      if (dst->flags & IR3_REG_SHARED)
+      if (dst->flags & (IR3_REG_SHARED | IR3_REG_UNIFORM))
          continue;
 
       for (unsigned elem = 0; elem < elems; elem++, num++) {
@@ -798,7 +808,7 @@ legalize_block(struct ir3_legalize_ctx *ctx, struct ir3_block *block)
          ctx->has_tex_prefetch = true;
       }
 
-      if (n->opc == OPC_RESINFO && n->dsts_count > 0) {
+      if (n->opc == OPC_RESINFO && !has_dummy_dst(n)) {
          ir3_update_legalize_state(state, ctx->compiler, n);
 
          n = ir3_NOP(&build);
@@ -920,25 +930,6 @@ apply_fine_deriv_macro(struct ir3_legalize_ctx *ctx, struct ir3_block *block)
    return true;
 }
 
-/* Some instructions can take a dummy destination of r63.x, which we model as it
- * not having a destination in the IR to avoid having special code to handle
- * this. Insert the dummy destination after everything else is done.
- */
-static bool
-expand_dummy_dests(struct ir3_block *block)
-{
-   foreach_instr (n, &block->instr_list) {
-      if ((n->opc == OPC_SAM || n->opc == OPC_LDC || n->opc == OPC_RESINFO) &&
-          n->dsts_count == 0) {
-         struct ir3_register *dst = ir3_dst_create(n, INVALID_REG, 0);
-         /* Copy the blob's writemask */
-         if (n->opc == OPC_SAM)
-            dst->wrmask = 0b1111;
-      }
-   }
-   return true;
-}
-
 static void
 apply_push_consts_load_macro(struct ir3_legalize_ctx *ctx,
                              struct ir3_block *block)
@@ -1239,7 +1230,7 @@ invert_branch(struct ir3_instruction *branch)
       branch->opc = OPC_BRAA;
       break;
    default:
-      unreachable("can't get here");
+      UNREACHABLE("can't get here");
    }
 
    branch->cat0.inv1 = !branch->cat0.inv1;
@@ -1884,6 +1875,257 @@ track_last(struct ir3_legalize_ctx *ctx, struct ir3 *ir,
    } while (progress);
 }
 
+/* An instruction where we may insert nops without increasing the runtime nop
+ * count. These are of the form `(rptN)nop` and `(nopN)foo`. We can insert nops
+ * after them while decreasing their rpt/nop count without affecting the runtime
+ * behavior.
+ */
+struct nop_insert_point {
+   struct ir3_instruction *instr;
+
+   /* The maximum number of nops that can be inserted after `instr` without
+    * pushing any subsequent alias sequences over the next cache line. Since
+    * inserting nops at an earlier insert point affects all later ones, the
+    * max_nops value of the last insert point is the maximum for all insert
+    * points. This is taken into account when iterating insert points so that we
+    * only have to update the last insert point whenever we encounter an alias
+    * sequence.
+    */
+   unsigned max_nops;
+};
+
+struct alias_align_ctx {
+   DECLARE_ARRAY(struct nop_insert_point, nop_insert_points);
+};
+
+#define ICACHE_LINE_INSTRS 16
+
+static unsigned
+icache_line(unsigned ip)
+{
+   return ip / ICACHE_LINE_INSTRS;
+}
+
+static uint8_t *
+extra_nops(struct ir3_instruction *instr)
+{
+   if (instr->opc == OPC_NOP) {
+      return &instr->repeat;
+   }
+
+   return &instr->nop;
+}
+
+static bool
+is_insert_point(struct ir3_instruction *instr)
+{
+   uint8_t *instr_extra_nops = extra_nops(instr);
+   return *instr_extra_nops != 0;
+}
+
+static void
+update_last_insert_point_max_nops(struct alias_align_ctx *ctx,
+                                  unsigned max_nops)
+{
+   if (max_nops == 0) {
+      /* If we cannot insert more nops at the last insert point, we cannot
+       * insert more anywhere.
+       */
+      ctx->nop_insert_points_count = 0;
+   } else if (ctx->nop_insert_points_count != 0) {
+      struct nop_insert_point *last_nop_insert_point =
+         &ctx->nop_insert_points[ctx->nop_insert_points_count - 1];
+      last_nop_insert_point->max_nops =
+         MIN2(last_nop_insert_point->max_nops, max_nops);
+   }
+}
+
+static void
+insert_nops(struct ir3_cursor cursor, unsigned n)
+{
+   struct ir3_builder b = ir3_builder_at(cursor);
+
+   for (unsigned i = 0; i < n; i++) {
+      ir3_NOP(&b);
+   }
+}
+
+/* There seems to be a hardware bug that sometimes causes a GPU hang when an
+ * alias...sam sequence crosses an instruction cache line boundary. This pass
+ * inserts padding nops to ensure no such sequence cross a cache line.
+ *
+ * While the number of nops we have to insert is fixed at this point, we can try
+ * to minimize the number of nops executed at runtime by replacing nops encoded
+ * in instructions by standalone nops. That is, if this pass has to insert one
+ * nop, it will try to make one of the following replacements:
+ * - (rptN)nop -> (rptN-1)nop; nop
+ * - (nopN)foo -> (nopN-1)foo; nop
+ *
+ * It does so by keeping track of "insert points". Each insert point keeps track
+ * of the instruction and the maximum number of nops that can be inserted there
+ * without pushing any subsequent alias sequences over the next cache line.
+ * Whenever we need to insert nops, we first try it at the encountered insert
+ * points and only if that doesn't work, we insert them right before the first
+ * alias. The pass makes sure the insert points are only visited a bounded
+ * number of times in total to keep the whole pass O(n).
+ */
+static void
+align_aliases(struct ir3 *ir)
+{
+   if (!ir->compiler->has_alias_tex) {
+      return;
+   }
+
+   struct alias_align_ctx *ctx = rzalloc(ir, struct alias_align_ctx);
+   unsigned ip = 0;
+
+   foreach_block (block, &ir->block_list) {
+      bool in_alias_sequence = false;
+
+      foreach_instr (instr, &block->instr_list) {
+         if (instr->opc == OPC_META_TEX_PREFETCH) {
+            /* These will not be part of the final binary. */
+            continue;
+         }
+
+         instr->ip = ip++;
+
+         if (is_insert_point(instr)) {
+            struct nop_insert_point nop_insert_point = {
+               .instr = instr,
+
+               /* This will be set when we encounter the next alias sequence. */
+               .max_nops = ~0,
+            };
+
+            array_insert(ctx, ctx->nop_insert_points, nop_insert_point);
+            continue;
+         }
+
+         if (instr->opc != OPC_ALIAS || instr->cat7.alias_scope != ALIAS_TEX) {
+            in_alias_sequence = false;
+            continue;
+         }
+
+         if (in_alias_sequence) {
+            /* We only have to handle the first alias. */
+            continue;
+         }
+
+         in_alias_sequence = true;
+         unsigned num_aliases = instr->cat7.alias_table_size_minus_one + 1;
+         unsigned last_ip = instr->ip + num_aliases;
+         unsigned num_aliases_first_icache_line =
+            num_aliases - (last_ip % ICACHE_LINE_INSTRS);
+
+         if (icache_line(instr->ip) != icache_line(last_ip)) {
+            unsigned nops_left = num_aliases_first_icache_line;
+            assert(nops_left > 0);
+
+            /* Maximum number of nops we can still insert without pushing any
+             * existing sequences over the next cache line. Will be set to the
+             * minimum of the insert points we encounter.
+             */
+            unsigned max_nops = ~0;
+
+            /* First try to insert nops at the available insert points. */
+            for (unsigned i = ctx->nop_insert_points_count; i > 0; i--) {
+               /* Every iteration, we either:
+                * 1. Break because max_nops has become zero, which removes all
+                *    insert points;
+                * 2. Pop the current insert point because it has no nops left;
+                * 3. Break because we're done, this may leave the last insert
+                *    point in the list but will have decreased the number of
+                *    nops available there.
+                *
+                * This ensures that every insert point is encountered at most a
+                * constant number of times (and this constant is bounded by the
+                * maximum number of nops that can be inserted at any insert
+                * point, which is 6 for (rpt6)nop). This means that the *total*
+                * number of iterations for this loop is O(n) and hence the whole
+                * pass is O(n).
+                */
+               assert(i == ctx->nop_insert_points_count);
+
+               struct nop_insert_point *nop_insert_point =
+                  &ctx->nop_insert_points[i - 1];
+
+               max_nops = MIN2(max_nops, nop_insert_point->max_nops);
+
+               uint8_t *instr_extra_nops = extra_nops(nop_insert_point->instr);
+               unsigned num_nops = MIN3(*instr_extra_nops, nops_left, max_nops);
+               assert(num_nops != 0);
+
+               insert_nops(ir3_after_instr(nop_insert_point->instr), num_nops);
+               *instr_extra_nops -= num_nops;
+               nops_left -= num_nops;
+
+               /* If max_nops == ~0, then nop_insert_point->max_nops == ~0. This
+                * means that nop_insert_point is currently unconstrained because
+                * we didn't encounter a subsequent sequence yet (other than the
+                * current one). Leave it unconstrained, if it hasn't been
+                * removed when we're done inserting nops, it will be updated
+                * when we finish handling this instruction.
+                */
+               if (max_nops != ~0) {
+                  max_nops -= num_nops;
+                  nop_insert_point->max_nops = max_nops;
+               }
+
+               if (max_nops == 0) {
+                  /* 1. If we cannot insert more nops at the last insert point,
+                   * we cannot insert more anywhere.
+                   */
+                  ctx->nop_insert_points_count = 0;
+                  break;
+               }
+
+               if (*instr_extra_nops == 0) {
+                  /* 2. The insert point is all used up, remove it. */
+                  ctx->nop_insert_points_count--;
+
+                  /* Since we only set max_nops on the last insert point, we
+                   * have to propagate it now to the new last.
+                   */
+                  update_last_insert_point_max_nops(ctx, max_nops);
+               } else {
+                  /* The current insert point has nops left so we should have
+                   * used them if needed.
+                   */
+                  assert(nops_left == 0);
+               }
+
+               if (nops_left == 0) {
+                  /* 3. All necessary nops inserted. */
+                  break;
+               }
+            }
+
+            /* Then, insert any remaining nops before the first alias. */
+            insert_nops(ir3_before_instr(instr), nops_left);
+
+            instr->ip += num_aliases_first_icache_line;
+            assert(util_is_aligned(instr->ip, ICACHE_LINE_INSTRS));
+
+            last_ip = instr->ip + num_aliases;
+            assert(icache_line(instr->ip) == icache_line(last_ip));
+
+            ip = instr->ip + 1;
+         }
+
+         /* Calculate how many nops we can insert before the first alias without
+          * moving the whole sequence over the next cache line. This allows us
+          * to insert nops for subsequent aliases before this one.
+          */
+         unsigned max_nops =
+            ICACHE_LINE_INSTRS - 1 - last_ip % ICACHE_LINE_INSTRS;
+         update_last_insert_point_max_nops(ctx, max_nops);
+      }
+   }
+
+   ralloc_free(ctx);
+}
+
 bool
 ir3_legalize(struct ir3 *ir, struct ir3_shader_variant *so, int *max_bary)
 {
@@ -1958,7 +2200,8 @@ ir3_legalize(struct ir3 *ir, struct ir3_shader_variant *so, int *max_bary)
             }
          }
 
-         if (in_preamble && writes_pred(instr)) {
+         if (in_preamble && writes_pred(instr) &&
+             !(instr->dsts[0]->flags & IR3_REG_UNIFORM)) {
             pred_in_preamble = true;
          }
       }
@@ -2036,10 +2279,6 @@ ir3_legalize(struct ir3 *ir, struct ir3_shader_variant *so, int *max_bary)
        so->compiler->gen >= 6)
       helper_sched(ctx, ir, so);
 
-   foreach_block (block, &ir->block_list) {
-      progress |= expand_dummy_dests(block);
-   }
-
    /* Note: insert (last) before alias.tex to have the sources that are actually
     * read by instructions (as opposed to alias registers) more easily
     * available.
@@ -2048,6 +2287,7 @@ ir3_legalize(struct ir3 *ir, struct ir3_shader_variant *so, int *max_bary)
       track_last(ctx, ir, so);
 
    ir3_insert_alias_tex(ir);
+   align_aliases(ir);
    ir3_count_instructions(ir);
    resolve_jumps(ir);
 
