@@ -217,7 +217,7 @@ handle_partial_const(nir_builder *b, nir_def **srcp, int *offp)
    if ((*srcp)->parent_instr->type != nir_instr_type_alu)
       return;
 
-   nir_alu_instr *alu = nir_instr_as_alu((*srcp)->parent_instr);
+   nir_alu_instr *alu = nir_def_as_alu((*srcp));
 
    if (alu->op == nir_op_imad24_ir3) {
       /* This case is slightly more complicated as we need to
@@ -428,20 +428,13 @@ copy_global_to_uniform(nir_shader *nir, struct ir3_ubo_analysis_state *state)
 }
 
 static bool
-copy_ubo_to_uniform(nir_shader *nir, const struct ir3_const_state *const_state,
-                    bool const_data_via_cp)
+copy_ubo_to_uniform(nir_shader *nir, const struct ir3_const_state *const_state)
 {
    const struct ir3_ubo_analysis_state *state = &const_state->ubo_state;
 
    if (state->num_enabled == 0)
       return false;
 
-   if (state->num_enabled == 1 &&
-       !state->range[0].ubo.bindless &&
-       state->range[0].ubo.block == const_state->consts_ubo.idx &&
-       const_data_via_cp)
-      return false;
-
    nir_function_impl *preamble = nir_shader_get_preamble(nir);
    nir_builder _b = nir_builder_at(nir_after_impl(preamble));
    nir_builder *b = &_b;
@@ -449,15 +442,6 @@ copy_ubo_to_uniform(nir_shader *nir, const struct ir3_const_state *const_state,
    for (unsigned i = 0; i < state->num_enabled; i++) {
       const struct ir3_ubo_range *range = &state->range[i];
 
-      /* The constant_data UBO is pushed in a different path from normal
-       * uniforms, and the state is setup earlier so it makes more sense to let
-       * the CP do it for us.
-       */
-      if (!range->ubo.bindless &&
-          range->ubo.block == const_state->consts_ubo.idx &&
-          const_data_via_cp)
-         continue;
-
       nir_def *ubo = nir_imm_int(b, range->ubo.block);
       if (range->ubo.bindless) {
          ubo = nir_bindless_resource_ir3(b, 32, ubo,
@@ -591,8 +575,7 @@ ir3_nir_lower_const_global_loads(nir_shader *nir, struct ir3_shader_variant *v)
       nir_foreach_function (function, nir) {
          if (function->impl) {
             if (function->is_preamble) {
-               nir_metadata_preserve(
-                  function->impl, nir_metadata_all);
+               nir_no_progress(function->impl);
                continue;
             }
 
@@ -607,8 +590,7 @@ ir3_nir_lower_const_global_loads(nir_shader *nir, struct ir3_shader_variant *v)
                }
             }
 
-            nir_metadata_preserve(
-               function->impl, nir_metadata_control_flow);
+            nir_progress(true, function->impl, nir_metadata_control_flow);
          }
       }
    }
@@ -697,7 +679,7 @@ ir3_nir_lower_ubo_loads(nir_shader *nir, struct ir3_shader_variant *v)
       if (function->impl) {
          if (function->is_preamble && push_ubos) {
             has_preamble = true;
-            nir_metadata_preserve(function->impl, nir_metadata_all);
+            nir_no_progress(function->impl);
             continue;
          }
          nir_builder builder = nir_builder_create(function->impl);
@@ -711,8 +693,7 @@ ir3_nir_lower_ubo_loads(nir_shader *nir, struct ir3_shader_variant *v)
             }
          }
 
-         nir_metadata_preserve(
-            function->impl, nir_metadata_control_flow);
+         nir_progress(true, function->impl, nir_metadata_control_flow);
       }
    }
    /* Update the num_ubos field for GL (first_ubo_is_default_ubo).  With
@@ -739,8 +720,7 @@ ir3_nir_lower_ubo_loads(nir_shader *nir, struct ir3_shader_variant *v)
    }
 
    if (compiler->has_preamble && push_ubos)
-      progress |= copy_ubo_to_uniform(
-         nir, const_state, !compiler->load_shader_consts_via_preamble);
+      progress |= copy_ubo_to_uniform(nir, const_state);
 
    return progress;
 }
