@@ -172,6 +172,8 @@ ir3_shader_assemble(struct ir3_shader_variant *v)
    if (compiler->gen >= 4)
       v->constlen = align(v->constlen, 4);
 
+   info->constlen = v->constlen;
+
    /* Use the per-wave layout by default on a6xx for compute shaders. It
     * should result in better performance when loads/stores are to a uniform
     * index.
@@ -404,7 +406,8 @@ assemble_variant(struct ir3_shader_variant *v, bool internal)
    _mesa_sha1_final(&ctx, sha1);
    _mesa_sha1_format(v->sha1_str, sha1);
 
-   bool dbg_enabled = shader_debug_enabled(v->type, internal);
+   bool dbg_enabled = shader_debug_enabled(v->type, internal) ||
+                      ir3_shader_bisect_disasm_select(v);
    if (dbg_enabled || ir3_shader_override_path || v->disasm_info.write_disasm) {
       bool shader_overridden =
          ir3_shader_override_path && try_override_shader_variant(v, v->sha1_str);
@@ -508,6 +511,7 @@ alloc_variant(struct ir3_shader *shader, const struct ir3_shader_key *key,
       return NULL;
 
    v->id = ++shader->variant_count;
+   v->shader = shader;
    v->shader_id = shader->id;
    v->binning_pass = !!nonbinning;
    v->nonbinning = nonbinning;
@@ -1253,7 +1257,7 @@ ir3_shader_outputs(const struct ir3_shader *so)
 void
 ir3_shader_get_subgroup_size(const struct ir3_compiler *compiler,
                              const struct ir3_shader_options *options,
-                             gl_shader_stage stage, unsigned *subgroup_size,
+                             mesa_shader_stage stage, unsigned *subgroup_size,
                              unsigned *max_subgroup_size)
 {
    switch (options->api_wavesize) {
