@@ -20,10 +20,15 @@ struct ir3_validate_ctx {
    /* Current instruction being validated: */
    struct ir3_instruction *current_instr;
 
-   /* Set of instructions found so far, used to validate that we
-    * don't have SSA uses that occure before def's
+   /* Lazily set up set of instructions that are rpts within the current block. */
+   struct set *rpt_set;
+   struct ir3_block *rpt_block;
+
+   /* Bitset of instructions found so far, used to validate that we don't have
+    * SSA uses that occur before defs.
     */
-   struct set *defs;
+   BITSET_WORD *defs;
+   unsigned defs_count;
 };
 
 static void
@@ -60,6 +65,12 @@ validate_reg(struct ir3_validate_ctx *ctx, struct ir3_register *reg)
       validate_assert(ctx, reg->num >= SHARED_REG_START);
       validate_assert(ctx, reg->num - SHARED_REG_START < SHARED_REG_SIZE);
    }
+
+   if (reg->flags & IR3_REG_UNIFORM) {
+      validate_assert(ctx, ctx->ir->compiler->has_scalar_predicates);
+      validate_assert(ctx, reg->flags & IR3_REG_PREDICATE);
+   }
+
 }
 
 static void
@@ -93,7 +104,8 @@ validate_src(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr,
 
    struct ir3_register *src = reg->def;
 
-   validate_assert(ctx, _mesa_set_search(ctx->defs, src->instr));
+   validate_assert(ctx, src->instr->serialno < ctx->defs_count &&
+                           BITSET_TEST(ctx->defs, src->instr->serialno));
 
    if (src->instr->opc == OPC_META_COLLECT) {
       /* We only support reading a subset of written components from collects.
@@ -146,10 +158,22 @@ validate_phi_src(struct ir3_validate_ctx *ctx, struct ir3_block *block,
    }
 }
 
+static void
+validate_def(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr)
+{
+   /* Should have been initialized when it was inserted. */
+   validate_assert(ctx, instr->serialno != 0);
+   /* The def should only be seen once (no
+                                 duplicate serialnos). */
+   validate_assert(ctx, instr->serialno < ctx->defs_count &&
+                           !BITSET_TEST(ctx->defs, instr->serialno));
+   BITSET_SET(ctx->defs, instr->serialno);
+}
+
 static void
 validate_phi(struct ir3_validate_ctx *ctx, struct ir3_instruction *phi)
 {
-   _mesa_set_add(ctx->defs, phi);
+   validate_def(ctx, phi);
    validate_assert(ctx, phi->dsts_count == 1);
    validate_assert(ctx, is_dest_gpr(phi->dsts[0]));
 }
@@ -190,6 +214,9 @@ validate_dst(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr,
    if (reg->flags & IR3_REG_RELATIV)
       validate_assert(ctx, instr->address);
 
+   if (reg->flags & IR3_REG_UNIFORM)
+      validate_assert(ctx, opc_cat(instr->opc) == 2);
+
    validate_reg(ctx, reg);
 }
 
@@ -198,14 +225,19 @@ validate_dst(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr,
       ctx, (type_size(type) <= 16) == !!((reg)->flags & IR3_REG_HALF))
 
 static bool
-block_contains(struct ir3_block *block, struct ir3_instruction *instr)
+block_contains_rpt(struct ir3_validate_ctx *ctx, struct ir3_block *block,
+                   struct ir3_instruction *instr)
 {
-   foreach_instr (block_instr, &block->instr_list) {
-      if (block_instr == instr)
-         return true;
+   if (ctx->rpt_block != block) {
+      _mesa_set_clear(ctx->rpt_set, NULL);
+      foreach_instr (block_instr, &block->instr_list) {
+         if (ir3_instr_is_rpt(block_instr))
+            _mesa_set_add(ctx->rpt_set, block_instr);
+      }
+      ctx->rpt_block = block;
    }
 
-   return false;
+   return _mesa_set_search(ctx->rpt_set, instr);
 }
 
 static void
@@ -222,7 +254,7 @@ validate_rpt(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr)
           * fail if, for example, list_delinit is called instead of
           * ir3_instr_remove.
           */
-         validate_assert(ctx, block_contains(instr->block, rpt));
+         validate_assert(ctx, block_contains_rpt(ctx, instr->block, rpt));
       }
    } else if (instr->repeat) {
       validate_assert(ctx, ir3_supports_rpt(ctx->ir->compiler, instr->opc));
@@ -297,7 +329,7 @@ validate_instr(struct ir3_validate_ctx *ctx, struct ir3_instruction *instr)
       validate_dst(ctx, instr, reg);
    }
 
-   _mesa_set_add(ctx->defs, instr);
+   validate_def(ctx, instr);
 
    if ((opc_cat(instr->opc) == 2 || opc_cat(instr->opc) == 3 ||
         opc_cat(instr->opc) == 4)) {
@@ -595,7 +627,10 @@ ir3_validate(struct ir3 *ir)
    struct ir3_validate_ctx *ctx = ralloc_size(NULL, sizeof(*ctx));
 
    ctx->ir = ir;
-   ctx->defs = _mesa_pointer_set_create(ctx);
+   ctx->defs_count = ir->instr_count + 1; /* serialno comes from pre-incrementing this. */
+   ctx->defs = rzalloc_array(ctx, BITSET_WORD, BITSET_WORDS(ctx->defs_count));
+   ctx->rpt_set = _mesa_pointer_set_create(ctx);
+   ctx->rpt_block = NULL;
 
    foreach_block (block, &ir->block_list) {
       ctx->current_block = block;
