@@ -517,18 +517,9 @@ do_clip_tri(struct draw_stage *stage,
       }
 
       /* swap in/out lists */
-      {
-         struct vertex_header **tmp = inlist;
-         inlist = outlist;
-         outlist = tmp;
-         n = outcount;
-      }
-      {
-         bool *tmp = inEdges;
-         inEdges = outEdges;
-         outEdges = tmp;
-      }
-
+      SWAP(inlist, outlist);
+      SWAP(inEdges, outEdges);
+      n = outcount;
    }
 
    /* If constant interpolated, copy provoking vertex attrib to polygon vertex[0]
