@@ -7,11 +7,11 @@
 
 #define FUNC_ENTER                                                \
    /* declare more local vars */                                  \
-   const enum pipe_prim_type prim = input_prims->prim;            \
+   const enum mesa_prim prim = input_prims->prim;            \
    const unsigned prim_flags = input_prims->flags;                \
-   const boolean last_vertex_last = !asmblr->draw->rasterizer->flatshade_first;  \
+   const bool last_vertex_last = !asmblr->draw->rasterizer->flatshade_first;  \
    switch (prim) {                                                  \
-   case PIPE_PRIM_POLYGON:                                          \
+   case MESA_PRIM_POLYGON:                                          \
       assert(!"unexpected primitive type in prim assembler"); \
       return;                                                       \
    default:                                                         \
