@@ -70,7 +70,7 @@ draw_create_vertex_shader(struct draw_context *draw,
    if (draw->pt.middle.llvm) {
       struct pipe_screen *screen = draw->pipe->screen;
       if (shader->type == PIPE_SHADER_IR_NIR &&
-          !screen->shader_caps[PIPE_SHADER_VERTEX].integers) {
+          !screen->shader_caps[MESA_SHADER_VERTEX].integers) {
         state.type = PIPE_SHADER_IR_TGSI;
         state.tokens = nir_to_tgsi(shader->ir.nir, screen);
         is_allocated = true;
@@ -163,7 +163,7 @@ draw_vs_init(struct draw_context *draw)
    draw->dump_vs = debug_get_option_gallium_dump_vs();
 
    if (!draw->llvm) {
-      draw->vs.tgsi.machine = tgsi_exec_machine_create(PIPE_SHADER_VERTEX);
+      draw->vs.tgsi.machine = tgsi_exec_machine_create(MESA_SHADER_VERTEX);
       if (!draw->vs.tgsi.machine)
          return false;
    }
