@@ -208,15 +208,9 @@ class DelegatingJITMemoryManager : public BaseMemoryManager {
       virtual void registerEHFrames(uint8_t *Addr, uint64_t LoadAddr, size_t Size) {
          mgr()->registerEHFrames(Addr, LoadAddr, Size);
       }
-#if LLVM_VERSION_MAJOR >= 5
       virtual void deregisterEHFrames() {
          mgr()->deregisterEHFrames();
       }
-#else
-      virtual void deregisterEHFrames(uint8_t *Addr, uint64_t LoadAddr, size_t Size) {
-         mgr()->deregisterEHFrames(Addr, LoadAddr, Size);
-      }
-#endif
       virtual void *getPointerToNamedFunction(const std::string &Name,
                                               bool AbortOnFailure=true) {
          return mgr()->getPointerToNamedFunction(Name, AbortOnFailure);
@@ -383,6 +377,7 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
    MAttrs.push_back(util_get_cpu_caps()->has_avx512bw ? "+avx512bw"  : "-avx512bw");
    MAttrs.push_back(util_get_cpu_caps()->has_avx512dq ? "+avx512dq"  : "-avx512dq");
    MAttrs.push_back(util_get_cpu_caps()->has_avx512vl ? "+avx512vl"  : "-avx512vl");
+   MAttrs.push_back(util_get_cpu_caps()->has_avx512vbmi ? "+avx512vbmi"  : "-avx512vbmi");
 #endif
 #if DETECT_ARCH_ARM
    if (!util_get_cpu_caps()->has_neon) {
@@ -425,8 +420,6 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
 
 #if DETECT_ARCH_LOONGARCH64 == 1
    /*
-    * TODO: Implement util_get_cpu_caps()
-    *
     * No FPU-less LoongArch64 systems are ever shipped yet, and LP64D is
     * the default ABI, so FPU is enabled here.
     *
@@ -435,9 +428,16 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
     * https://github.com/loongson/la-softdev-convention/releases/download/v0.1/la-softdev-convention.pdf
     */
    MAttrs = {"+f","+d"};
-#if LLVM_VERSION_MAJOR == 17
-   /* LLVM 17's LSX support is incomplete, so explicitly mask it */
+#if LLVM_VERSION_MAJOR >= 18
+   MAttrs.push_back(util_get_cpu_caps()->has_lsx ? "+lsx" : "-lsx");
+   MAttrs.push_back(util_get_cpu_caps()->has_lasx ? "+lasx" : "-lasx");
+#else
+   /*
+    * LLVM 17's LSX support is incomplete, and LLVM 16 isn't supported
+    * LSX and LASX. So explicitly mask it.
+    */
    MAttrs.push_back("-lsx");
+   MAttrs.push_back("-lasx");
 #endif
 #endif
 }
