@@ -34,7 +34,7 @@
 #ifndef U_RANGE_H
 #define U_RANGE_H
 
-#include "os/os_thread.h"
+#include "util/u_thread.h"
 #include "pipe/p_state.h"
 #include "pipe/p_screen.h"
 #include "util/u_atomic.h"
@@ -76,13 +76,19 @@ util_range_add(struct pipe_resource *resource, struct util_range *range,
    }
 }
 
-static inline boolean
+static inline bool
 util_ranges_intersect(const struct util_range *range,
                       unsigned start, unsigned end)
 {
    return MAX2(start, range->start) < MIN2(end, range->end);
 }
 
+static inline bool
+util_ranges_covered(const struct util_range *range,
+                    unsigned start, unsigned end)
+{
+   return (start <= range->start) && (end >= range->end);
+}
 
 /* Init/deinit */
 
