@@ -29,6 +29,7 @@
 #include "d3d12_resource.h"
 #include "d3d12_video_buffer.h"
 #include "d3d12_format.h"
+#include "d3d12_interop_public.h"
 
 void
 d3d12_video_processor_begin_frame(struct pipe_video_codec * codec,
@@ -55,7 +56,7 @@ d3d12_video_processor_begin_frame(struct pipe_video_codec * codec,
     if (FAILED(hr)) {
         debug_printf(
             "[d3d12_video_processor] resetting ID3D12GraphicsCommandList failed with HR %x\n",
-            hr);
+            (unsigned)hr);
         assert(false);
     }
 
@@ -276,8 +277,8 @@ d3d12_video_processor_process_frame(struct pipe_video_codec *codec,
     };
 
     debug_printf("ProcessFrame InArgs Orientation %d \n\tSrc top: %d left: %d right: %d bottom: %d\n\tDst top: %d left: %d right: %d bottom: %d\n", InputArguments.Transform.Orientation, 
-        InputArguments.Transform.SourceRectangle.top, InputArguments.Transform.SourceRectangle.left, InputArguments.Transform.SourceRectangle.right, InputArguments.Transform.SourceRectangle.bottom,
-        InputArguments.Transform.DestinationRectangle.top, InputArguments.Transform.DestinationRectangle.left, InputArguments.Transform.DestinationRectangle.right, InputArguments.Transform.DestinationRectangle.bottom);
+        (int)InputArguments.Transform.SourceRectangle.top, (int)InputArguments.Transform.SourceRectangle.left, (int)InputArguments.Transform.SourceRectangle.right, (int)InputArguments.Transform.SourceRectangle.bottom,
+        (int)InputArguments.Transform.DestinationRectangle.top, (int)InputArguments.Transform.DestinationRectangle.left, (int)InputArguments.Transform.DestinationRectangle.right, (int)InputArguments.Transform.DestinationRectangle.bottom);
 
     pD3D12Proc->m_ProcessInputs.push_back(InputArguments);    
     pD3D12Proc->m_InputBuffers.push_back(pInputVideoBuffer);
@@ -305,6 +306,16 @@ d3d12_video_processor_destroy(struct pipe_video_codec * codec)
         d3d12_video_processor_sync_completion(codec, curBatchFence, OS_TIMEOUT_INFINITE);
     }
 
+   struct d3d12_context* ctx = d3d12_context(pD3D12Proc->base.context);
+   if (ctx->priority_manager)
+   {
+      if (ctx->priority_manager->unregister_work_queue(ctx->priority_manager, pD3D12Proc->m_spCommandQueue.Get()) != 0)
+      {
+         debug_printf("D3D12: Failed to unregister command queue with frontend priority manager\n");
+         assert(false);
+      }
+   }
+
     // Call dtor to make ComPtr work
     delete pD3D12Proc;
 }
@@ -343,7 +354,7 @@ d3d12_video_processor_flush(struct pipe_video_codec * codec)
             debug_printf("[d3d12_video_processor] d3d12_video_processor_flush"
                             " - D3D12Device was removed BEFORE commandlist "
                             "execution with HR %x.\n",
-                            hr);
+                            (unsigned)hr);
             goto flush_fail;
         }
 
@@ -358,7 +369,7 @@ d3d12_video_processor_flush(struct pipe_video_codec * codec)
 
         hr = pD3D12Proc->m_spCommandList->Close();
         if (FAILED(hr)) {
-            debug_printf("[d3d12_video_processor] d3d12_video_processor_flush - Can't close command list with HR %x\n", hr);
+            debug_printf("[d3d12_video_processor] d3d12_video_processor_flush - Can't close command list with HR %x\n", (unsigned)hr);
             goto flush_fail;
         }
 
@@ -383,7 +394,7 @@ d3d12_video_processor_flush(struct pipe_video_codec * codec)
             debug_printf("[d3d12_video_processor] d3d12_video_processor_flush"
                             " - D3D12Device was removed AFTER commandlist "
                             "execution with HR %x, but wasn't before.\n",
-                            hr);
+                            (unsigned)hr);
             goto flush_fail;
         }
 
@@ -473,6 +484,17 @@ d3d12_video_processor_create(struct pipe_context *context, const struct pipe_vid
 
     debug_printf("[d3d12_video_processor] d3d12_video_create_processor - Created successfully!\n");
 
+   if (pD3D12Ctx->priority_manager)
+   {
+      // Register queue with priority manager
+      if (pD3D12Ctx->priority_manager->register_work_queue(pD3D12Ctx->priority_manager, pD3D12Proc->m_spCommandQueue.Get()) != 0)
+      {
+         debug_printf("[d3d12_video_processor] d3d12_video_create_processor - Failure on "
+                      "pipe_priority_manager::register_work_queue\n");
+         goto failed;
+      }
+   }
+
    return &pD3D12Proc->base;
 
 failed:
@@ -609,7 +631,7 @@ d3d12_video_processor_check_caps_and_create_processor(struct d3d12_video_process
         if (FAILED(hr)) {
         debug_printf("[d3d12_video_processor] d3d12_video_processor_check_caps_and_create_processor - CheckFeatureSupport "
                         "failed with HR %x\n",
-                        hr);
+                        (unsigned)hr);
         return false;
         }
 
@@ -639,7 +661,7 @@ d3d12_video_processor_check_caps_and_create_processor(struct d3d12_video_process
     if (FAILED(hr)) {
         debug_printf("[d3d12_video_processor] d3d12_video_processor_check_caps_and_create_processor - CreateVideoProcessor "
                     "failed with HR %x\n",
-                    hr);
+                    (unsigned)hr);
         return false;
     }
 
@@ -652,6 +674,8 @@ d3d12_video_processor_create_command_objects(struct d3d12_video_processor *pD3D1
     assert(pD3D12Proc->m_spD3D12VideoDevice);
 
     D3D12_COMMAND_QUEUE_DESC commandQueueDesc = { D3D12_COMMAND_LIST_TYPE_VIDEO_PROCESS };
+    if (pD3D12Proc->m_pD3D12Screen->supports_dynamic_queue_priority)
+        commandQueueDesc.Flags |= D3D12_COMMAND_QUEUE_FLAG_ALLOW_DYNAMIC_PRIORITY;
     HRESULT hr = pD3D12Proc->m_pD3D12Screen->dev->CreateCommandQueue(
                 &commandQueueDesc,
                 IID_PPV_ARGS(pD3D12Proc->m_spCommandQueue.GetAddressOf()));
@@ -659,7 +683,7 @@ d3d12_video_processor_create_command_objects(struct d3d12_video_processor *pD3D1
     if (FAILED(hr)) {
         debug_printf("[d3d12_video_processor] d3d12_video_processor_create_command_objects - Call to CreateCommandQueue "
                         "failed with HR %x\n",
-                        hr);
+                        (unsigned)hr);
         return false;
     }
 
@@ -670,7 +694,7 @@ d3d12_video_processor_create_command_objects(struct d3d12_video_processor *pD3D1
     if (FAILED(hr)) {
         debug_printf(
             "[d3d12_video_processor] d3d12_video_processor_create_command_objects - Call to CreateFence failed with HR %x\n",
-            hr);
+            (unsigned)hr);
         return false;
     }
 
@@ -683,7 +707,7 @@ d3d12_video_processor_create_command_objects(struct d3d12_video_processor *pD3D1
         if (FAILED(hr)) {
             debug_printf("[d3d12_video_processor] d3d12_video_processor_create_command_objects - Call to "
                             "CreateCommandAllocator failed with HR %x\n",
-                            hr);
+                            (unsigned)hr);
             return false;
         }
     }
@@ -704,7 +728,7 @@ d3d12_video_processor_create_command_objects(struct d3d12_video_processor *pD3D1
     if (FAILED(hr)) {
         debug_printf("[d3d12_video_processor] d3d12_video_processor_create_command_objects - Call to CreateCommandList "
                         "failed with HR %x\n",
-                        hr);
+                        (unsigned)hr);
         return false;
     }
 
@@ -781,7 +805,7 @@ d3d12_video_processor_ensure_fence_finished(struct pipe_video_codec *codec,
          debug_printf("[d3d12_video_processor] d3d12_video_processor_ensure_fence_finished - SetEventOnCompletion for "
                       "fenceValue %" PRIu64 " failed with HR %x\n",
                       fenceValueToWaitOn,
-                      hr);
+                      (unsigned)hr);
          goto ensure_fence_finished_fail;
       }
 
@@ -822,7 +846,7 @@ d3d12_video_processor_sync_completion(struct pipe_video_codec *codec, uint64_t f
    hr =
       pD3D12Proc->m_spCommandAllocators[fenceValueToWaitOn % D3D12_VIDEO_PROC_ASYNC_DEPTH]->Reset();
    if (FAILED(hr)) {
-      debug_printf("m_spCommandAllocator->Reset() failed with %x.\n", hr);
+      debug_printf("m_spCommandAllocator->Reset() failed with %x.\n", (unsigned)hr);
       goto sync_with_token_fail;
    }
 
@@ -832,7 +856,7 @@ d3d12_video_processor_sync_completion(struct pipe_video_codec *codec, uint64_t f
       debug_printf("[d3d12_video_processor] d3d12_video_processor_sync_completion"
                    " - D3D12Device was removed AFTER d3d12_video_processor_ensure_fence_finished "
                    "execution with HR %x, but wasn't before.\n",
-                   hr);
+                   (unsigned)hr);
       goto sync_with_token_fail;
    }
 
