@@ -25,6 +25,22 @@ etna_ml_lower_pad(struct etna_ml_subgraph *subgraph,
                   const struct pipe_ml_operation *pad,
                   struct etna_operation *operation);
 
+void
+etna_ml_lower_relu(struct etna_ml_subgraph *subgraph,
+                   const struct pipe_ml_operation *operation,
+                   const struct pipe_tensor *input_tensor,
+                   struct etna_operation *relu);
+
+void
+etna_ml_lower_absolute(struct etna_ml_subgraph *subgraph,
+                       const struct pipe_ml_operation *pad,
+                       struct etna_operation *operation);
+
+void
+etna_ml_lower_logistic(struct etna_ml_subgraph *subgraph,
+                       const struct pipe_ml_operation *pad,
+                       struct etna_operation *operation);
+
 void
 etna_ml_compile_operation_tp(struct etna_ml_subgraph *subgraph,
                              const struct etna_operation *operation,
