@@ -284,7 +284,13 @@ etna_link_shaders(struct etna_context *ctx, struct compiled_shader_state *cs,
    cs->ps_inst_mem_size = fs->code_size;
    cs->PS_INST_MEM = fs->code;
 
-   if (vs->needs_icache || fs->needs_icache) {
+   if (vs->needs_icache || fs->needs_icache ||
+       (ctx->screen->specs.has_unified_instmem &&
+        ((cs->vs_inst_mem_size + cs->ps_inst_mem_size) / 4 >
+         ctx->screen->specs.max_instructions))) {
+      if (!ctx->screen->specs.has_icache)
+         return false;
+
       /* If either of the shaders needs ICACHE, we use it for both. It is
        * either switched on or off for the entire shader processor.
        */
@@ -395,7 +401,7 @@ etna_shader_stage(struct etna_shader *shader)
    case MESA_SHADER_FRAGMENT:   return "FRAG";
    case MESA_SHADER_COMPUTE:    return "CL";
    default:
-      unreachable("invalid type");
+      UNREACHABLE("invalid type");
       return NULL;
    }
 }
@@ -607,7 +613,7 @@ etna_set_max_shader_compiler_threads(struct pipe_screen *pscreen,
 static bool
 etna_is_parallel_shader_compilation_finished(struct pipe_screen *pscreen,
                                              void *hwcso,
-                                             enum pipe_shader_type shader_type)
+                                             mesa_shader_stage shader_type)
 {
    struct etna_shader *shader = (struct etna_shader *)hwcso;
 
@@ -638,6 +644,9 @@ etna_shader_screen_init(struct pipe_screen *pscreen)
    if (!screen->compiler)
       return false;
 
+   for (unsigned i = 0; i <= MESA_SHADER_COMPUTE; i++)
+      pscreen->nir_options[i] = etna_compiler_get_options(screen->compiler);
+
    pscreen->set_max_shader_compiler_threads = etna_set_max_shader_compiler_threads;
    pscreen->is_parallel_shader_compilation_finished = etna_is_parallel_shader_compilation_finished;
 
