@@ -42,7 +42,7 @@ draw_impl(struct fd_context *ctx, struct fd_ringbuffer *ring,
 
    fd3_emit_state(ctx, ring, emit);
 
-   if ((ctx->dirty_shader[PIPE_SHADER_VERTEX] & FD_DIRTY_SHADER_PROG) ||
+   if ((ctx->dirty_shader[MESA_SHADER_VERTEX] & FD_DIRTY_SHADER_PROG) ||
        (emit->dirty & (FD_DIRTY_VTXBUF | FD_DIRTY_VTXSTATE)))
       fd3_emit_vertex_bufs(ring, emit);
 
