@@ -22,21 +22,17 @@ struct fd_ringbuffer *
 fd6_build_driver_params(struct fd6_emit *emit) assert_dt;
 
 template <chip CHIP>
-void fd6_emit_cs_driver_params(struct fd_context *ctx,
-                               struct fd_ringbuffer *ring,
-                               struct fd6_compute_state *cs,
+void fd6_emit_cs_driver_params(struct fd_context *ctx, fd_cs &cs,
+                               const struct ir3_shader_variant *v,
                                const struct pipe_grid_info *info) assert_dt;
 template <chip CHIP>
-void fd6_emit_cs_user_consts(struct fd_context *ctx,
-                             struct fd_ringbuffer *ring,
-                             struct fd6_compute_state *cs) assert_dt;
+void fd6_emit_cs_user_consts(struct fd_context *ctx, fd_cs &cs,
+                             const struct ir3_shader_variant *v) assert_dt;
 template <chip CHIP>
-void fd6_emit_immediates(const struct ir3_shader_variant *v,
-                         struct fd_ringbuffer *ring) assert_dt;
+void fd6_emit_immediates(const struct ir3_shader_variant *v, fd_cs &cs) assert_dt;
 template <chip CHIP>
-void fd6_emit_link_map(struct fd_context *ctx,
+void fd6_emit_link_map(struct fd_context *ctx, fd_cs &cs,
                        const struct ir3_shader_variant *producer,
-                       const struct ir3_shader_variant *consumer,
-                       struct fd_ringbuffer *ring) assert_dt;
+                       const struct ir3_shader_variant *consumer) assert_dt;
 
 #endif /* FD6_CONST_H */
