@@ -80,7 +80,7 @@ rebind_resource_in_ctx(struct fd_context *ctx,
       return;
 
    /* per-shader-stage resources: */
-   for (unsigned stage = 0; stage < PIPE_SHADER_TYPES; stage++) {
+   for (unsigned stage = 0; stage < MESA_SHADER_STAGES; stage++) {
       /* Constbufs.. note that constbuf[0] is normal uniforms emitted in
        * cmdstream rather than by pointer..
        */
@@ -522,7 +522,7 @@ fd_try_shadow_resource(struct fd_context *ctx, struct fd_resource *rsc,
       case PIPE_TEXTURE_2D:
          /* TODO */
       default:
-         unreachable("TODO");
+         UNREACHABLE("TODO");
       }
    }
 
@@ -1104,8 +1104,11 @@ fd_resource_get_handle(struct pipe_screen *pscreen, struct pipe_context *pctx,
 
       pctx = threaded_context_unwrap_sync(pctx);
 
-      struct fd_context *ctx = pctx ?
-            fd_context(pctx) : fd_screen_aux_context_get(pscreen);
+      /* Always use aux ctx, even if we are provided a valid pctx, since in
+       * the TC case this could be called from something other than the
+       * driver thread
+       */
+      struct fd_context *ctx = fd_screen_aux_context_get(pscreen);
 
       /* Since gl is horrible, we can end up getting asked to export a handle
        * for a rsc which was not originally allocated in a way that can be
@@ -1119,8 +1122,7 @@ fd_resource_get_handle(struct pipe_screen *pscreen, struct pipe_context *pctx,
 
       ret = fd_try_shadow_resource(ctx, rsc, 0, NULL, handle->modifier);
 
-      if (!pctx)
-         fd_screen_aux_context_put(pscreen);
+      fd_screen_aux_context_put(pscreen);
 
       if (!ret)
          return false;
@@ -1513,8 +1515,11 @@ fd_resource_from_handle(struct pipe_screen *pscreen,
 
    rsc->internal_format = tmpl->format;
 
-   if (!screen->layout_resource_for_handle(rsc, handle))
+   if (tmpl->target == PIPE_BUFFER) {
+      fdl_layout_buffer(&rsc->layout, tmpl->width0);
+   } else if (!screen->layout_resource_for_handle(rsc, handle)) {
       goto fail;
+   }
 
    if (rsc->layout.pitch0 != handle->stride)
       goto fail;
