@@ -30,6 +30,7 @@
 #include "util/slab.h"
 #include "util/u_debug.h"
 #include "util/macros.h"
+#include "util/u_framebuffer.h"
 #include "util/u_threaded_context.h"
 #include "intel/blorp/blorp.h"
 #include "intel/dev/intel_debug.h"
@@ -231,7 +232,8 @@ struct iris_vue_prog_key {
    struct iris_base_prog_key base;
 
    unsigned nr_userclip_plane_consts:4;
-   unsigned padding:28;
+   enum intel_vue_layout layout:2;
+   unsigned padding:26;
 };
 
 struct iris_vs_prog_key {
@@ -276,14 +278,13 @@ struct iris_fs_prog_key {
    uint8_t color_outputs_valid;
 
    unsigned nr_color_regions:5;
-   bool flat_shade:1;
    bool alpha_test_replicate_alpha:1;
    bool alpha_to_coverage:1;
-   bool clamp_fragment_color:1;
    bool persample_interp:1;
    bool multisample_fbo:1;
    bool force_dual_color_blend:1;
    bool coherent_fb_fetch:1;
+   enum intel_vue_layout vue_layout:2;
    uint64_t padding:43;
 };
 
@@ -590,9 +591,6 @@ struct iris_uncompiled_shader {
    /* Whether shader uses atomic operations. */
    bool uses_atomic_load_store;
 
-   /** Size (in bytes) of the kernel input data */
-   unsigned kernel_input_size;
-
    /** Size (in bytes) of the local (shared) data passed as kernel inputs */
    unsigned kernel_shared_size;
 
@@ -684,9 +682,6 @@ struct iris_compiled_shader {
    uint32_t *system_values;
    unsigned num_system_values;
 
-   /** Size (in bytes) of the kernel input data */
-   unsigned kernel_input_size;
-
    /** Number of constbufs expected by the shader. */
    unsigned num_cbufs;
 
@@ -698,7 +693,7 @@ struct iris_compiled_shader {
 
    struct iris_binding_table bt;
 
-   gl_shader_stage stage;
+   mesa_shader_stage stage;
 
    /**
     * Data derived from prog_data.
@@ -769,7 +764,7 @@ iris_vue_data(struct iris_compiled_shader *shader)
    case MESA_SHADER_TESS_EVAL: return &shader->tes.base;
    case MESA_SHADER_GEOMETRY:  return &shader->gs.base;
    default:
-      unreachable("invalid shader stage for vue prog data");
+      UNREACHABLE("invalid shader stage for vue prog data");
       return NULL;
    }
 }
@@ -1012,6 +1007,7 @@ struct iris_context {
       struct pipe_viewport_state viewports[IRIS_MAX_VIEWPORTS];
       struct pipe_scissor_state scissors[IRIS_MAX_VIEWPORTS];
       struct pipe_stencil_ref stencil_ref;
+      PIPE_FB_SURFACES; //STOP USING THIS
       struct pipe_framebuffer_state framebuffer;
       struct pipe_clip_state clip_planes;
       /* width and height treated like x2 and y2 */
@@ -1053,8 +1049,6 @@ struct iris_context {
       /** Aux usage of the fb's depth buffer (which may or may not exist). */
       enum isl_aux_usage hiz_usage;
 
-      enum intel_urb_deref_block_size urb_deref_block_size;
-
       /** Are depth writes enabled?  (Depth buffer may or may not exist.) */
       bool depth_writes_enabled;
 
@@ -1108,7 +1102,7 @@ struct iris_context {
        * Query BO with a MI_PREDICATE_RESULT snapshot calculated on the
        * render context that needs to be uploaded to the compute context.
        */
-      struct iris_bo *compute_predicate;
+      struct iris_address compute_predicate;
 
       /** Is a PIPE_QUERY_PRIMITIVES_GENERATED query active? */
       bool prims_generated_query_active;
@@ -1161,6 +1155,11 @@ struct iris_context {
       struct pipe_resource *pixel_hashing_tables;
 
       bool use_tbimr;
+
+      /* For compute engine only */
+      uint8_t pixel_async_compute_thread_limit;
+      uint8_t z_pass_async_compute_thread_limit;
+      uint8_t np_z_async_throttle_settings;
    } state;
 };
 
@@ -1219,9 +1218,10 @@ struct iris_gen_indirect_params {
 };
 
 #define perf_debug(dbg, ...) do {                      \
+   void *__var = (void*)(dbg);                         \
    if (INTEL_DEBUG(DEBUG_PERF))                        \
       dbg_printf(__VA_ARGS__);                         \
-   if (unlikely(dbg))                                  \
+   if (unlikely(__var))                                \
       util_debug_message(dbg, PERF_INFO, __VA_ARGS__); \
 } while(0)
 
@@ -1290,7 +1290,7 @@ iris_blorp_batch_usage(struct iris_batch *batch, bool is_dest)
    case IRIS_BATCH_BLITTER:
       return is_dest ? ISL_SURF_USAGE_BLITTER_DST_BIT : ISL_SURF_USAGE_BLITTER_SRC_BIT;
    default:
-      unreachable("Unhandled batch type");
+      UNREACHABLE("Unhandled batch type");
    }
 }
 
@@ -1331,10 +1331,10 @@ void iris_upload_ubo_ssbo_surf_state(struct iris_context *ice,
                                      struct iris_state_ref *surf_state,
                                      isl_surf_usage_flags_t usage);
 const struct shader_info *iris_get_shader_info(const struct iris_context *ice,
-                                               gl_shader_stage stage);
+                                               mesa_shader_stage stage);
 struct iris_bo *iris_get_scratch_space(struct iris_context *ice,
                                        unsigned per_thread_scratch,
-                                       gl_shader_stage stage);
+                                       mesa_shader_stage stage);
 const struct iris_state_ref *iris_get_scratch_surf(struct iris_context *ice,
                                                    unsigned per_thread_scratch);
 uint32_t iris_group_index_to_bti(const struct iris_binding_table *bt,
@@ -1359,9 +1359,9 @@ iris_fs_barycentric_modes(const struct iris_compiled_shader *shader,
                           enum intel_msaa_flags pushed_msaa_flags);
 bool iris_use_tcs_multi_patch(struct iris_screen *screen);
 bool iris_indirect_ubos_use_sampler(struct iris_screen *screen);
-const void *iris_get_compiler_options(struct pipe_screen *pscreen,
-                                      enum pipe_shader_ir ir,
-                                      enum pipe_shader_type pstage);
+const struct nir_shader_compiler_options *
+iris_get_compiler_options(struct pipe_screen *pscreen,
+                          mesa_shader_stage pstage);
 
 /* iris_disk_cache.c */
 
@@ -1389,7 +1389,7 @@ struct iris_compiled_shader *iris_find_cached_shader(struct iris_context *ice,
 
 struct iris_compiled_shader *iris_create_shader_variant(const struct iris_screen *,
                                                         void *mem_ctx,
-                                                        gl_shader_stage stage,
+                                                        mesa_shader_stage stage,
                                                         enum iris_program_cache_id cache_id,
                                                         uint32_t key_size,
                                                         const void *key);
@@ -1398,7 +1398,6 @@ void iris_finalize_program(struct iris_compiled_shader *shader,
                            uint32_t *streamout,
                            uint32_t *system_values,
                            unsigned num_system_values,
-                           unsigned kernel_input_size,
                            unsigned num_cbufs,
                            const struct iris_binding_table *bt);
 
@@ -1466,17 +1465,17 @@ void iris_ensure_indirect_generation_shader(struct iris_batch *batch);
 void iris_predraw_resolve_inputs(struct iris_context *ice,
                                  struct iris_batch *batch,
                                  bool *draw_aux_buffer_disabled,
-                                 gl_shader_stage stage,
+                                 mesa_shader_stage stage,
                                  bool consider_framebuffer);
 void iris_predraw_resolve_framebuffer(struct iris_context *ice,
                                       struct iris_batch *batch,
                                       bool *draw_aux_buffer_disabled);
 void iris_predraw_flush_buffers(struct iris_context *ice,
                                 struct iris_batch *batch,
-                                gl_shader_stage stage);
+                                mesa_shader_stage stage);
 void iris_postdraw_update_resolve_tracking(struct iris_context *ice);
 void iris_postdraw_update_image_resolve_tracking(struct iris_context *ice,
-                                                 gl_shader_stage stage);
+                                                 mesa_shader_stage stage);
 int iris_get_driver_query_info(struct pipe_screen *pscreen, unsigned index,
                                struct pipe_driver_query_info *info);
 int iris_get_driver_query_group_info(struct pipe_screen *pscreen,
@@ -1487,7 +1486,7 @@ int iris_get_driver_query_group_info(struct pipe_screen *pscreen,
 void gfx9_toggle_preemption(struct iris_context *ice,
                             struct iris_batch *batch,
                             const struct pipe_draw_info *draw);
-static const bool
+static bool
 iris_execute_indirect_draw_supported(const struct iris_context *ice,
                                      const struct pipe_draw_indirect_info *indirect,
                                      const struct pipe_draw_info *draw)
