@@ -29,21 +29,29 @@
 
 #include "util/slab.h"
 #include "util/list.h"
-#include "os/os_thread.h"
+#include "util/disk_cache.h"
+#include "util/u_thread.h"
 
 #include "pipe/p_screen.h"
 
-#define LIMA_DEBUG_GP      (1 << 0)
-#define LIMA_DEBUG_PP      (1 << 1)
-#define LIMA_DEBUG_DUMP    (1 << 2)
-#define LIMA_DEBUG_SHADERDB (1 << 3)
-#define LIMA_DEBUG_NO_BO_CACHE (1 << 4)
+#define LIMA_DEBUG_GP             (1 << 0)
+#define LIMA_DEBUG_PP             (1 << 1)
+#define LIMA_DEBUG_DUMP           (1 << 2)
+#define LIMA_DEBUG_SHADERDB       (1 << 3)
+#define LIMA_DEBUG_NO_BO_CACHE    (1 << 4)
+#define LIMA_DEBUG_BO_CACHE       (1 << 5)
+#define LIMA_DEBUG_NO_TILING      (1 << 6)
+#define LIMA_DEBUG_NO_GROW_HEAP   (1 << 7)
+#define LIMA_DEBUG_SINGLE_JOB     (1 << 8)
+#define LIMA_DEBUG_PRECOMPILE     (1 << 9)
+#define LIMA_DEBUG_DISK_CACHE     (1 << 10)
+#define LIMA_DEBUG_NO_BLIT        (1 << 11)
 
 extern uint32_t lima_debug;
-extern FILE *lima_dump_command_stream;
 extern int lima_ctx_num_plb;
 extern int lima_plb_max_blk;
 extern int lima_ppir_force_spilling;
+extern int lima_plb_pp_stream_cache_size;
 
 struct ra_regs;
 
@@ -52,13 +60,15 @@ struct ra_regs;
 
 #define NR_BO_CACHE_BUCKETS (MAX_BO_CACHE_BUCKET - MIN_BO_CACHE_BUCKET + 1)
 
+/* const0 1 0 0 -1.67773, mov.v0 $0 ^const0.xxxx, stop */
+#define PP_CLEAR_PROGRAM \
+   0x00020425, 0x0000000c, 0x01e007cf, 0xb0000000, \
+   0x000005f5, 0x00000000, 0x00000000, 0x00000000, \
+
 struct lima_screen {
    struct pipe_screen base;
    struct renderonly *ro;
 
-   int refcnt;
-   void *winsys_priv;
-
    int fd;
    int gpu_type;
    int num_pp;
@@ -67,8 +77,8 @@ struct lima_screen {
    /* bo table */
    mtx_t bo_table_lock;
    mtx_t bo_cache_lock;
-   struct util_hash_table *bo_handles;
-   struct util_hash_table *bo_flink_names;
+   struct hash_table *bo_handles;
+   struct hash_table *bo_flink_names;
    struct list_head bo_cache_buckets[NR_BO_CACHE_BUCKETS];
    struct list_head bo_cache_time;
 
@@ -84,6 +94,9 @@ struct lima_screen {
    #define pp_clear_gl_pos_offset    0x0100
    #define pp_buffer_size            0x1000
 
+   bool has_growable_heap_buffer;
+
+   struct disk_cache *disk_cache;
 };
 
 static inline struct lima_screen *
@@ -93,6 +106,7 @@ lima_screen(struct pipe_screen *pscreen)
 }
 
 struct pipe_screen *
-lima_screen_create(int fd, struct renderonly *ro);
+lima_screen_create(int fd, const struct pipe_screen_config *config,
+                   struct renderonly *ro);
 
 #endif
