@@ -19,8 +19,8 @@
  * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  * OTHER DEALINGS IN THE SOFTWARE.
  */
-#include "codegen/nv50_ir_emit_gv100.h"
-#include "codegen/nv50_ir_sched_gm107.h"
+#include "nv50_ir_emit_gv100.h"
+#include "nv50_ir_sched_gm107.h"
 
 namespace nv50_ir {
 
@@ -313,6 +313,12 @@ CodeEmitterGV100::emitMOV()
          emitField(90,  1, 1);
          emitPRED (87, insn->src(0));
          break;
+      case FILE_BARRIER:
+      case FILE_THREAD_STATE:
+         emitInsn (0x355);
+         emitBTS  (24, insn->src(0));
+         emitGPR  (16, insn->def(0));
+         break;
       default:
          assert(!"bad src file");
          break;
@@ -329,6 +335,31 @@ CodeEmitterGV100::emitMOV()
       emitGPR  (24, insn->src(0));
       emitGPR  (32);
       break;
+   case FILE_BARRIER:
+   case FILE_THREAD_STATE:
+      switch (insn->src(0).getFile()) {
+      case FILE_GPR:
+         emitInsn (0x356);
+         emitGPR  (32, insn->src(0));
+         emitBTS  (24, insn->def(0));
+         break;
+      case FILE_BARRIER:
+         emitInsn (0xf56);
+         emitBTS  (24, insn->def(0));
+         emitBTS  (16, insn->src(0));
+         break;
+      case FILE_THREAD_STATE:
+         assert(insn->def(0).getFile() == FILE_BARRIER);
+         emitInsn (0xf55);
+         emitBTS  (24, insn->src(0));
+         emitBTS  (16, insn->def(0));
+         break;
+      default:
+         assert(!"bad src file");
+         break;
+      }
+      emitField(84, 1, insn->getDef(0)->reg.data.ts == TS_PQUAD_MACTIVE ? 1 : 0);
+      break;
    default:
       assert(!"bad dst file");
       break;
@@ -354,7 +385,16 @@ void
 gv100_selpFlip(const FixupEntry *entry, uint32_t *code, const FixupData& data)
 {
    int loc = entry->loc;
-   if (data.force_persample_interp)
+   bool val = false;
+   switch (entry->ipa) {
+   case 0:
+      val = data.force_persample_interp;
+      break;
+   case 1:
+      val = data.msaa;
+      break;
+   }
+   if (val)
       code[loc + 2] |= 1 << 26;
    else
       code[loc + 2] &= ~(1 << 26);
@@ -366,8 +406,8 @@ CodeEmitterGV100::emitSEL()
    emitFormA(0x007, FA_RRR | FA_RIR | FA_RCR, __(0), __(1), EMPTY);
    emitNOT  (90, insn->src(2));
    emitPRED (87, insn->src(2));
-   if (insn->subOp == 1)
-      addInterp(0, 0, gv100_selpFlip);
+   if (insn->subOp >= 1)
+      addInterp(insn->subOp - 1, 0, gv100_selpFlip);
 }
 
 void
@@ -803,6 +843,7 @@ CodeEmitterGV100::emitALD()
    emitField(74, 2, (insn->getDef(0)->reg.size / 4) - 1);
    emitGPR  (32, insn->src(0).getIndirect(1));
    emitO    (79);
+   emitField(77, 1, insn->subOp); // .PHYS
    emitP    (76);
    emitADDR (24, 40, 10, 0, insn->src(0));
    emitGPR  (16, insn->def(0));
@@ -814,6 +855,7 @@ CodeEmitterGV100::emitAST()
    emitInsn (0x322);
    emitField(74, 2, (typeSizeof(insn->dType) / 4) - 1);
    emitGPR  (64, insn->src(0).getIndirect(1));
+   emitField(77, 1, insn->subOp); // .PHYS
    emitP    (76);
    emitADDR (24, 40, 10, 0, insn->src(0));
    emitGPR  (32, insn->src(1));
@@ -862,7 +904,12 @@ CodeEmitterGV100::emitATOM()
    }
 
    emitPRED (81);
-   emitField(79, 2, 1);
+   if (targ->getChipset() < 0x170) {
+      emitField(79, 2, 2); // .INVALID0/./.STRONG/.INVALID3
+      emitField(77, 2, 3); // .CTA/.SM/.GPU/.SYS
+   } else {
+      emitField(77, 4, 0xa); // .STRONG.SYS
+   }
    emitField(72, 1, insn->src(0).getIndirect(0)->getSize() == 8);
    emitGPR  (32, insn->src(1));
    emitADDR (24, 40, 24, 0, insn->src(0));
@@ -927,7 +974,7 @@ gv100_interpApply(const FixupEntry *entry, uint32_t *code, const FixupData& data
    case NV50_IR_INTERP_DEFAULT : sample = 0; break;
    case NV50_IR_INTERP_CENTROID: sample = 1; break;
    case NV50_IR_INTERP_OFFSET  : sample = 2; break;
-   default: unreachable("invalid sample mode");
+   default: UNREACHABLE("invalid sample mode");
    }
 
    int interp;
@@ -936,7 +983,7 @@ gv100_interpApply(const FixupEntry *entry, uint32_t *code, const FixupData& data
    case NV50_IR_INTERP_PERSPECTIVE: interp = 0; break;
    case NV50_IR_INTERP_FLAT       : interp = 1; break;
    case NV50_IR_INTERP_SC         : interp = 2; break;
-   default: unreachable("invalid ipa mode");
+   default: UNREACHABLE("invalid ipa mode");
    }
 
    code[loc + 2] &= ~(0xf << 12);
@@ -995,18 +1042,23 @@ CodeEmitterGV100::emitLDSTc(int posm, int poso)
 {
    int mode = 0;
    int order = 1;
+   int sm80 = 0;
 
    switch (insn->cache) {
-   case CACHE_CA: mode = 0; order = 1; break;
-   case CACHE_CG: mode = 2; order = 2; break;
-   case CACHE_CV: mode = 3; order = 2; break;
+   case CACHE_CA: mode = 0; order = 1; sm80 = 0x0; break; // .CTA
+   case CACHE_CG: mode = 2; order = 2; sm80 = 0x7; break; // .STRONG.GPU
+   case CACHE_CV: mode = 3; order = 2; sm80 = 0xa; break; // .STRONG.SYS
    default:
       assert(!"invalid caching mode");
       break;
    }
 
-   emitField(poso, 2, order);
-   emitField(posm, 2, mode);
+   if (targ->getChipset() < 0x170) {
+      emitField(poso, 2, order);
+      emitField(posm, 2, mode);
+   } else {
+      emitField(posm, 4, sm80);
+   }
 }
 
 void
@@ -1032,8 +1084,12 @@ void
 CodeEmitterGV100::emitLD()
 {
    emitInsn (0x980);
-   emitField(79, 2, 2); // .CONSTANT/./.STRONG/.MMIO
-   emitField(77, 2, 2); // .CTA/.SM/.GPU/.SYS
+   if (targ->getChipset() < 0x170) {
+      emitField(79, 2, 2); // .CONSTANT/./.STRONG/.MMIO
+      emitField(77, 2, 2); // .CTA/.SM/.GPU/.SYS
+   } else {
+      emitField(77, 4, 0x7); // .STRONG.GPU
+   }
    emitLDSTs(73, insn->dType);
    emitField(72, 1, insn->src(0).getIndirect(0)->getSize() == 8);
    emitADDR (24, 32, 32, 0, insn->src(0));
@@ -1076,8 +1132,11 @@ CodeEmitterGV100::emitOUT()
 
    if (insn->op != OP_FINAL)
       emitFormA(0x124, FA_RRR | FA_RIR, __(0), __(1), EMPTY);
-   else
+   else {
       emitFormA(0x124, FA_RRR | FA_RIR, __(0), EMPTY, EMPTY);
+      if (targ->getChipset() >= 0x170)
+         emitGPR(32);
+   }
    emitField(78, 2, (cut << 1) | emit);
 }
 
@@ -1099,8 +1158,12 @@ CodeEmitterGV100::emitRED()
    emitInsn (0x98e);
    emitField(87, 3, insn->subOp);
    emitField(84, 3, 1); // 0=.EF, 1=, 2=.EL, 3=.LU, 4=.EU, 5=.NA
-   emitField(79, 2, 2); // .INVALID0/./.STRONG/.INVALID3
-   emitField(77, 2, 2); // .CTA/.SM/.GPU/.SYS
+   if (targ->getChipset() < 0x170) {
+      emitField(79, 2, 2); // .INVALID0/./.STRONG/.INVALID3
+      emitField(77, 2, 3); // .CTA/.SM/.GPU/.SYS
+   } else {
+      emitField(77, 4, 0xa); // .STRONG.SYS
+   }
    emitField(73, 3, dType);
    emitField(72, 1, insn->src(0).getIndirect(0)->getSize() == 8);
    emitGPR  (32, insn->src(1));
@@ -1111,8 +1174,12 @@ void
 CodeEmitterGV100::emitST()
 {
    emitInsn (0x385);
-   emitField(79, 2, 2); // .INVALID0/./.STRONG/.MMIO
-   emitField(77, 2, 2); // .CTA/.SM/.GPU/.SYS
+   if (targ->getChipset() < 0x170) {
+      emitField(79, 2, 2); // .INVALID0/./.STRONG/.MMIO
+      emitField(77, 2, 2); // .CTA/.SM/.GPU/.SYS
+   } else {
+      emitField(77, 4, 0x7); // .STRONG.GPU
+   }
    emitLDSTs(73, insn->dType);
    emitField(72, 1, insn->src(0).getIndirect(0)->getSize() == 8);
    emitGPR  (64, insn->src(1));
@@ -1428,7 +1495,8 @@ CodeEmitterGV100::emitSUATOM()
 
    emitField(87, 4, subOp);
    emitPRED (81);
-   emitField(79, 2, 1);
+   if (targ->getChipset() < 0x170)
+      emitField(79, 2, 1);
    emitField(73, 3, type);
    emitField(72, 1, 0); // .BA
    emitGPR  (32, insn->src(1));
@@ -1515,6 +1583,12 @@ CodeEmitterGV100::emitBAR()
 {
    uint8_t subop, redop = 0x00;
 
+   //XXX: ILLEGAL_INSTR_PARAM - why?
+   if (targ->getChipset() >= 0x170) {
+      emitNOP();
+      return;
+   }
+
    // 80
    //    01: DEFER_BLOCKING
    // 78:77
@@ -1743,7 +1817,11 @@ CodeEmitterGV100::emitInstruction(Instruction *i)
    case OP_FLOOR:
    case OP_TRUNC:
       if (insn->op == OP_CVT && (insn->def(0).getFile() == FILE_PREDICATE ||
-                                 insn->src(0).getFile() == FILE_PREDICATE)) {
+                                 insn->def(0).getFile() == FILE_BARRIER ||
+                                 insn->def(0).getFile() == FILE_THREAD_STATE ||
+                                 insn->src(0).getFile() == FILE_PREDICATE ||
+                                 insn->src(0).getFile() == FILE_BARRIER ||
+                                 insn->src(0).getFile() == FILE_THREAD_STATE)) {
          emitMOV();
       } else if (isFloatType(insn->dType)) {
          if (isFloatType(insn->sType)) {
@@ -2043,7 +2121,7 @@ CodeEmitterGV100::prepareEmission(Program *prog)
 }
 
 CodeEmitterGV100::CodeEmitterGV100(TargetGV100 *target)
-   : CodeEmitter(target), targ(target)
+   : CodeEmitter(target), prog(NULL), targ(target), insn(NULL)
 {
    code = NULL;
    codeSize = codeSizeLimit = 0;
