@@ -327,8 +327,7 @@ nv30_set_sample_mask(struct pipe_context *pipe, unsigned sample_mask)
 
 static void
 nv30_set_constant_buffer(struct pipe_context *pipe,
-                         enum pipe_shader_type shader, uint index,
-                         bool pass_reference,
+                         mesa_shader_stage shader, uint index,
                          const struct pipe_constant_buffer *cb)
 {
    struct nv30_context *nv30 = nv30_context(pipe);
@@ -345,23 +344,13 @@ nv30_set_constant_buffer(struct pipe_context *pipe,
    if (buf)
       size = buf->width0 / (4 * sizeof(float));
 
-   if (shader == PIPE_SHADER_VERTEX) {
-      if (pass_reference) {
-         pipe_resource_reference(&nv30->vertprog.constbuf, NULL);
-         nv30->vertprog.constbuf = buf;
-      } else {
-         pipe_resource_reference(&nv30->vertprog.constbuf, buf);
-      }
+   if (shader == MESA_SHADER_VERTEX) {
+      pipe_resource_reference(&nv30->vertprog.constbuf, buf);
       nv30->vertprog.constbuf_nr = size;
       nv30->dirty |= NV30_NEW_VERTCONST;
    } else
-   if (shader == PIPE_SHADER_FRAGMENT) {
-      if (pass_reference) {
-         pipe_resource_reference(&nv30->fragprog.constbuf, NULL);
-         nv30->fragprog.constbuf = buf;
-      } else {
-         pipe_resource_reference(&nv30->fragprog.constbuf, buf);
-      }
+   if (shader == MESA_SHADER_FRAGMENT) {
+      pipe_resource_reference(&nv30->fragprog.constbuf, buf);
       nv30->fragprog.constbuf_nr = size;
       nv30->dirty |= NV30_NEW_FRAGCONST;
    }
@@ -446,7 +435,7 @@ nv30_set_vertex_buffers(struct pipe_context *pipe,
     nouveau_bufctx_reset(nv30->bufctx, BUFCTX_VTXBUF);
 
     util_set_vertex_buffers_count(nv30->vtxbuf, &nv30->num_vtxbufs,
-                                  vb, count, true);
+                                  vb, count);
 
     nv30->dirty |= NV30_NEW_ARRAYS;
 }
