@@ -256,13 +256,13 @@ static int
 nv50_program_assign_varying_slots(struct nv50_ir_prog_info_out *info)
 {
    switch (info->type) {
-   case PIPE_SHADER_VERTEX:
+   case MESA_SHADER_VERTEX:
       return nv50_vertprog_assign_slots(info);
-   case PIPE_SHADER_GEOMETRY:
+   case MESA_SHADER_GEOMETRY:
       return nv50_vertprog_assign_slots(info);
-   case PIPE_SHADER_FRAGMENT:
+   case MESA_SHADER_FRAGMENT:
       return nv50_fragprog_assign_slots(info);
-   case PIPE_SHADER_COMPUTE:
+   case MESA_SHADER_COMPUTE:
       return 0;
    default:
       return -1;
@@ -332,7 +332,7 @@ nv50_program_translate(struct nv50_program *prog, uint16_t chipset,
    struct nv50_ir_prog_info *info;
    struct nv50_ir_prog_info_out info_out = {};
    int i, ret;
-   const uint8_t map_undef = (prog->type == PIPE_SHADER_VERTEX) ? 0x40 : 0x80;
+   const uint8_t map_undef = (prog->type == MESA_SHADER_VERTEX) ? 0x40 : 0x80;
 
    info = CALLOC_STRUCT(nv50_ir_prog_info);
    if (!info)
@@ -370,7 +370,7 @@ nv50_program_translate(struct nv50_program *prog, uint16_t chipset,
    prog->gp.has_layer = 0;
    prog->gp.has_viewport = 0;
 
-   if (prog->type == PIPE_SHADER_COMPUTE)
+   if (prog->type == MESA_SHADER_COMPUTE)
       info->prop.cp.inputOffset = 0x14;
 
    info_out.driverPriv = prog;
@@ -406,7 +406,7 @@ nv50_program_translate(struct nv50_program *prog, uint16_t chipset,
    for (i = 0; i < info_out.io.cullDistances; ++i)
       prog->vp.clip_mode |= 1 << ((info_out.io.clipDistances + i) * 4);
 
-   if (prog->type == PIPE_SHADER_FRAGMENT) {
+   if (prog->type == MESA_SHADER_FRAGMENT) {
       if (info_out.prop.fp.writesDepth) {
          prog->fp.flags[0] |= NV50_3D_FP_CONTROL_EXPORTS_Z;
          prog->fp.flags[1] = 0x11;
@@ -414,7 +414,7 @@ nv50_program_translate(struct nv50_program *prog, uint16_t chipset,
       if (info_out.prop.fp.usesDiscard)
          prog->fp.flags[0] |= NV50_3D_FP_CONTROL_USES_KIL;
    } else
-   if (prog->type == PIPE_SHADER_GEOMETRY) {
+   if (prog->type == MESA_SHADER_GEOMETRY) {
       switch (info_out.prop.gp.outputPrim) {
       case MESA_PRIM_LINE_STRIP:
          prog->gp.prim_type = NV50_3D_GP_OUTPUT_PRIMITIVE_TYPE_LINE_STRIP;
@@ -430,7 +430,7 @@ nv50_program_translate(struct nv50_program *prog, uint16_t chipset,
       }
       prog->gp.vert_count = CLAMP(info_out.prop.gp.maxVertices, 1, 1024);
    } else
-   if (prog->type == PIPE_SHADER_COMPUTE) {
+   if (prog->type == MESA_SHADER_COMPUTE) {
       for (i = 0; i < NV50_MAX_GLOBALS; i++) {
          prog->cp.gmem[i] = (struct nv50_gmem_state){
             .valid = info_out.prop.cp.gmem[i].valid,
@@ -465,10 +465,10 @@ nv50_program_upload_code(struct nv50_context *nv50, struct nv50_program *prog)
    uint8_t prog_type;
 
    switch (prog->type) {
-   case PIPE_SHADER_VERTEX:   heap = nv50->screen->vp_code_heap; break;
-   case PIPE_SHADER_GEOMETRY: heap = nv50->screen->gp_code_heap; break;
-   case PIPE_SHADER_FRAGMENT: heap = nv50->screen->fp_code_heap; break;
-   case PIPE_SHADER_COMPUTE:  heap = nv50->screen->fp_code_heap; break;
+   case MESA_SHADER_VERTEX:   heap = nv50->screen->vp_code_heap; break;
+   case MESA_SHADER_GEOMETRY: heap = nv50->screen->gp_code_heap; break;
+   case MESA_SHADER_FRAGMENT: heap = nv50->screen->fp_code_heap; break;
+   case MESA_SHADER_COMPUTE:  heap = nv50->screen->fp_code_heap; break;
    default:
       assert(!"invalid program type");
       return false;
@@ -493,7 +493,7 @@ nv50_program_upload_code(struct nv50_context *nv50, struct nv50_program *prog)
       }
    }
 
-   if (prog->type == PIPE_SHADER_COMPUTE) {
+   if (prog->type == MESA_SHADER_COMPUTE) {
       /* CP code must be uploaded in FP code segment. */
       prog_type = NV50_SHADER_STAGE_FRAGMENT;
    } else {
