@@ -12,7 +12,6 @@
 #include "util/hex.h"
 #include "util/os_time.h"
 #include "util/xmlconfig.h"
-#include "vl/vl_decoder.h"
 #include "vl/vl_video_buffer.h"
 
 #include "r300_context.h"
@@ -104,34 +103,6 @@ static struct disk_cache* r300_get_disk_shader_cache(struct pipe_screen* pscreen
 	return r300screen->disk_shader_cache;
 }
 
-static int r300_get_video_param(struct pipe_screen *screen,
-				enum pipe_video_profile profile,
-				enum pipe_video_entrypoint entrypoint,
-				enum pipe_video_cap param)
-{
-   switch (param) {
-      case PIPE_VIDEO_CAP_SUPPORTED:
-         return vl_profile_supported(screen, profile, entrypoint);
-      case PIPE_VIDEO_CAP_NPOT_TEXTURES:
-         return 0;
-      case PIPE_VIDEO_CAP_MAX_WIDTH:
-      case PIPE_VIDEO_CAP_MAX_HEIGHT:
-         return vl_video_buffer_max_size(screen);
-      case PIPE_VIDEO_CAP_PREFERRED_FORMAT:
-         return PIPE_FORMAT_NV12;
-      case PIPE_VIDEO_CAP_PREFERS_INTERLACED:
-         return false;
-      case PIPE_VIDEO_CAP_SUPPORTS_INTERLACED:
-         return false;
-      case PIPE_VIDEO_CAP_SUPPORTS_PROGRESSIVE:
-         return true;
-      case PIPE_VIDEO_CAP_MAX_LEVEL:
-         return vl_level_supported(screen, profile);
-      default:
-         return 0;
-   }
-}
-
 #define COMMON_NIR_OPTIONS                    \
    .fdot_replicates = true,                   \
    .fuse_ffma32 = true,                       \
@@ -208,34 +179,10 @@ static const nir_shader_compiler_options gallivm_compiler_options = {
    .has_fused_comp_and_csel = true,
    .max_unroll_iterations = 32,
 
-   .support_indirect_inputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
-   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
+   .support_indirect_inputs = (uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES),
+   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES),
 };
 
-static const struct nir_shader_compiler_options *
-r300_get_compiler_options(struct pipe_screen *pscreen,
-                          enum pipe_shader_type shader)
-{
-   struct r300_screen* r300screen = r300_screen(pscreen);
-
-   if (shader == PIPE_SHADER_VERTEX && !r300screen->caps.has_tcl) {
-      return &gallivm_compiler_options;
-   } else if (r300screen->caps.is_r500) {
-      if (shader == PIPE_SHADER_VERTEX)
-         return &r500_vs_compiler_options;
-       else
-         return &r500_fs_compiler_options;
-   } else {
-      if (shader == PIPE_SHADER_VERTEX) {
-         if (r300screen->caps.is_r400)
-            return &r400_vs_compiler_options;
-
-         return &r300_vs_compiler_options;
-      } else {
-         return &r300_fs_compiler_options;
-      }
-   }
-}
 
 /**
  * Whether the format matches:
@@ -447,7 +394,7 @@ static void r300_init_shader_caps(struct r300_screen* r300screen)
    bool is_r500 = r300screen->caps.is_r500;
 
    struct pipe_shader_caps *caps =
-      (struct pipe_shader_caps *)&r300screen->screen.shader_caps[PIPE_SHADER_VERTEX];
+      (struct pipe_shader_caps *)&r300screen->screen.shader_caps[MESA_SHADER_VERTEX];
 
    if (r300screen->caps.has_tcl) {
       caps->max_instructions =
@@ -489,7 +436,7 @@ static void r300_init_shader_caps(struct r300_screen* r300screen)
    }
    caps->supported_irs = (1 << PIPE_SHADER_IR_NIR) | (1 << PIPE_SHADER_IR_TGSI);
 
-   caps = (struct pipe_shader_caps *)&r300screen->screen.shader_caps[PIPE_SHADER_FRAGMENT];
+   caps = (struct pipe_shader_caps *)&r300screen->screen.shader_caps[MESA_SHADER_FRAGMENT];
 
    caps->max_instructions = is_r500 || is_r400 ? 512 : 96;
    caps->max_alu_instructions = is_r500 || is_r400 ? 512 : 64;
@@ -539,6 +486,7 @@ static void r300_init_screen_caps(struct r300_screen* r300screen)
    caps->clip_halfz = true;
    caps->allow_mapped_buffers_during_execution = true;
    caps->legacy_math_rules = true;
+   caps->query_memory_info = true;
 
    caps->texture_transfer_modes = PIPE_TEXTURE_TRANSFER_BLIT;
 
@@ -666,6 +614,34 @@ static int r300_screen_get_fd(struct pipe_screen *screen)
     return rws->get_fd(rws);
 }
 
+static void r300_query_memory_info(struct pipe_screen *pscreen, 
+                                   struct pipe_memory_info *info)
+{
+   struct r300_screen *rscreen = (struct r300_screen*) pscreen;
+   struct radeon_winsys *ws = rscreen->rws;
+
+   info->total_device_memory = rscreen->info.vram_size_kb;
+   info->total_staging_memory = rscreen->info.gart_size_kb;
+
+   /* The real TTM memory usage is somewhat random, because:
+    *
+    * 1) TTM delays freeing memory, because it can only free it after
+    *    fences expire.
+    *
+    * 2) The memory usage can be really low if big VRAM evictions are
+    *    taking place, but the real usage is well above the size of VRAM.
+    *
+    * Instead, return statistics of this process.
+    */
+   unsigned vram_used = ws->query_value(ws, RADEON_VRAM_USAGE) / 1024;
+   unsigned gtt_used = ws->query_value(ws, RADEON_GTT_USAGE) / 1024;
+
+   info->avail_device_memory = (vram_used > info->total_device_memory) ? 0 : info->total_device_memory - vram_used;
+   info->avail_staging_memory = (gtt_used > info->total_staging_memory) ? 0 : info->total_staging_memory - gtt_used;
+   info->device_memory_evicted = ws->query_value(ws, RADEON_NUM_BYTES_MOVED) / 1024;
+   info->nr_device_memory_evictions = ws->query_value(ws, RADEON_NUM_EVICTIONS);
+}
+
 struct pipe_screen* r300_screen_create(struct radeon_winsys *rws,
                                        const struct pipe_screen_config *config)
 {
@@ -706,16 +682,22 @@ struct pipe_screen* r300_screen_create(struct radeon_winsys *rws,
     r300screen->screen.destroy = r300_destroy_screen;
     r300screen->screen.get_name = r300_get_name;
     r300screen->screen.get_vendor = r300_get_vendor;
-    r300screen->screen.get_compiler_options = r300_get_compiler_options;
     r300screen->screen.get_device_vendor = r300_get_device_vendor;
     r300screen->screen.get_disk_shader_cache = r300_get_disk_shader_cache;
     r300screen->screen.get_screen_fd = r300_screen_get_fd;
-    r300screen->screen.get_video_param = r300_get_video_param;
     r300screen->screen.is_format_supported = r300_is_format_supported;
-    r300screen->screen.is_video_format_supported = vl_video_buffer_is_format_supported;
     r300screen->screen.context_create = r300_create_context;
     r300screen->screen.fence_reference = r300_fence_reference;
     r300screen->screen.fence_finish = r300_fence_finish;
+    r300screen->screen.query_memory_info = r300_query_memory_info;
+
+    r300screen->screen.nir_options[MESA_SHADER_VERTEX] =
+        !r300screen->caps.has_tcl ? &gallivm_compiler_options :
+        r300screen->caps.is_r500 ? &r500_vs_compiler_options :
+        r300screen->caps.is_r400 ? &r400_vs_compiler_options :
+                                   &r300_vs_compiler_options;
+    r300screen->screen.nir_options[MESA_SHADER_FRAGMENT] =
+        r300screen->caps.is_r500 ? &r500_fs_compiler_options : &r300_fs_compiler_options;
 
     r300_init_screen_resource_functions(r300screen);
 
