@@ -1,49 +1,31 @@
 /* -*- mesa-c++  -*-
- *
- * Copyright (c) 2019 Collabora LTD
- *
+ * Copyright 2019 Collabora LTD
  * Author: Gert Wollny <gert.wollny@collabora.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * on the rights to use, copy, modify, merge, publish, distribute, sub
- * license, and/or sell copies of the Software, and to permit persons to whom
- * the Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
- * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
- * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  */
 
 #include "sfn_conditionaljumptracker.h"
+
 #include "sfn_debug.h"
 
+#include <iostream>
+#include <memory>
 #include <stack>
 #include <vector>
-#include <memory>
-#include <iostream>
 
 namespace r600 {
 
+using std::shared_ptr;
 using std::stack;
 using std::vector;
-using std::shared_ptr;
 
 struct StackFrame {
 
    StackFrame(r600_bytecode_cf *s, JumpType t):
-      type(t),
-      start(s)
-   {}
+       type(t),
+       start(s)
+   {
+   }
 
    virtual ~StackFrame();
 
@@ -77,29 +59,26 @@ struct ConditionalJumpTrackerImpl {
 };
 
 ConditionalJumpTrackerImpl::ConditionalJumpTrackerImpl():
-   m_current_loop_stack_pos(0)
+    m_current_loop_stack_pos(0)
 {
-
 }
 
-ConditionalJumpTracker::~ConditionalJumpTracker()
-{
-   delete impl;
-}
+ConditionalJumpTracker::~ConditionalJumpTracker() { delete impl; }
 
 ConditionalJumpTracker::ConditionalJumpTracker()
 {
    impl = new ConditionalJumpTrackerImpl();
 }
 
-void ConditionalJumpTracker::push(r600_bytecode_cf *start, JumpType type)
+void
+ConditionalJumpTracker::push(r600_bytecode_cf *start, JumpType type)
 {
    PStackFrame f;
    switch (type) {
-   case  jt_if:
+   case jt_if:
       f.reset(new IfFrame(start));
       break;
-   case  jt_loop:
+   case jt_loop:
       f.reset(new LoopFrame(start));
       impl->m_loop_stack.push(f);
       break;
@@ -107,7 +86,8 @@ void ConditionalJumpTracker::push(r600_bytecode_cf *start, JumpType type)
    impl->m_jump_stack.push(f);
 }
 
-bool ConditionalJumpTracker::pop(r600_bytecode_cf *final, JumpType type)
+bool
+ConditionalJumpTracker::pop(r600_bytecode_cf *final, JumpType type)
 {
    if (impl->m_jump_stack.empty())
       return false;
@@ -123,7 +103,8 @@ bool ConditionalJumpTracker::pop(r600_bytecode_cf *final, JumpType type)
    return true;
 }
 
-bool ConditionalJumpTracker::add_mid(r600_bytecode_cf *source, JumpType type)
+bool
+ConditionalJumpTracker::add_mid(r600_bytecode_cf *source, JumpType type)
 {
    if (impl->m_jump_stack.empty()) {
       sfn_log << "Jump stack empty\n";
@@ -147,21 +128,21 @@ bool ConditionalJumpTracker::add_mid(r600_bytecode_cf *source, JumpType type)
 }
 
 IfFrame::IfFrame(r600_bytecode_cf *s):
-   StackFrame (s, jt_if)
+    StackFrame(s, jt_if)
 {
 }
 
-StackFrame::~StackFrame()
-{
-}
+StackFrame::~StackFrame() {}
 
-void IfFrame::fixup_mid(r600_bytecode_cf *source)
+void
+IfFrame::fixup_mid(r600_bytecode_cf *source)
 {
    /* JUMP target is ELSE */
    start->cf_addr = source->id;
 }
 
-void IfFrame::fixup_pop(r600_bytecode_cf *final)
+void
+IfFrame::fixup_pop(r600_bytecode_cf *final)
 {
    /* JUMP or ELSE target is one past last CF instruction */
    unsigned offset = final->eg_alu_extended ? 4 : 2;
@@ -171,15 +152,17 @@ void IfFrame::fixup_pop(r600_bytecode_cf *final)
 }
 
 LoopFrame::LoopFrame(r600_bytecode_cf *s):
-   StackFrame(s, jt_loop)
+    StackFrame(s, jt_loop)
 {
 }
 
-void LoopFrame::fixup_mid(UNUSED r600_bytecode_cf *mid)
+void
+LoopFrame::fixup_mid(UNUSED r600_bytecode_cf *mid)
 {
 }
 
-void LoopFrame::fixup_pop(r600_bytecode_cf *final)
+void
+LoopFrame::fixup_pop(r600_bytecode_cf *final)
 {
    /* LOOP END address is past LOOP START */
    final->cf_addr = start->id + 2;
@@ -192,4 +175,4 @@ void LoopFrame::fixup_pop(r600_bytecode_cf *final)
       m->cf_addr = final->id;
 }
 
-}
+} // namespace r600
