@@ -431,7 +431,7 @@ BLOCK_END
 )";
 
    const char *expect =
-R"(
+      R"(
 FS
 CHIPCLASS CAYMAN
 PROP MAX_COLOR_EXPORTS:1
@@ -440,7 +440,7 @@ PROP COLOR_EXPORT_MASK:15
 PROP WRITE_ALL_COLORS:0
 OUTPUT LOC:0 FRAG_RESULT:2 MASK:15
 SHADER
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MIN_UINT S3.w@free : KC0[0].x L[0x2] {WL}
 ALU_GROUP_END
@@ -448,7 +448,7 @@ ALU_GROUP_BEGIN
   ALU MOVA_INT IDX0 : S3.w@free {L}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOV S4.x@chgr : KC1[IDX0][0].x {W}
   ALU MOV S4.y@chgr : KC1[IDX0][0].y {W}
@@ -493,7 +493,7 @@ BLOCK_END
 )";
 
    const char *expect =
-R"(
+      R"(
 FS
 CHIPCLASS EVERGREEN
 PROP MAX_COLOR_EXPORTS:1
@@ -502,7 +502,7 @@ PROP COLOR_EXPORT_MASK:15
 PROP WRITE_ALL_COLORS:0
 OUTPUT LOC:0 FRAG_RESULT:2 MASK:15
 SHADER
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOVA_INT AR : KC0[0].x {}
   ALU MOV S0.y@free : KC0[1].y {W}
@@ -512,15 +512,15 @@ ALU_GROUP_BEGIN
   ALU SET_CF_IDX0 IDX0 : AR {L}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START TEX
   TEX SAMPLE S3.xyzw : S0.zy__ RID:0 SID:0 NNNN
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
-  ALU ADD S4.x@chgr : KC1[IDX0][0].x S3.x {W}
-  ALU ADD S4.y@chgr : KC1[IDX0][0].y S3.y {W}
-  ALU ADD S4.z@chgr : KC1[IDX0][0].z S3.z {W}
-  ALU ADD S4.w@chgr : KC1[IDX0][0].w S3.w {WL}
+  ALU ADD S4.x@chgr : KC1[IDX0][0].x S3.x@chgr {W}
+  ALU ADD S4.y@chgr : KC1[IDX0][0].y S3.y@chgr {W}
+  ALU ADD S4.z@chgr : KC1[IDX0][0].z S3.z@chgr {W}
+  ALU ADD S4.w@chgr : KC1[IDX0][0].w S3.w@chgr {WL}
 ALU_GROUP_END
 BLOCK_END
 BLOCK_START
@@ -559,7 +559,7 @@ BLOCK_END
 )";
 
    const char *expect =
-R"(
+      R"(
 FS
 CHIPCLASS CAYMAN
 PROP MAX_COLOR_EXPORTS:1
@@ -568,7 +568,7 @@ PROP COLOR_EXPORT_MASK:15
 PROP WRITE_ALL_COLORS:0
 OUTPUT LOC:0 FRAG_RESULT:2 MASK:15
 SHADER
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MIN_UINT S3.x@free : KC0[0].x L[0x2] {W}
   ALU MIN_UINT S3.y@free : KC0[0].y L[0x2] {W}
@@ -582,7 +582,7 @@ ALU_GROUP_BEGIN
   ALU MOVA_INT IDX1 : S3.y@free {L}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOV S4.x@chgr : KC1[IDX0][0].x {WL}
 ALU_GROUP_END
@@ -590,12 +590,12 @@ ALU_GROUP_BEGIN
   ALU MOVA_INT IDX0 : S3.z@free {L}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOV S4.z@chgr : KC1[IDX0][0].z {WL}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOV S4.y@chgr : KC1[IDX1][0].y {WL}
 ALU_GROUP_END
@@ -603,7 +603,7 @@ ALU_GROUP_BEGIN
   ALU MOVA_INT IDX1 : S3.w@free {L}
 ALU_GROUP_END
 BLOCK_END
-BLOCK_START
+BLOCK_START ALU
 ALU_GROUP_BEGIN
   ALU MOV S4.w@chgr : KC1[IDX1][0].w {WL}
 ALU_GROUP_END
