@@ -125,17 +125,15 @@ static void scan_io_usage(const nir_shader *nir, struct si_shader_info *info,
    unsigned num_slots = indirect ? nir_intrinsic_io_semantics(intr).num_slots : 1;
 
    if (is_input) {
-      assert(driver_location + num_slots <= ARRAY_SIZE(info->input));
+      assert(driver_location + num_slots <= ARRAY_SIZE(info->input_semantic));
 
       for (unsigned i = 0; i < num_slots; i++) {
          unsigned loc = driver_location + i;
 
-         info->input[loc].semantic = semantic + i;
+         info->input_semantic[loc] = semantic + i;
 
-         if (mask) {
-            info->input[loc].usage_mask |= mask;
+         if (mask)
             info->num_inputs = MAX2(info->num_inputs, loc + 1);
-         }
       }
    } else {
       /* Outputs. */
@@ -352,7 +350,7 @@ static void scan_instruction(const struct nir_shader *nir, struct si_shader_info
       case nir_intrinsic_interp_deref_at_centroid:
       case nir_intrinsic_interp_deref_at_sample:
       case nir_intrinsic_interp_deref_at_offset:
-         unreachable("these opcodes should have been lowered");
+         UNREACHABLE("these opcodes should have been lowered");
          break;
       case nir_intrinsic_ordered_add_loop_gfx12_amd:
          info->uses_atomic_ordered_add = true;
@@ -404,7 +402,7 @@ void si_nir_scan_shader(struct si_screen *sscreen, struct nir_shader *nir,
 
    info->base.use_aco_amd = nir->info.use_aco_amd;
    info->base.writes_memory = nir->info.writes_memory;
-   info->base.subgroup_size = nir->info.subgroup_size;
+   info->base.api_subgroup_size = nir->info.api_subgroup_size;
 
    info->base.num_ubos = nir->info.num_ubos;
    info->base.num_ssbos = nir->info.num_ssbos;
@@ -460,7 +458,7 @@ void si_nir_scan_shader(struct si_screen *sscreen, struct nir_shader *nir,
       break;
 
    default:
-      unreachable("unexpected shader stage");
+      UNREACHABLE("unexpected shader stage");
    }
 
    /* Get options from shader profiles. */
@@ -591,8 +589,7 @@ void si_nir_scan_shader(struct si_screen *sscreen, struct nir_shader *nir,
             if ((info->colors_read >> (i * 4)) & 0xf) {
                unsigned index = num_inputs_with_colors;
 
-               info->input[index].semantic = (back ? VARYING_SLOT_BFC0 : VARYING_SLOT_COL0) + i;
-               info->input[index].usage_mask = info->colors_read >> (i * 4);
+               info->input_semantic[index] = (back ? VARYING_SLOT_BFC0 : VARYING_SLOT_COL0) + i;
                num_inputs_with_colors++;
 
                /* Back-face color don't increment num_inputs. si_emit_spi_map will use
@@ -649,7 +646,7 @@ void si_nir_scan_shader(struct si_screen *sscreen, struct nir_shader *nir,
 
    if (nir->info.stage == MESA_SHADER_FRAGMENT) {
       for (unsigned i = 0; i < info->num_inputs; i++) {
-         unsigned semantic = info->input[i].semantic;
+         unsigned semantic = info->input_semantic[i];
 
          if ((semantic <= VARYING_SLOT_VAR31 || semantic >= VARYING_SLOT_VAR0_16BIT) &&
              semantic != VARYING_SLOT_PNTC) {
@@ -662,16 +659,16 @@ void si_nir_scan_shader(struct si_screen *sscreen, struct nir_shader *nir,
             info->colors_written_4bit |= 0xf << (4 * i);
 
       for (unsigned i = 0; i < info->num_inputs; i++) {
-         if (info->input[i].semantic == VARYING_SLOT_COL0)
+         if (info->input_semantic[i] == VARYING_SLOT_COL0)
             info->color_attr_index[0] = i;
-         else if (info->input[i].semantic == VARYING_SLOT_COL1)
+         else if (info->input_semantic[i] == VARYING_SLOT_COL1)
             info->color_attr_index[1] = i;
       }
    }
 }
 
 enum ac_hw_stage
-si_select_hw_stage(const gl_shader_stage stage, const union si_shader_key *const key,
+si_select_hw_stage(const mesa_shader_stage stage, const union si_shader_key *const key,
                    const enum amd_gfx_level gfx_level)
 {
    switch (stage) {
@@ -698,6 +695,6 @@ si_select_hw_stage(const gl_shader_stage stage, const union si_shader_key *const
    case MESA_SHADER_KERNEL:
       return AC_HW_COMPUTE_SHADER;
    default:
-      unreachable("Unsupported HW stage");
+      UNREACHABLE("Unsupported HW stage");
    }
 }
