@@ -176,7 +176,7 @@ VKAPI_ATTR VkResult VKAPI_CALL lvp_CreateDescriptorSetLayout(
          break;
       }
 
-      set_layout->shader_stages |= binding->stageFlags;
+      set_layout->shader_stages |= binding->stageFlags & MESA_VK_SHADER_STAGE_ALL;
    }
 
    for (uint32_t i = 0; i < pCreateInfo->bindingCount; i++)
@@ -639,7 +639,7 @@ VKAPI_ATTR void VKAPI_CALL lvp_UpdateDescriptorSets(
          break;
 
       default:
-         unreachable("Unsupported descriptor type");
+         UNREACHABLE("Unsupported descriptor type");
          break;
       }
    }
@@ -933,7 +933,7 @@ lvp_descriptor_set_update_with_template(VkDevice _device, VkDescriptorSet descri
          }
 
          default:
-            unreachable("Unsupported descriptor type");
+            UNREACHABLE("Unsupported descriptor type");
             break;
          }
 
@@ -996,7 +996,7 @@ VKAPI_ATTR void VKAPI_CALL lvp_GetDescriptorEXT(
 
    switch (pCreateInfo->type) {
    case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK: {
-      unreachable("this is a spec violation");
+      UNREACHABLE("this is a spec violation");
       break;
    }
    case VK_DESCRIPTOR_TYPE_SAMPLER: {
@@ -1138,7 +1138,7 @@ VKAPI_ATTR void VKAPI_CALL lvp_GetDescriptorEXT(
       break;
    }
    default:
-      unreachable("Unsupported descriptor type");
+      UNREACHABLE("Unsupported descriptor type");
       break;
    }
 }
