@@ -204,8 +204,6 @@ enum tgsi_semantic {
    TGSI_SEMANTIC_SUBGROUP_LT_MASK,
    TGSI_SEMANTIC_CS_USER_DATA_AMD,
    TGSI_SEMANTIC_VIEWPORT_MASK,
-   TGSI_SEMANTIC_TESS_DEFAULT_OUTER_LEVEL, /**< from set_tess_state */
-   TGSI_SEMANTIC_TESS_DEFAULT_INNER_LEVEL, /**< from set_tess_state */
    TGSI_SEMANTIC_COUNT,       /**< number of semantic values */
 };
 
@@ -233,7 +231,6 @@ enum tgsi_return_type {
    TGSI_RETURN_TYPE_SINT,
    TGSI_RETURN_TYPE_UINT,
    TGSI_RETURN_TYPE_FLOAT,
-   TGSI_RETURN_TYPE_UNKNOWN,
    TGSI_RETURN_TYPE_COUNT
 };
 
@@ -568,7 +565,6 @@ enum tgsi_opcode {
    TGSI_OPCODE_DMAD               = 209,
    TGSI_OPCODE_DFRAC              = 210 /* eg, cayman */,
    TGSI_OPCODE_DLDEXP             = 211 /* eg, cayman */,
-   TGSI_OPCODE_DFRACEXP           = 212 /* eg, cayman */,
    TGSI_OPCODE_D2I                = 213,
    TGSI_OPCODE_I2D                = 214,
    TGSI_OPCODE_D2U                = 215,
@@ -704,8 +700,7 @@ struct tgsi_instruction_texture
 {
    unsigned Texture  : 8;    /* TGSI_TEXTURE_ */
    unsigned NumOffsets : 4;
-   unsigned ReturnType : 3; /* TGSI_RETURN_TYPE_x */
-   unsigned Padding : 17;
+   unsigned Padding : 20;
 };
 
 /* for texture offsets in GLSL and DirectX.
