@@ -10,9 +10,6 @@
 #include <stdio.h>
 #include <vector>
 #include <gtest/gtest.h>
-#include <xtensor/xrandom.hpp>
-
-#include "util/macros.h"
 
 #include "tensorflow/lite/c/c_api.h"
 #include "tensorflow/lite/c/common.h"
@@ -27,6 +24,28 @@ randf(float min, float max)
    return ((max - min) * ((float)rand() / (float)RAND_MAX)) + min;
 }
 
+template<typename T>
+std::vector<T> rand(const std::vector<int>& shape, T min, T max) {
+    size_t size = 1;
+    for (int dim : shape) {
+        size *= dim;
+    }
+
+    std::vector<T> result(size);
+
+    if constexpr (std::is_integral<T>::value) {
+      std::vector<T> result(size);
+      std::generate(result.begin(), result.end(), [&]() { return rand() % (max - min + 1) + min; });
+      return result;
+    } else if constexpr (std::is_floating_point<T>::value) {
+      std::vector<T> result(size);
+      std::generate(result.begin(), result.end(), [&]() { return randf(-1.0, 1.0); });
+      return result;
+    }
+
+    return result;
+}
+
 static void
 read_model(const char *file_name, tflite::ModelT &model)
 {
@@ -111,7 +130,7 @@ patch_conv2d(unsigned operation_index,
    bias_tensor->shape.data()[0] = output_channels;
 
    auto bias_data = &model->buffers[bias_buffer_index]->data;
-   xt::xarray<int32_t> bias_array = xt::random::randint<int32_t>({output_channels}, -20000, 20000);
+   std::vector<int32_t> bias_array = rand<int32_t>({output_channels}, -20000, 20000);
    bias_data->resize(bias_array.size() * sizeof(int32_t));
    memcpy(bias_data->data(), bias_array.data(), bias_array.size() * sizeof(int32_t));
 
@@ -140,7 +159,7 @@ patch_conv2d(unsigned operation_index,
    else
       weight_shape = {output_channels, weight_size, weight_size, input_channels};
 
-   xt::xarray<uint8_t> weights_array = xt::random::randint<uint8_t>(weight_shape, 0, 255);
+   std::vector<uint8_t> weights_array = rand<uint8_t>(weight_shape, 0, 255);
    weights_data->resize(weights_array.size());
    memcpy(weights_data->data(), weights_array.data(), weights_array.size());
 
@@ -250,6 +269,8 @@ add_generate_model(int input_size,
    return buf;
 }
 
+
+
 static void
 patch_fully_connected(unsigned operation_index,
                       tflite::ModelT *model,
@@ -287,7 +308,7 @@ patch_fully_connected(unsigned operation_index,
    bias_tensor->shape.data()[0] = output_channels;
 
    auto bias_data = &model->buffers[bias_buffer_index]->data;
-   xt::xarray<int32_t> bias_array = xt::random::randint<int32_t>({output_channels}, -20000, 20000);
+   std::vector<int32_t> bias_array = rand<int32_t>({output_channels}, -20000, 20000);
    bias_data->resize(bias_array.size() * sizeof(int32_t));
    memcpy(bias_data->data(), bias_array.data(), bias_array.size() * sizeof(int32_t));
 
@@ -302,7 +323,7 @@ patch_fully_connected(unsigned operation_index,
    std::vector<int> weight_shape;
    weight_shape = {output_channels, input_size};
 
-   xt::xarray<uint8_t> weights_array = xt::random::randint<uint8_t>(weight_shape, 0, 255);
+   std::vector<uint8_t> weights_array = rand<uint8_t>(weight_shape, 0, 255);
    weights_data->resize(weights_array.size());
    memcpy(weights_data->data(), weights_array.data(), weights_array.size());
 
@@ -391,7 +412,7 @@ read_buf(const char *path, size_t *buf_size)
 
    fclose(f);
 
-   if(buf_size != NULL)
+   if (buf_size != NULL)
       *buf_size = fsize;
 
    return buf;
@@ -420,7 +441,7 @@ run_model(TfLiteModel *model, enum executor executor, void ***input, size_t *num
 
    *num_inputs = TfLiteInterpreterGetInputTensorCount(interpreter);
    if (*input == NULL)
-      *input = (void**)calloc(*num_inputs, sizeof(*input));
+      *input = (void **)calloc(*num_inputs, sizeof(*input));
    for (unsigned i = 0; i < *num_inputs; i++) {
       TfLiteTensor *input_tensor = TfLiteInterpreterGetInputTensor(interpreter, i);
       std::ostringstream input_cache;
@@ -432,23 +453,23 @@ run_model(TfLiteModel *model, enum executor executor, void ***input, size_t *num
          if ((*input)[i] == NULL) {
             (*input)[i] = malloc(input_tensor->bytes);
 
-            std::vector<size_t> shape;
+            std::vector<int> shape;
 
             shape.resize(input_tensor->dims->size);
             for (int j = 0; j < input_tensor->dims->size; j++)
                shape[j] = input_tensor->dims->data[j];
 
             switch (input_tensor->type) {
-               case kTfLiteFloat32: {
-                  xt::xarray<float_t> a = xt::random::rand<float_t>(shape);
-                  memcpy((*input)[i], a.data(), input_tensor->bytes);
-                  break;
-               }
-               default: {
-                  xt::xarray<uint8_t> a = xt::random::randint<uint8_t>(shape, 0, 255);
-                  memcpy((*input)[i], a.data(), input_tensor->bytes);
-                  break;
-               }
+            case kTfLiteFloat32: {
+               std::vector<float> a = rand<float>(shape, -1.0, 1.0);
+               memcpy((*input)[i], a.data(), input_tensor->bytes);
+               break;
+            }
+            default: {
+               std::vector<uint8_t> a = rand<uint8_t>(shape, 0, 255);
+               memcpy((*input)[i], a.data(), input_tensor->bytes);
+               break;
+            }
             }
 
             if (cache_is_enabled()) {
@@ -473,9 +494,9 @@ run_model(TfLiteModel *model, enum executor executor, void ***input, size_t *num
    }
 
    *num_outputs = TfLiteInterpreterGetOutputTensorCount(interpreter);
-   *output = (void**)malloc(sizeof(*output) * *num_outputs);
-   *output_sizes = (size_t*)malloc(sizeof(*output_sizes) * *num_outputs);
-   *output_types = (TfLiteType*)malloc(sizeof(*output_types) * *num_outputs);
+   *output = (void **)malloc(sizeof(*output) * *num_outputs);
+   *output_sizes = (size_t *)malloc(sizeof(*output_sizes) * *num_outputs);
+   *output_types = (TfLiteType *)malloc(sizeof(*output_types) * *num_outputs);
    for (unsigned i = 0; i < *num_outputs; i++) {
       const TfLiteTensor *output_tensor = TfLiteInterpreterGetOutputTensor(interpreter, i);
       output_cache.str("");
@@ -496,14 +517,14 @@ run_model(TfLiteModel *model, enum executor executor, void ***input, size_t *num
       }
 
       switch (output_tensor->type) {
-         case kTfLiteFloat32: {
-            (*output_sizes)[i] = output_tensor->bytes / 4;
-            break;
-         }
-         default: {
-            (*output_sizes)[i] = output_tensor->bytes;
-            break;
-         }
+      case kTfLiteFloat32: {
+         (*output_sizes)[i] = output_tensor->bytes / 4;
+         break;
+      }
+      default: {
+         (*output_sizes)[i] = output_tensor->bytes;
+         break;
+      }
       }
    }
 
