@@ -134,7 +134,7 @@ static void get_hs_info(struct radeon_info *info)
       max_hs_out_vram_dwords_enum = V_03093C_X_1K_DWORDS;
       break;
    default:
-      unreachable("invalid TCS workgroup size");
+      UNREACHABLE("invalid TCS workgroup size");
    }
 #endif
 
@@ -432,7 +432,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
                         &ws->info.max_gpu_freq_mhz);
    ws->info.max_gpu_freq_mhz /= 1000;
 
-   ws->num_cpus = sysconf(_SC_NPROCESSORS_ONLN);
+   ws->num_cpus = util_get_cpu_caps()->nr_cpus;
 
    /* Generation-specific queries. */
    if (ws->gen == DRV_R300) {
@@ -653,8 +653,6 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    ws->info.max_vgpr_alloc = 256;
    ws->info.wave64_vgpr_alloc_granularity = 4;
    ws->info.lds_size_per_workgroup = ws->info.gfx_level == GFX7 ? 64 * 1024 : 32 * 1024;
-   ws->info.lds_encode_granularity = ws->info.gfx_level == GFX7 ? 128 * 4 : 64 * 4;
-   ws->info.lds_alloc_granularity = ws->info.lds_encode_granularity;
 
    for (unsigned se = 0; se < ws->info.max_se; se++) {
       for (unsigned sa = 0; sa < ws->info.max_sa_per_se; sa++)
@@ -1041,7 +1039,15 @@ radeon_drm_winsys_create(int fd, const struct pipe_screen_config *config,
    ws->vm64.end = 1ull << 33;
 
    /* TTM aligns the BO size to the CPU page size */
-   ws->info.gart_page_size = sysconf(_SC_PAGESIZE);
+   uint64_t page_size = 0;
+   if (!os_get_page_size(&page_size)) {
+      radeon_winsys_destroy(&ws->base);
+      simple_mtx_unlock(&fd_tab_mutex);
+      return NULL;
+   }
+
+   ws->info.gart_page_size = page_size;
+
    ws->info.pte_fragment_size = 64 * 1024; /* GPUVM page size */
 
    if (ws->num_cpus > 1 && debug_get_option_thread())
