@@ -105,14 +105,14 @@
 /* Number of TEXSTATE_SAMPLER state words that need setting up. */
 #define ROGUE_NUM_TEXSTATE_SAMPLER_WORDS 2U
 
+#define ROGUE_NUM_TEXSTATE_DWORDS 4U
+
 /* 12 dwords reserved for shared register management. The first dword is the
  * number of shared register blocks to reload. Should be a multiple of 4 dwords,
  * size in bytes.
  */
 #define ROGUE_LLS_SHARED_REGS_RESERVE_SIZE 48U
 
-#define ROGUE_USC_TASK_PROGRAM_SIZE 512U
-
 #define ROGUE_CSRM_LINE_SIZE_IN_DWORDS (64U * 4U * 4U)
 
 /* The maximum amount of local memory which can be allocated by a single kernel
@@ -173,4 +173,31 @@ enum {
    ROGUE_USC_COEFFICIENT_SET_SIZE,
 };
 
+enum {
+   ROGUE_SMP_COEFF_UFRAC = 0,
+   ROGUE_SMP_COEFF_VFRAC,
+   ROGUE_SMP_COEFF_SFRAC,
+   ROGUE_SMP_COEFF_LOD_DVAL_POST_CLAMP,
+   ROGUE_SMP_COEFF_LOD_DVAL_PRE_CLAMP,
+   ROGUE_SMP_COEFF_TFRAC_POST_CLAMP,
+   ROGUE_SMP_COEFF_TFRAC_PRE_CLAMP,
+   ROGUE_SMP_COEFF_COUNT,
+};
+
+#define ROGUE_SMP_MAX_DATA_WORDS 18U
+
+#define ROGUE_SMP_LOOKUP_MS_NUM_START 16U
+#define ROGUE_SMP_LOOKUP_MS_NUM_BITS 3U
+
+#define ROGUE_SMP_LOOKUP_W_OFF_START 12U
+#define ROGUE_SMP_LOOKUP_W_OFF_BITS 4U
+
+#define ROGUE_SMP_LOOKUP_V_OFF_START 6U
+#define ROGUE_SMP_LOOKUP_V_OFF_BITS 6U
+
+#define ROGUE_SMP_LOOKUP_U_OFF_START 0U
+#define ROGUE_SMP_LOOKUP_U_OFF_BITS 6U
+
+#define ROGUE_REG_SIZE_BYTES 4U
+
 #endif /* ROGUE_HW_DEFS_H */
