@@ -97,7 +97,7 @@ static void pco_assert(struct val_state *state,
          break;
 
       default:
-         unreachable("");
+         UNREACHABLE("");
       }
 
       printf(" (");
@@ -303,6 +303,7 @@ static inline bool ref_is_in_map(pco_ref ref, enum pco_ref_map ref_maps)
 
          CHECK_IO(FT0, ref)
          CHECK_IO(FT1, ref)
+         CHECK_IO(FT1_INVERT, ref)
          CHECK_IO(FT2, ref)
          CHECK_IO(FTE, ref)
          CHECK_IO(FT3, ref)
@@ -344,7 +345,7 @@ static inline bool ref_is_in_map(pco_ref ref, enum pco_ref_map ref_maps)
          break;
 
       default:
-         unreachable("");
+         UNREACHABLE("");
       }
    }
 
