@@ -27,6 +27,8 @@
  * SOFTWARE.
  */
 
+#include "pvr_csb.h"
+
 #include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -35,10 +37,10 @@
 
 #include "hwdef/rogue_hw_utils.h"
 #include "pvr_bo.h"
-#include "pvr_csb.h"
 #include "pvr_debug.h"
+#include "pvr_device.h"
 #include "pvr_device_info.h"
-#include "pvr_private.h"
+#include "pvr_macros.h"
 #include "pvr_types.h"
 #include "util/list.h"
 #include "util/u_dynarray.h"
@@ -201,7 +203,7 @@ pvr_csb_emit_link_unmarked(struct pvr_csb *csb, pvr_dev_addr_t addr, bool ret)
       break;
 
    default:
-      unreachable("Unknown stream type");
+      UNREACHABLE("Unknown stream type");
       break;
    }
 }
@@ -471,7 +473,7 @@ VkResult pvr_csb_emit_terminate(struct pvr_csb *csb)
       break;
 
    default:
-      unreachable("Unknown stream type");
+      UNREACHABLE("Unknown stream type");
       break;
    }
 
