@@ -26,12 +26,14 @@
 #include <stdint.h>
 #include <vulkan/vulkan.h>
 
+#include "pvr_cmd_buffer.h"
 #include "pvr_csb.h"
 #include "pvr_debug.h"
+#include "pvr_device.h"
 #include "pvr_job_common.h"
 #include "pvr_job_context.h"
 #include "pvr_job_compute.h"
-#include "pvr_private.h"
+#include "pvr_macros.h"
 #include "pvr_types.h"
 #include "pvr_winsys.h"
 #include "util/macros.h"
@@ -120,14 +122,13 @@ pvr_submit_info_stream_init(struct pvr_compute_ctx *ctx,
    }
 
    if (PVR_HAS_FEATURE(dev_info, tpu_dm_global_registers)) {
-      pvr_csb_pack (stream_ptr, CR_TPU_TAG_CDM_CTRL, value) {
-      }
+      pvr_csb_pack (stream_ptr, CR_TPU_TAG_CDM_CTRL, value) {}
       stream_ptr += pvr_cmd_length(CR_TPU_TAG_CDM_CTRL);
    }
 
    if (PVR_HAS_FEATURE(dev_info, gpu_multicore_support)) {
-      pvr_finishme(
-         "Emit execute_count when feature gpu_multicore_support is present");
+      if (device->pdevice->dev_runtime_info.core_count > 1)
+         pvr_finishme("Emit execute_count, core_count is greater than one");
       *stream_ptr = 0;
       stream_ptr++;
    }
