@@ -132,6 +132,18 @@ bool intel_gem_create_context_ext(int fd, enum intel_gem_create_context_flags fl
 bool intel_gem_supports_protected_context(int fd,
                                           enum intel_kmd_type kmd_type);
 
+/* Definitions used by the execbuf ioctl wrappers in the backends.
+ * With the values below, after 80 retries we have slept ~16s, and the warning
+ * is given at ~2s.
+ */
+#define INTEL_EXEC_IOCTL_MAX_RETRIES 80
+#define INTEL_EXEC_IOCTL_RETRY_WARN 40
+static inline int
+intel_exec_ioctl_sleep_us(int retries)
+{
+   return 100 * retries * retries;
+}
+
 #define DRM_IOCTL_I915_LAST             DRM_IO(DRM_COMMAND_END - 1)
 
 struct drm_intel_stub_devinfo {
