@@ -44,6 +44,7 @@ enum intel_measure_snapshot_type {
    INTEL_SNAPSHOT_HIZ_AMBIGUATE,
    INTEL_SNAPSHOT_HIZ_CLEAR,
    INTEL_SNAPSHOT_HIZ_RESOLVE,
+   INTEL_SNAPSHOT_MCS_AMBIGUATE,
    INTEL_SNAPSHOT_MCS_COLOR_CLEAR,
    INTEL_SNAPSHOT_MCS_PARTIAL_RESOLVE,
    INTEL_SNAPSHOT_SLOW_COLOR_CLEAR,
@@ -60,11 +61,16 @@ enum intel_measure_events {
    INTEL_MEASURE_FRAME      = (1 << 4),
 };
 
+enum intel_measure_device_type { INTEL_MEASURE_DEVICE_VK=0, INTEL_MEASURE_DEVICE_OGL };
+
 struct intel_measure_config {
 
    /* Stderr, or optionally set with INTEL_MEASURE=file={path{ */
    FILE                      *file;
 
+   /* Defer log file create until first write */
+   char                      *deferred_create_filename;
+
    /* Events that will be measured.  Set only one flag, with
     * INTEL_MEASURE=[draw,rt,shader,batch,frame] */
    enum intel_measure_events  flags;
@@ -100,6 +106,9 @@ struct intel_measure_config {
 
    /* true when snapshots are currently being collected */
    bool                       enabled;
+
+   /* Measure CPU timing, not GPU timing */
+   bool                       cpu_measure;
 };
 
 struct intel_measure_batch;
@@ -108,15 +117,17 @@ struct intel_measure_snapshot {
    enum intel_measure_snapshot_type type;
    unsigned count, event_count;
    const char* event_name;
-   uintptr_t framebuffer, vs, tcs, tes, gs, fs, cs;
+   uint32_t renderpass;
+   uint32_t vs, tcs, tes, gs, fs, cs, ms, ts;
    /* for vulkan secondary command buffers */
    struct intel_measure_batch *secondary;
 };
 
 struct intel_measure_buffered_result {
    struct intel_measure_snapshot snapshot;
-   uint64_t start_ts, end_ts, idle_duration;
-   unsigned frame, batch_count, event_index;
+   uint64_t start_ts, end_ts, idle_duration, batch_size;
+   unsigned frame, batch_count, event_index, primary_renderpass;
+;
 };
 
 struct intel_measure_ringbuffer {
@@ -124,9 +135,15 @@ struct intel_measure_ringbuffer {
    struct intel_measure_buffered_result results[0];
 };
 
+/* This function will be called when enqueued snapshots have been processed */
+typedef void (*intel_measure_release_batch_cb)(struct intel_measure_batch *base);
+
 struct intel_measure_device {
    struct intel_measure_config *config;
    unsigned frame;
+   unsigned render_pass_count;
+   intel_measure_release_batch_cb release_batch;
+   enum intel_measure_device_type type;
 
    /* Holds the list of (iris/anv)_measure_batch snapshots that have been
     * submitted for rendering, but have not completed.
@@ -144,7 +161,8 @@ struct intel_measure_batch {
    struct list_head link;
    unsigned index;
    unsigned frame, batch_count, event_count;
-   uintptr_t framebuffer;
+   uint64_t batch_size;
+   uint32_t renderpass, primary_renderpass;
    uint64_t *timestamps;
    struct intel_measure_snapshot snapshots[0];
 };
@@ -152,14 +170,22 @@ struct intel_measure_batch {
 void intel_measure_init(struct intel_measure_device *device);
 const char * intel_measure_snapshot_string(enum intel_measure_snapshot_type type);
 bool intel_measure_state_changed(const struct intel_measure_batch *batch,
-                                 uintptr_t vs, uintptr_t tcs, uintptr_t tes,
-                                 uintptr_t gs, uintptr_t fs, uintptr_t cs);
+                                 uint32_t vs, uint32_t tcs, uint32_t tes,
+                                 uint32_t gs, uint32_t fs, uint32_t cs,
+                                 uint32_t ms, uint32_t ts);
 void intel_measure_frame_transition(unsigned frame);
 
 bool intel_measure_ready(struct intel_measure_batch *batch);
 
-struct gen_device_info;
+struct intel_device_info;
+void intel_measure_print_cpu_result(unsigned int frame,
+                                    unsigned int batch_count,
+                                    uint64_t batch_size,
+                                    unsigned int event_index,
+                                    unsigned int event_count,
+                                    unsigned int count,
+                                    const char* event_name);
 void intel_measure_gather(struct intel_measure_device *device,
-                          struct gen_device_info *info);
+                          const struct intel_device_info *info);
 
 #endif /* INTEL_MEASURE_H */
