@@ -1913,7 +1913,7 @@ compaction_state_init(struct compaction_state *c,
       c->src1_index_table = g45_src_index_table;
       break;
    default:
-      unreachable("unknown generation");
+      UNREACHABLE("unknown generation");
    }
 }
 
@@ -2122,7 +2122,7 @@ elk_compact_instructions(struct elk_codegen *p, int start_offset,
    if (disasm) {
       int offset = 0;
 
-      foreach_list_typed(struct inst_group, group, link, &disasm->group_list) {
+      brw_foreach_list_typed(struct inst_group, group, link, &disasm->group_list) {
          while (start_offset + old_ip[offset / sizeof(elk_compact_inst)] *
                 sizeof(elk_inst) != group->offset) {
             assert(start_offset + old_ip[offset / sizeof(elk_compact_inst)] *
