@@ -205,7 +205,7 @@ count_to_loop_end(const elk_bblock_t *block)
             return block->end_ip;
       }
    }
-   unreachable("not reached");
+   UNREACHABLE("not reached");
 }
 
 void elk_fs_visitor::calculate_payload_ranges(unsigned payload_node_count,
@@ -1015,8 +1015,8 @@ elk_fs_reg_alloc::spill_reg(unsigned spill_reg)
    int ip = 0;
    foreach_block_and_inst (block, elk_fs_inst, inst, fs->cfg) {
       const fs_builder ibld = fs_builder(fs, block, inst);
-      exec_node *before = inst->prev;
-      exec_node *after = inst->next;
+      brw_exec_node *before = inst->prev;
+      brw_exec_node *after = inst->next;
 
       for (unsigned int i = 0; i < inst->sources; i++) {
 	 if (inst->src[i].file == VGRF &&
