@@ -23,6 +23,7 @@
 
 #include "intel_decoder.h"
 #include "intel_decoder_private.h"
+#include "intel/common/intel_gem.h"
 
 #include "util/macros.h"
 #include "util/u_debug.h"
@@ -61,7 +62,7 @@ intel_batch_decode_ctx_init(struct intel_batch_decode_ctx *ctx,
    ctx->get_state_size = get_state_size;
    ctx->user_data = user_data;
    ctx->fp = fp;
-   ctx->flags = parse_enable_string(getenv("INTEL_DECODE"), flags, debug_control);
+   ctx->flags = parse_enable_string(os_get_option("INTEL_DECODE"), flags, debug_control);
    ctx->max_vbo_decoded_lines = -1; /* No limit! */
    ctx->engine = INTEL_ENGINE_CLASS_RENDER;
 
@@ -75,7 +76,7 @@ intel_batch_decode_ctx_init(struct intel_batch_decode_ctx *ctx,
    ctx->stats =
       _mesa_hash_table_create(NULL, _mesa_hash_string, _mesa_key_string_equal);
 
-   const char *filters = getenv("INTEL_DECODE_FILTERS");
+   const char *filters = os_get_option("INTEL_DECODE_FILTERS");
    if (filters != NULL) {
       ctx->filters =
          _mesa_hash_table_create(NULL, _mesa_hash_string, _mesa_key_string_equal);
@@ -134,7 +135,7 @@ ctx_get_bo(struct intel_batch_decode_ctx *ctx, bool ppgtt, uint64_t addr)
        * bits. In order to correctly handle those aub dumps, we need to mask
        * off the top 16 bits.
        */
-      addr &= (~0ull >> 16);
+      addr = intel_48b_address(addr);
    }
 
    struct intel_batch_decode_bo bo = ctx->get_bo(ctx->user_data, ppgtt, addr);
@@ -407,7 +408,6 @@ dump_samplers(struct intel_batch_decode_ctx *ctx, uint32_t offset, int count)
 
    if (count * sampler_state_size >= bo.size) {
       fprintf(ctx->fp, "  sampler state ends after bo ends\n");
-      assert(!"sampler state ends after bo ends");
       return;
    }
 
@@ -1498,6 +1498,7 @@ struct custom_decoder state_handlers[] = {
 /* Special printing of instructions */
 struct custom_decoder custom_decoders[] = {
    { "COMPUTE_WALKER", handle_compute_walker },
+   { "EXECUTE_INDIRECT_DISPATCH", handle_compute_walker },
    { "MEDIA_CURBE_LOAD", handle_media_curbe_load },
    { "3DSTATE_VERTEX_BUFFERS", handle_3dstate_vertex_buffers },
    { "3DSTATE_INDEX_BUFFER", handle_3dstate_index_buffer },
