@@ -48,9 +48,12 @@ def define_tracepoints(args):
     def begin_end_tp(name, tp_args=[], tp_struct=None, tp_print=None,
                      tp_default_enabled=True, end_pipelined=True,
                      compute=False, repeat_last=False,
-                     need_cs_param=False):
+                     need_cs_param=False,
+                     toggle_name=None):
         global intel_default_tps
-        if tp_default_enabled:
+        if toggle_name is None:
+            toggle_name = name
+        if tp_default_enabled and toggle_name not in intel_default_tps:
             intel_default_tps.append(name)
 
         # Preprocess arguments to handle display_as_hex
@@ -62,7 +65,7 @@ def define_tracepoints(args):
             processed_args.append(arg)
 
         Tracepoint('intel_begin_{0}'.format(name),
-                   toggle_name=name,
+                   toggle_name=toggle_name,
                    tp_perfetto='intel_ds_begin_{0}'.format(name),
                    need_cs_param=need_cs_param)
         tp_flags = []
@@ -74,7 +77,7 @@ def define_tracepoints(args):
             else:
                 tp_flags.append('INTEL_DS_TRACEPOINT_FLAG_END_OF_PIPE')
         Tracepoint('intel_end_{0}'.format(name),
-                   toggle_name=name,
+                   toggle_name=toggle_name,
                    args=processed_args,
                    tp_struct=tp_struct,
                    tp_perfetto='intel_ds_end_{0}'.format(name),
@@ -82,6 +85,10 @@ def define_tracepoints(args):
                    tp_flags=tp_flags,
                    need_cs_param=need_cs_param)
 
+    def draw_tp(name, tp_args=[]):
+        begin_end_tp(name, tp_args=tp_args, toggle_name='draw',
+                     tp_default_enabled=not args.android)
+
     # Frame tracepoints
     begin_end_tp('frame',
                  tp_args=[Arg(type='uint32_t', var='frame', c_format='%u'),],
@@ -141,7 +148,8 @@ def define_tracepoints(args):
                           Arg(type='enum isl_format', var='dst_fmt', c_format='%s', to_prim_type='isl_format_get_short_name({})'),
                           Arg(type='enum isl_format', var='src_fmt', c_format='%s', to_prim_type='isl_format_get_short_name({})'),
                           Arg(type='uint8_t', var='predicated', c_format='%hhu'),
-                          ])
+                          ],
+                 tp_default_enabled=not args.android)
 
     # vkCmdWriteBufferMarker*, only for Anv
     begin_end_tp('write_buffer_marker',
@@ -166,53 +174,53 @@ def define_tracepoints(args):
                  tp_args=[Arg(type='uint32_t', var='count', c_format='%u')])
 
     # Various draws/dispatch, Anv & Iris
-    begin_end_tp('draw',
-                 tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_multi',
-                 tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indexed',
-                 tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indexed_multi',
-                 tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indirect_byte_count',
-                 tp_args=[Arg(type='uint32_t', var='instance_count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indirect',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indexed_indirect',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indirect_count',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
-                              is_indirect=True),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
-    begin_end_tp('draw_indexed_indirect_count',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
-                              is_indirect=True),
-                          Arg(type='uint32_t', var='vs_hash', c_format='%u'),
-                          Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw',
+            tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_multi',
+            tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indexed',
+            tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indexed_multi',
+            tp_args=[Arg(type='uint32_t', var='count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indirect_byte_count',
+            tp_args=[Arg(type='uint32_t', var='instance_count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indirect',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indexed_indirect',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indirect_count',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
+                         is_indirect=True),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
+    draw_tp('draw_indexed_indirect_count',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
+                         is_indirect=True),
+                     Arg(type='uint32_t', var='vs_hash', c_format='%u'),
+                     Arg(type='uint32_t', var='fs_hash', c_format='%u')])
 
-    begin_end_tp('draw_mesh',
-                 tp_args=[Arg(type='uint32_t', var='group_x', c_format='%u'),
-                          Arg(type='uint32_t', var='group_y', c_format='%u'),
-                          Arg(type='uint32_t', var='group_z', c_format='%u'),])
-    begin_end_tp('draw_mesh_indirect',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),])
-    begin_end_tp('draw_mesh_indirect_count',
-                 tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
+    draw_tp('draw_mesh',
+            tp_args=[Arg(type='uint32_t', var='group_x', c_format='%u'),
+                     Arg(type='uint32_t', var='group_y', c_format='%u'),
+                     Arg(type='uint32_t', var='group_z', c_format='%u'),])
+    draw_tp('draw_mesh_indirect',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u'),])
+    draw_tp('draw_mesh_indirect_count',
+            tp_args=[Arg(type='uint32_t', var='draw_count', c_format='%u',
                               is_indirect=True),])
 
     begin_end_tp('compute',
@@ -245,7 +253,9 @@ def define_tracepoints(args):
     begin_end_tp('as_lbvh_build_internal', repeat_last=True)
     begin_end_tp('as_ploc_build_internal', repeat_last=True)
     begin_end_tp('as_encode', repeat_last=True,
-                  tp_args=[Arg(type='uint32_t', var='n_leaves', c_format='%u'),
+                  tp_args=[Arg(type='uint32_t', var='pass', c_format='%u'),
+                           Arg(type='uint32_t', var='key', c_format='%x'),
+                           Arg(type='uint32_t', var='n_leaves', c_format='%u'),
                            Arg(type='uint32_t', var='n_ir_leaves', c_format='%u')])
     begin_end_tp('as_copy', repeat_last=True)
 
@@ -335,6 +345,7 @@ def main():
     parser.add_argument('--utrace-src', required=True)
     parser.add_argument('--utrace-hdr', required=True)
     parser.add_argument('--perfetto-hdr', required=True)
+    parser.add_argument('--android', action="store_true")
     args = parser.parse_args()
     sys.path.insert(0, args.import_path)
     define_tracepoints(args)
