@@ -134,12 +134,12 @@ void _mesa_print_info( struct gl_context *ctx )
     */
    _mesa_debug(NULL, "Mesa GL_EXTENSIONS = %s\n", ctx->Extensions.String);
 
-#if defined(USE_X86_ASM)
+#if DETECT_ARCH_X86
    _mesa_debug(NULL, "Mesa x86-optimized: YES\n");
 #else
    _mesa_debug(NULL, "Mesa x86-optimized: NO\n");
 #endif
-#if defined(USE_SPARC_ASM)
+#if DETECT_ARCH_SPARC64
    _mesa_debug(NULL, "Mesa sparc-optimized: YES\n");
 #else
    _mesa_debug(NULL, "Mesa sparc-optimized: NO\n");
@@ -205,7 +205,8 @@ set_debug_flags(const char *str)
       { "flush", DEBUG_ALWAYS_FLUSH }, /* flush after each drawing command */
       { "incomplete_tex", DEBUG_INCOMPLETE_TEXTURE },
       { "incomplete_fbo", DEBUG_INCOMPLETE_FBO },
-      { "context", DEBUG_CONTEXT } /* force set GL_CONTEXT_FLAG_DEBUG_BIT flag */
+      { "context", DEBUG_CONTEXT }, /* force set GL_CONTEXT_FLAG_DEBUG_BIT flag */
+      { "fallback_tex", DEBUG_FALLBACK_TEXTURE },
    };
    GLuint i;
 
