@@ -23,6 +23,7 @@
  */
 
 #include "util/glheader.h"
+#include "util/perf/cpu_trace.h"
 
 #include "blend.h"
 #include "bufferobj.h"
@@ -1046,6 +1047,8 @@ read_pixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format,
    struct gl_renderbuffer *rb;
    struct gl_pixelstore_attrib clippedPacking;
 
+   MESA_TRACE_FUNC();
+
    GET_CURRENT_CONTEXT(ctx);
 
    FLUSH_VERTICES(ctx, 0, 0);
@@ -1164,6 +1167,20 @@ read_pixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format,
             return;
          }
       }
+
+      /**
+       * OVR_multiview
+
+         INVALID_FRAMEBUFFER_OPERATION is generated by commands that read from the
+         framebuffer such as BlitFramebuffer, ReadPixels, CopyTexImage*, and
+         CopyTexSubImage*, if the number of views in the current read framebuffer
+         is greater than 1.
+       */
+      if (rb->rtt_numviews > 1) {
+         _mesa_error(ctx, GL_INVALID_FRAMEBUFFER_OPERATION,
+                     "glReadPixels(NumViews > 1 on read framebuffer)");
+         return;
+      }
    }
 
    /* Do all needed clipping here, so that we can forget about it later */
