@@ -213,7 +213,7 @@ st_convert_sampler_from_unit(const struct st_context *st,
  */
 static void
 update_shader_samplers(struct st_context *st,
-                       enum pipe_shader_type shader_stage,
+                       mesa_shader_stage shader_stage,
                        const struct gl_program *prog,
                        struct pipe_sampler_state *samplers,
                        unsigned *out_num_samplers)
@@ -288,6 +288,9 @@ update_shader_samplers(struct st_context *st,
             /* no additional views needed */
             break;
          FALLTHROUGH;
+      case PIPE_FORMAT_NV61:
+      case PIPE_FORMAT_NV24:
+      case PIPE_FORMAT_NV42:
       case PIPE_FORMAT_P010:
       case PIPE_FORMAT_P012:
       case PIPE_FORMAT_P016:
@@ -358,7 +361,7 @@ st_update_vertex_samplers(struct st_context *st)
    const struct gl_context *ctx = st->ctx;
 
    update_shader_samplers(st,
-                          PIPE_SHADER_VERTEX,
+                          MESA_SHADER_VERTEX,
                           ctx->VertexProgram._Current,
                           st->state.vert_samplers,
                           &st->state.num_vert_samplers);
@@ -372,7 +375,7 @@ st_update_tessctrl_samplers(struct st_context *st)
 
    if (ctx->TessCtrlProgram._Current) {
       update_shader_samplers(st,
-                             PIPE_SHADER_TESS_CTRL,
+                             MESA_SHADER_TESS_CTRL,
                              ctx->TessCtrlProgram._Current, NULL, NULL);
    }
 }
@@ -385,7 +388,7 @@ st_update_tesseval_samplers(struct st_context *st)
 
    if (ctx->TessEvalProgram._Current) {
       update_shader_samplers(st,
-                             PIPE_SHADER_TESS_EVAL,
+                             MESA_SHADER_TESS_EVAL,
                              ctx->TessEvalProgram._Current, NULL, NULL);
    }
 }
@@ -398,7 +401,7 @@ st_update_geometry_samplers(struct st_context *st)
 
    if (ctx->GeometryProgram._Current) {
       update_shader_samplers(st,
-                             PIPE_SHADER_GEOMETRY,
+                             MESA_SHADER_GEOMETRY,
                              ctx->GeometryProgram._Current, NULL, NULL);
    }
 }
@@ -410,7 +413,7 @@ st_update_fragment_samplers(struct st_context *st)
    const struct gl_context *ctx = st->ctx;
 
    update_shader_samplers(st,
-                          PIPE_SHADER_FRAGMENT,
+                          MESA_SHADER_FRAGMENT,
                           ctx->FragmentProgram._Current,
                           st->state.frag_samplers,
                           &st->state.num_frag_samplers);
@@ -424,7 +427,31 @@ st_update_compute_samplers(struct st_context *st)
 
    if (ctx->ComputeProgram._Current) {
       update_shader_samplers(st,
-                             PIPE_SHADER_COMPUTE,
+                             MESA_SHADER_COMPUTE,
                              ctx->ComputeProgram._Current, NULL, NULL);
    }
 }
+
+
+void
+st_update_task_samplers(struct st_context *st)
+{
+   const struct gl_context *ctx = st->ctx;
+
+   if (ctx->TaskProgram._Current) {
+      update_shader_samplers(st, MESA_SHADER_TASK,
+                             ctx->TaskProgram._Current, NULL, NULL);
+   }
+}
+
+
+void
+st_update_mesh_samplers(struct st_context *st)
+{
+   const struct gl_context *ctx = st->ctx;
+
+   if (ctx->MeshProgram._Current) {
+      update_shader_samplers(st, MESA_SHADER_MESH,
+                             ctx->MeshProgram._Current, NULL, NULL);
+   }
+}
