@@ -72,6 +72,9 @@ is_format_supported(struct pipe_screen *screen, enum pipe_format format,
       case PIPE_FORMAT_NV12:
       case PIPE_FORMAT_NV21:
       case PIPE_FORMAT_NV16:
+      case PIPE_FORMAT_NV61:
+      case PIPE_FORMAT_NV24:
+      case PIPE_FORMAT_NV42:
          supported = screen->is_format_supported(screen, PIPE_FORMAT_R8_UNORM,
                                                  PIPE_TEXTURE_2D, nr_samples,
                                                  nr_storage_samples, usage) &&
@@ -493,12 +496,18 @@ st_bind_egl_image(struct gl_context *ctx,
             texObj->RequiredTextureImageUnits = 2;
          }
          break;
+      case PIPE_FORMAT_NV61:
+      case PIPE_FORMAT_NV24:
+      case PIPE_FORMAT_NV42:
+         texFormat = MESA_FORMAT_R_UNORM8;
+         texObj->RequiredTextureImageUnits = 2;
+         break;
       case PIPE_FORMAT_NV15:
          if (stimg->texture->format == PIPE_FORMAT_R10_G10B10_420_UNORM) {
             texFormat = MESA_FORMAT_R10G10B10X2_UNORM;
             texObj->RequiredTextureImageUnits = 1;
          } else {
-            unreachable("NV15 emulation requires R10_G10B10_420_UNORM support");
+            UNREACHABLE("NV15 emulation requires R10_G10B10_420_UNORM support");
          }
          break;
       case PIPE_FORMAT_NV20:
@@ -506,7 +515,7 @@ st_bind_egl_image(struct gl_context *ctx,
             texFormat = MESA_FORMAT_R10G10B10X2_UNORM;
             texObj->RequiredTextureImageUnits = 1;
          } else {
-            unreachable("NV20 emulation requires R10_G10B10_422_UNORM support");
+            UNREACHABLE("NV20 emulation requires R10_G10B10_422_UNORM support");
          }
          break;
       case PIPE_FORMAT_P010:
@@ -586,7 +595,7 @@ st_bind_egl_image(struct gl_context *ctx,
          texObj->RequiredTextureImageUnits = 1;
          break;
       default:
-         unreachable("unexpected emulated format");
+         UNREACHABLE("unexpected emulated format");
          break;
       }
    } else {
