@@ -5,6 +5,8 @@
 #ifndef MME_TU104_SIM_H
 #define MME_TU104_SIM_H
 
+#include "mme_sim.h"
+
 #include <stdint.h>
 #include <stdlib.h>
 
@@ -20,6 +22,10 @@ struct mme_tu104_sim_mem {
    size_t size;
 };
 
+void mme_tu104_sim_core(uint32_t inst_count, const struct mme_tu104_inst *insts,
+                        const struct mme_sim_state_ops *state_ops,
+                        void *state_handler);
+
 void mme_tu104_sim(uint32_t inst_count, const struct mme_tu104_inst *insts,
                    uint32_t param_count, const uint32_t *params,
                    uint32_t mem_count, struct mme_tu104_sim_mem *mems);
