@@ -20,6 +20,7 @@
 #include "nv_push_cla0c0.h"
 #include "nv_push_clb1c0.h"
 #include "nv_push_clc6c0.h"
+#include "nv_push_clc7c0.h"
 #include "nv_push_clc86f.h"
 
 struct nvk_indirect_commands_layout {
@@ -395,7 +396,10 @@ build_process_cs_cmd_seq(nir_builder *b, struct nvk_nir_push *p,
             /* Now emit commands */
             nir_def *invoc = nir_imul_2x32_64(b, disp_size_x, disp_size_y);
             invoc = nir_imul(b, invoc, nir_u2u64(b, disp_size_z));
-            nvk_nir_P_1INC(b, p, NV9097, CALL_MME_MACRO(NVK_MME_ADD_CS_INVOCATIONS), 2);
+            if (pdev->info.cls_compute >= AMPERE_COMPUTE_B)
+               nvk_nir_P_1INC(b, p, NVC7C0, CALL_MME_MACRO(NVK_MME_ADD_CS_INVOCATIONS), 2);
+            else
+               nvk_nir_P_1INC(b, p, NV9097, CALL_MME_MACRO(NVK_MME_ADD_CS_INVOCATIONS), 2);
             nvk_nir_push_dw(b, p, nir_unpack_64_2x32_split_y(b, invoc));
             nvk_nir_push_dw(b, p, nir_unpack_64_2x32_split_x(b, invoc));
 
@@ -423,7 +427,7 @@ build_process_cs_cmd_seq(nir_builder *b, struct nvk_nir_push *p,
       }
 
       default:
-         unreachable("Unsupported indirect token type");
+         UNREACHABLE("Unsupported indirect token type");
       }
    }
 }
@@ -452,11 +456,11 @@ build_gfx_set_exec(nir_builder *b, struct nvk_nir_push *p, nir_def *token_addr,
 
    case VK_INDIRECT_EXECUTION_SET_INFO_TYPE_SHADER_OBJECTS_EXT: {
       int32_t i = 0;
-      gl_shader_stage type_stage[6] = {};
+      mesa_shader_stage type_stage[6] = {};
       nir_def *type_shader_idx[6] = {};
-      gl_shader_stage last_vtgm = MESA_SHADER_VERTEX;
+      mesa_shader_stage last_vtgm = MESA_SHADER_VERTEX;
       u_foreach_bit(s, token->shaderStages) {
-         gl_shader_stage stage = vk_to_mesa_shader_stage(1 << s);
+         mesa_shader_stage stage = vk_to_mesa_shader_stage(1 << s);
 
          if (stage != MESA_SHADER_FRAGMENT)
             last_vtgm = stage;
@@ -487,7 +491,7 @@ build_gfx_set_exec(nir_builder *b, struct nvk_nir_push *p, nir_def *token_addr,
    }
 
    default:
-      unreachable("Unknown indirect execution set type");
+      UNREACHABLE("Unknown indirect execution set type");
    }
 }
 
@@ -689,7 +693,7 @@ build_process_gfx_cmd_seq(nir_builder *b, struct nvk_nir_push *p,
          break;
 
       default:
-         unreachable("Unsupported indirect token type");
+         UNREACHABLE("Unsupported indirect token type");
       }
    }
 }
@@ -794,7 +798,7 @@ build_process_shader(struct nvk_device *dev,
          build_process_gfx_cmd_seq(b, &push, in_seq_addr, seq_idx,
                                    &in, pdev, info);
       } else {
-         unreachable("Unknown shader stage");
+         UNREACHABLE("Unknown shader stage");
       }
    }
    nir_pop_if(b, NULL);
@@ -807,12 +811,12 @@ build_process_shader(struct nvk_device *dev,
    } else if (info->shaderStages & NVK_SHADER_STAGE_GRAPHICS_BITS) {
       nvk_nir_pad_NOP(b, &push, NV9097);
    } else {
-      unreachable("Unknown shader stage");
+      UNREACHABLE("Unknown shader stage");
    }
 
    /* Replace the out stride with the actual size of a command stream */
    nir_load_const_instr *out_stride_const =
-      nir_instr_as_load_const(out_stride->parent_instr);
+      nir_def_as_load_const(out_stride);
    out_stride_const->value[0].u32 = push.max_dw_count * 4;
 
    /* We also output this stride to go in the layout struct */
@@ -1084,7 +1088,7 @@ nvk_CmdExecuteGeneratedCommandsEXT(VkCommandBuffer commandBuffer,
 
          uint8_t set_types = 0;
          u_foreach_bit(s, layout->set_stages) {
-            gl_shader_stage stage = vk_to_mesa_shader_stage(1 << s);
+            mesa_shader_stage stage = vk_to_mesa_shader_stage(1 << s);
             uint32_t type = mesa_to_nv9097_shader_type(stage);
             set_types |= BITFIELD_BIT(type);
          }
