@@ -117,6 +117,12 @@ instrs_equal(const void *_i1, const void *_i2)
          return false;
    }
 
+   if (i1->table != i2->table)
+      return false;
+
+   if (i1->index != i2->index)
+      return false;
+
    return true;
 }
 
@@ -136,7 +142,7 @@ instr_can_cse(const bi_instr *I)
    /* Be conservative about which message-passing instructions we CSE,
     * since most are not pure even within a thread.
     */
-   if (bi_opcode_props[I->op].message && I->op != BI_OPCODE_LEA_BUF_IMM)
+   if (bi_get_opcode_props(I)->message && I->op != BI_OPCODE_LEA_BUF_IMM)
       return false;
 
    if (I->branch_target)
