@@ -552,7 +552,7 @@ const struct pan_format GENX(pan_pipe_format)[PIPE_FORMAT_COUNT] = {
    FMT(Z16_UNORM,               Z16_UNORM,       RGBA, L, _T_Z_),
    FMT(Z24_UNORM_S8_UINT,       Z24X8_UNORM,     RGBA, L, _T_Z_),
    FMT(Z24X8_UNORM,             Z24X8_UNORM,     RGBA, L, _T_Z_),
-   FMT(Z32_FLOAT,               R32F,            RGBA, L, _T_Z_),
+   FMT(Z32_FLOAT,               Z32,             RGBA, L, _T_Z_),
 
 #if PAN_ARCH >= 9
    /* Specify interchange formats, the actual format for depth/stencil is
@@ -563,6 +563,7 @@ const struct pan_format GENX(pan_pipe_format)[PIPE_FORMAT_COUNT] = {
     * we want stencil in the red channel, so we use the GRBA swizzles.
     */
    FMT(Z32_FLOAT_S8X24_UINT,    R32F,            GRBA, L, _T_Z_),
+   FMT(Z24_UNORM_PACKED,        Z24X8_UNORM,     RGBA, L, _T_Z_),
    FMT(X32_S8X24_UINT,          S8,              GRBA, L, _T_Z_),
    FMT(X24S8_UINT,              S8,              GRBA, L, _T_Z_),
    FMT(S8_UINT,                 S8,              GRBA, L, _T_Z_),
@@ -657,7 +658,7 @@ GENX(pan_decompose_swizzle)(enum mali_rgb_component_order order)
       CASE(0001, 0001, X, Y, Z, W);
       CASE(0000, 0000, X, Y, Z, W);
    default:
-      unreachable("Invalid case for texturing");
+      UNREACHABLE("Invalid case for texturing");
    }
 
 #undef CASE
