@@ -32,77 +32,11 @@
 
 #include "util/macros.h"
 
+#include "pan_model.h"
+
 struct pan_kmod_dev;
 struct pan_kmod_dev_props;
-
-/** Implementation-defined tiler features */
-struct pan_tiler_features {
-   /** Number of bytes per tiler bin */
-   unsigned bin_size;
-
-   /** Maximum number of levels that may be simultaneously enabled.
-    * Invariant: bitcount(hierarchy_mask) <= max_levels */
-   unsigned max_levels;
-};
-
-#define ARCH_MAJOR     BITFIELD_RANGE(28, 4)
-#define ARCH_MINOR     BITFIELD_RANGE(24, 4)
-#define ARCH_REV       BITFIELD_RANGE(20, 4)
-#define PRODUCT_MAJOR  BITFIELD_RANGE(16, 4)
-#define VERSION_MAJOR  BITFIELD_RANGE(12, 4)
-#define VERSION_MINOR  BITFIELD_RANGE(4, 8)
-#define VERSION_STATUS BITFIELD_RANGE(0, 4)
-
-struct pan_model {
-   /* GPU product ID */
-   uint32_t gpu_prod_id;
-
-   /* Mask to apply to the GPU ID to get a product ID. */
-   uint32_t gpu_prod_id_mask;
-
-   /* GPU variant. */
-   uint32_t gpu_variant;
-
-   /* Marketing name for the GPU, used as the GL_RENDERER */
-   const char *name;
-
-   /* Set of associated performance counters */
-   const char *performance_counters;
-
-   /* Minimum GPU revision required for anisotropic filtering. ~0 and 0
-    * means "no revisions support anisotropy" and "all revisions support
-    * anistropy" respectively -- so checking for anisotropy is simply
-    * comparing the reivsion.
-    */
-   uint32_t min_rev_anisotropic;
-
-   struct {
-      /* Default tilebuffer size in bytes for the model. */
-      uint32_t color_size;
-
-      /* Default tilebuffer depth size in bytes for the model. */
-      uint32_t z_size;
-   } tilebuffer;
-
-   /* Maximum number of pixels, texels, and FMA ops, per clock per shader
-    * core, or 0 if it can't be determined for the given GPU. */
-   struct {
-      uint32_t pixel;
-      uint32_t texel;
-      uint32_t fma;
-   } rates;
-
-   struct {
-      /* The GPU lacks the capability for hierarchical tiling, without
-       * an "Advanced Tiling Unit", instead requiring a single bin
-       * size for the entire framebuffer be selected by the driver
-       */
-      bool no_hierarchical_tiling;
-      bool max_4x_msaa;
-   } quirks;
-};
-
-const struct pan_model *pan_get_model(uint32_t gpu_id, uint32_t gpu_variant);
+struct pan_kmod_vm;
 
 unsigned pan_query_l2_slices(const struct pan_kmod_dev_props *props);
 
@@ -120,13 +54,44 @@ bool pan_query_afbc(const struct pan_kmod_dev_props *props);
 
 bool pan_query_afrc(const struct pan_kmod_dev_props *props);
 
-unsigned pan_query_optimal_tib_size(const struct pan_model *model);
+unsigned pan_query_tib_size(const struct pan_model *model);
 
-unsigned pan_query_optimal_z_tib_size(const struct pan_model *model);
+unsigned pan_query_z_tib_size(const struct pan_model *model);
+
+static inline unsigned
+pan_query_optimal_tib_size(unsigned arch, const struct pan_model *model)
+{
+   unsigned tib_size = pan_query_tib_size(model);
+
+   /* On V5, as well as V7 and later, we can disable pipelining to gain some
+    * extra tib memory.
+    */
+   if (arch > 4 && arch != 6)
+      return tib_size / 2;
+
+   return tib_size;
+}
+
+static inline unsigned
+pan_query_optimal_z_tib_size(unsigned arch, const struct pan_model *model)
+{
+   unsigned tib_size = pan_query_z_tib_size(model);
+
+   /* On V5, as well as V7 and later, we can disable pipelining to gain some
+    * extra tib memory.
+    */
+   if (arch > 4 && arch != 6)
+      return tib_size / 2;
+
+   return tib_size;
+}
 
 uint64_t pan_clamp_to_usable_va_range(const struct pan_kmod_dev *dev,
                                       uint64_t va);
 
+uint64_t pan_choose_gpu_va_alignment(const struct pan_kmod_vm *vm,
+                                     uint64_t size);
+
 unsigned pan_compute_max_thread_count(const struct pan_kmod_dev_props *props,
                                       unsigned work_reg_count);
 
@@ -174,23 +139,6 @@ pan_meta_tile_size(unsigned arch)
    return 32;
 }
 
-/* Returns the maximum usable color tilebuffer-size. This is *usually* twice
- * the optimal tilebuffer-size, but not always.
- */
-static inline unsigned
-pan_get_max_tib_size(unsigned arch, const struct pan_model *model)
-{
-   unsigned tib_size = pan_query_optimal_tib_size(model);
-
-   /* On V5, as well as V6 and later, we can disable pipelining to gain some
-    * extra tib memory.
-    */
-   if (arch > 4 && arch != 6)
-      return tib_size * 2;
-
-   return tib_size;
-}
-
 static inline uint32_t
 pan_get_max_cbufs(unsigned arch, unsigned max_tib_size)
 {
