@@ -30,6 +30,8 @@
 #include "util/u_dynarray.h"
 
 void midgard_preprocess_nir(nir_shader *nir, unsigned gpu_id);
+void midgard_postprocess_nir(nir_shader *nir, unsigned gpu_id);
+void midgard_lower_texture_nir(nir_shader *nir, unsigned gpu_id);
 
 void midgard_compile_shader_nir(nir_shader *nir,
                                 const struct pan_compile_inputs *inputs,
