@@ -64,7 +64,7 @@
 
 /* Does a block use helpers directly */
 static bool
-mir_block_uses_helpers(gl_shader_stage stage, midgard_block *block)
+mir_block_uses_helpers(mesa_shader_stage stage, midgard_block *block)
 {
    mir_foreach_instr_in_block(block, ins) {
       if (ins->type != TAG_TEXTURE_4)
@@ -190,7 +190,7 @@ mir_analyze_helper_requirements(compiler_context *ctx)
 {
    mir_compute_temp_count(ctx);
    unsigned temp_count = ctx->temp_count;
-   BITSET_WORD *deps = calloc(sizeof(BITSET_WORD), BITSET_WORDS(temp_count));
+   BITSET_WORD *deps = BITSET_CALLOC(temp_count);
 
    /* Initialize with the sources of instructions consuming
     * derivatives */
