@@ -33,25 +33,23 @@
 
 #define PANFROST_MINMAX_SIZE 64
 
-struct panfrost_minmax_cache {
+struct pan_minmax_cache {
    uint64_t keys[PANFROST_MINMAX_SIZE];
    uint64_t values[PANFROST_MINMAX_SIZE];
    unsigned size;
    unsigned index;
 };
 
-bool panfrost_minmax_cache_get(struct panfrost_minmax_cache *cache,
-                               unsigned index_size,
-                               unsigned start, unsigned count,
-                               unsigned *min_index, unsigned *max_index);
+bool pan_minmax_cache_get(struct pan_minmax_cache *cache, unsigned index_size,
+                          unsigned start, unsigned count, unsigned *min_index,
+                          unsigned *max_index);
 
-void panfrost_minmax_cache_add(struct panfrost_minmax_cache *cache,
-                               unsigned index_size,
-                               unsigned start, unsigned count,
-                               unsigned min_index, unsigned max_index);
+void pan_minmax_cache_add(struct pan_minmax_cache *cache, unsigned index_size,
+                          unsigned start, unsigned count, unsigned min_index,
+                          unsigned max_index);
 
-void panfrost_minmax_cache_invalidate(struct panfrost_minmax_cache *cache,
-                                      unsigned index_size,
-                                      size_t offset, size_t size);
+void pan_minmax_cache_invalidate(struct pan_minmax_cache *cache,
+                                 unsigned index_size, size_t offset,
+                                 size_t size);
 
 #endif
