@@ -7,10 +7,13 @@
 
 #include "vulkan/util/vk_util.h"
 
+#include "panvk_android.h"
 #include "panvk_device.h"
 #include "panvk_device_memory.h"
 #include "panvk_entrypoints.h"
 
+#include "pan_props.h"
+
 #include "vk_log.h"
 
 static void *
@@ -46,9 +49,12 @@ panvk_AllocateMemory(VkDevice _device,
                      const VkAllocationCallbacks *pAllocator,
                      VkDeviceMemory *pMem)
 {
+   if (panvk_android_is_ahb_memory(pAllocateInfo)) {
+      return panvk_android_allocate_ahb_memory(_device, pAllocateInfo,
+                                               pAllocator, pMem);
+   }
+
    VK_FROM_HANDLE(panvk_device, device, _device);
-   struct panvk_instance *instance =
-      to_panvk_instance(device->vk.physical->instance);
    struct panvk_device_memory *mem;
    bool can_be_exported = false;
    VkResult result;
@@ -112,9 +118,8 @@ panvk_AllocateMemory(VkDevice _device,
    };
 
    if (!(device->kmod.vm->flags & PAN_KMOD_VM_FLAG_AUTO_VA)) {
-      op.va.start =
-         panvk_as_alloc(device, op.va.size,
-                        op.va.size > 0x200000 ? 0x200000 : 0x1000);
+      op.va.start = panvk_as_alloc(device, op.va.size,
+         pan_choose_gpu_va_alignment(device->kmod.vm, op.va.size));
       if (!op.va.start) {
          result = panvk_error(device, VK_ERROR_OUT_OF_DEVICE_MEMORY);
          goto err_put_bo;
@@ -144,7 +149,7 @@ panvk_AllocateMemory(VkDevice _device,
    }
 
    if (device->debug.decode_ctx) {
-      if (instance->debug_flags & (PANVK_DEBUG_DUMP | PANVK_DEBUG_TRACE)) {
+      if (PANVK_DEBUG(DUMP) || PANVK_DEBUG(TRACE)) {
          void *cpu = pan_kmod_bo_mmap(mem->bo, 0, pan_kmod_bo_size(mem->bo),
                                       PROT_READ | PROT_WRITE, MAP_SHARED, NULL);
          if (cpu != MAP_FAILED)
