@@ -146,8 +146,7 @@ util_format_z16_unorm_unpack_z_float(float *restrict dst_row, unsigned dst_strid
       float *dst = dst_row;
       const uint16_t *src = (const uint16_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint16_t value = util_cpu_to_le16(*src++);
-         *dst++ = z16_unorm_to_z32_float(value);
+         *dst++ = z16_unorm_to_z32_float(*src++);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -164,9 +163,7 @@ util_format_z16_unorm_pack_z_float(uint8_t *restrict dst_row, unsigned dst_strid
       const float *src = src_row;
       uint16_t *dst = (uint16_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint16_t value;
-         value = z32_float_to_z16_unorm(*src++);
-         *dst++ = util_le16_to_cpu(value);
+         *dst++ = z32_float_to_z16_unorm(*src++);
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -183,8 +180,7 @@ util_format_z16_unorm_unpack_z_32unorm(uint32_t *restrict dst_row, unsigned dst_
       uint32_t *dst = dst_row;
       const uint16_t *src = (const uint16_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint16_t value = util_cpu_to_le16(*src++);
-         *dst++ = z16_unorm_to_z32_unorm(value);
+         *dst++ = z16_unorm_to_z32_unorm(*src++);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -201,9 +197,91 @@ util_format_z16_unorm_pack_z_32unorm(uint8_t *restrict dst_row, unsigned dst_str
       const uint32_t *src = src_row;
       uint16_t *dst = (uint16_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint16_t value;
-         value = z32_unorm_to_z16_unorm(*src++);
-         *dst++ = util_le16_to_cpu(value);
+         *dst++ = z32_unorm_to_z16_unorm(*src++);
+      }
+      dst_row += dst_stride/sizeof(*dst_row);
+      src_row += src_stride/sizeof(*src_row);
+   }
+}
+
+void
+util_format_z24_unorm_packed_unpack_z_float(float *restrict dst_row, unsigned dst_stride,
+                                            const uint8_t *restrict src_row, unsigned src_stride,
+                                            unsigned width, unsigned height)
+{
+   unsigned x, y;
+   for(y = 0; y < height; ++y) {
+      float *dst = dst_row;
+      const uint8_t *src = src_row;
+      for(x = 0; x < width; ++x) {
+         uint32_t tmp = src[0] | ((uint32_t)src[1] << 8) | ((uint32_t)src[2] << 16);
+
+         *dst++ = z24_unorm_to_z32_float(tmp);
+         src += 3;
+      }
+      src_row += src_stride/sizeof(*src_row);
+      dst_row += dst_stride/sizeof(*dst_row);
+   }
+}
+
+void
+util_format_z24_unorm_packed_pack_z_float(uint8_t *restrict dst_row, unsigned dst_stride,
+                                          const float *restrict src_row, unsigned src_stride,
+                                          unsigned width, unsigned height)
+{
+   unsigned x, y;
+   for(y = 0; y < height; ++y) {
+      const float *src = src_row;
+      uint8_t *dst = dst_row;
+      for(x = 0; x < width; ++x) {
+         uint32_t tmp = z32_float_to_z24_unorm(*src++);
+
+         dst[0] = tmp;
+         dst[1] = tmp >> 8;
+         dst[2] = tmp >> 16;
+         dst += 3;
+      }
+      dst_row += dst_stride/sizeof(*dst_row);
+      src_row += src_stride/sizeof(*src_row);
+   }
+}
+
+void
+util_format_z24_unorm_packed_unpack_z_32unorm(uint32_t *restrict dst_row, unsigned dst_stride,
+                                              const uint8_t *restrict src_row, unsigned src_stride,
+                                              unsigned width, unsigned height)
+{
+   unsigned x, y;
+   for(y = 0; y < height; ++y) {
+      uint32_t *dst = dst_row;
+      const uint8_t *src = src_row;
+      for(x = 0; x < width; ++x) {
+         uint32_t tmp = src[0] | ((uint32_t)src[1] << 8) | ((uint32_t)src[2] << 16);
+
+         *dst++ = z24_unorm_to_z32_unorm(tmp);
+         src += 3;
+      }
+      src_row += src_stride/sizeof(*src_row);
+      dst_row += dst_stride/sizeof(*dst_row);
+   }
+}
+
+void
+util_format_z24_unorm_packed_pack_z_32unorm(uint8_t *restrict dst_row, unsigned dst_stride,
+                                            const uint32_t *restrict src_row, unsigned src_stride,
+                                            unsigned width, unsigned height)
+{
+   unsigned x, y;
+   for(y = 0; y < height; ++y) {
+      const uint32_t *src = src_row;
+      uint16_t *dst = (uint16_t *)dst_row;
+      for(x = 0; x < width; ++x) {
+         uint32_t tmp = z32_unorm_to_z24_unorm(*src++);
+
+         dst[0] = tmp;
+         dst[1] = tmp >> 8;
+         dst[2] = tmp >> 16;
+         dst += 3;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -220,8 +298,7 @@ util_format_z32_unorm_unpack_z_float(float *restrict dst_row, unsigned dst_strid
       float *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z32_unorm_to_z32_float(value);
+         *dst++ = z32_unorm_to_z32_float(*src++);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -238,9 +315,7 @@ util_format_z32_unorm_pack_z_float(uint8_t *restrict dst_row, unsigned dst_strid
       const float *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value;
-         value = z32_float_to_z32_unorm(*src++);
-         *dst++ = util_le32_to_cpu(value);
+         *dst++ =z32_float_to_z32_unorm(*src++);
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -339,7 +414,7 @@ util_format_z16_unorm_s8_uint_unpack_z_float(float *restrict dst_row, unsigned d
                                              const uint8_t *restrict src_row, unsigned src_stride,
                                              unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -347,7 +422,7 @@ util_format_z16_unorm_s8_uint_pack_z_float(uint8_t *restrict dst_row, unsigned d
                                            const float *restrict src_row, unsigned src_stride,
                                            unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -355,7 +430,7 @@ util_format_z16_unorm_s8_uint_unpack_z_32unorm(uint32_t *restrict dst_row, unsig
                                                const uint8_t *restrict src_row, unsigned src_stride,
                                                unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -363,7 +438,7 @@ util_format_z16_unorm_s8_uint_pack_z_32unorm(uint8_t *restrict dst_row, unsigned
                                              const uint32_t *restrict src_row, unsigned src_stride,
                                              unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -371,7 +446,7 @@ util_format_z16_unorm_s8_uint_unpack_s_8uint(uint8_t *restrict dst_row, unsigned
                                              const uint8_t *restrict src_row, unsigned src_stride,
                                              unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -379,7 +454,7 @@ util_format_z16_unorm_s8_uint_pack_s_8uint(uint8_t *restrict dst_row, unsigned d
                                            const uint8_t *restrict src_row, unsigned src_stride,
                                            unsigned width, unsigned height)
 {
-   unreachable("z16_s8 packing/unpacking is not implemented.");
+   UNREACHABLE("z16_s8 packing/unpacking is not implemented.");
 }
 
 void
@@ -392,8 +467,7 @@ util_format_z24_unorm_s8_uint_unpack_z_float(float *restrict dst_row, unsigned d
       float *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value =  util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_float(value & 0xffffff);
+         *dst++ = z24_unorm_to_z32_float((*src++) & 0xffffff);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -410,10 +484,10 @@ util_format_z24_unorm_s8_uint_pack_z_float(uint8_t *restrict dst_row, unsigned d
       const float *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0xff000000;
          value |= z32_float_to_z24_unorm(*src++);
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -431,8 +505,7 @@ util_format_z24_unorm_s8_uint_unpack_z24(uint8_t *restrict dst_row, unsigned dst
       uint32_t *dst = (uint32_t *)dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value =  util_cpu_to_le32(*src++);
-         *dst++ = (value & 0xffffff);
+         *dst++ = ((*src++) & 0xffffff);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -449,11 +522,11 @@ util_format_z24_unorm_s8_uint_pack_z24(uint8_t *restrict dst_row, unsigned dst_s
       const uint32_t *src = (const uint32_t *)src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0xff000000;
          value |= *src & 0xffffff;
          src++;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -470,8 +543,7 @@ util_format_z24_unorm_s8_uint_unpack_z_32unorm(uint32_t *restrict dst_row, unsig
       uint32_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_unorm(value & 0xffffff);
+         *dst++ = z24_unorm_to_z32_unorm((*src++) & 0xffffff);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -488,10 +560,10 @@ util_format_z24_unorm_s8_uint_pack_z_32unorm(uint8_t *restrict dst_row, unsigned
       const uint32_t *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0xff000000;
          value |= z32_unorm_to_z24_unorm(*src++);
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -508,8 +580,7 @@ util_format_z24_unorm_s8_uint_unpack_s_8uint(uint8_t *restrict dst_row, unsigned
       uint8_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = value >> 24;
+         *dst++ = (*src++) >> 24;
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -529,7 +600,7 @@ util_format_z24_unorm_s8_uint_pack_s_8uint(uint8_t *restrict dst_row, unsigned d
          uint32_t value = util_le32_to_cpu(*dst);
          value &= 0x00ffffff;
          value |= (uint32_t)*src++ << 24;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -556,6 +627,26 @@ util_format_z24_unorm_s8_uint_pack_separate(uint8_t *restrict dst_row, unsigned
    }
 }
 
+void
+util_format_z24_unorm_s8_uint_pack_separate_z32(uint8_t *restrict dst_row, unsigned dst_stride,
+                                                const float *z_src_row, unsigned z_src_stride,
+                                                const uint8_t *s_src_row, unsigned s_src_stride,
+                                                unsigned width, unsigned height)
+{
+   unsigned x, y;
+   for (y = 0; y < height; ++y) {
+      const float *z_src = z_src_row;
+      const uint8_t *s_src = s_src_row;
+      uint32_t *dst = (uint32_t *)dst_row;
+      for (x = 0; x < width; ++x) {
+         *dst++ = (z32_float_to_z24_unorm(*z_src++) & 0x00ffffff) | ((uint32_t)*s_src++ << 24);
+      }
+      dst_row += dst_stride / sizeof(*dst_row);
+      z_src_row += z_src_stride / sizeof(*z_src_row);
+      s_src_row += s_src_stride / sizeof(*s_src_row);
+   }
+}
+
 void
 util_format_s8_uint_z24_unorm_unpack_z_float(float *restrict dst_row, unsigned dst_stride,
                                                 const uint8_t *restrict src_row, unsigned src_stride,
@@ -566,8 +657,7 @@ util_format_s8_uint_z24_unorm_unpack_z_float(float *restrict dst_row, unsigned d
       float *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_float(value >> 8);
+         *dst++ = z24_unorm_to_z32_float((*src++) >> 8);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -584,10 +674,10 @@ util_format_s8_uint_z24_unorm_pack_z_float(uint8_t *restrict dst_row, unsigned d
       const float *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0x000000ff;
          value |= z32_float_to_z24_unorm(*src++) << 8;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -604,7 +694,7 @@ util_format_s8_uint_z24_unorm_unpack_z_32unorm(uint32_t *restrict dst_row, unsig
       uint32_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
+         uint32_t value = *src++;
          *dst++ = z24_unorm_to_z32_unorm(value >> 8);
       }
       src_row += src_stride/sizeof(*src_row);
@@ -622,10 +712,10 @@ util_format_s8_uint_z24_unorm_pack_z_32unorm(uint8_t *restrict dst_row, unsigned
       const uint32_t *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0x000000ff;
          value |= *src++ & 0xffffff00;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -642,8 +732,7 @@ util_format_s8_uint_z24_unorm_unpack_s_8uint(uint8_t *restrict dst_row, unsigned
       uint8_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = value & 0xff;
+         *dst++ = (*src++) & 0xff;
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -660,10 +749,10 @@ util_format_s8_uint_z24_unorm_pack_s_8uint(uint8_t *restrict dst_row, unsigned d
       const uint8_t *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_le32_to_cpu(*dst);
+         uint32_t value = *dst;
          value &= 0xffffff00;
          value |= *src++;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = value;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -680,8 +769,7 @@ util_format_z24x8_unorm_unpack_z_float(float *restrict dst_row, unsigned dst_str
       float *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_float(value & 0xffffff);
+         *dst++ = z24_unorm_to_z32_float((*src++) & 0xffffff);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -698,9 +786,7 @@ util_format_z24x8_unorm_pack_z_float(uint8_t *restrict dst_row, unsigned dst_str
       const float *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value;
-         value = z32_float_to_z24_unorm(*src++);
-         *dst++ = util_le32_to_cpu(value);
+         *dst++ = z32_float_to_z24_unorm(*src++);
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -717,8 +803,7 @@ util_format_z24x8_unorm_unpack_z_32unorm(uint32_t *restrict dst_row, unsigned ds
       uint32_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_unorm(value & 0xffffff);
+         *dst++ = z24_unorm_to_z32_unorm((*src++) & 0xffffff);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -735,9 +820,7 @@ util_format_z24x8_unorm_pack_z_32unorm(uint8_t *restrict dst_row, unsigned dst_s
       const uint32_t *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value;
-         value = z32_unorm_to_z24_unorm(*src++);
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = z32_unorm_to_z24_unorm(*src++);
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -754,8 +837,7 @@ util_format_x8z24_unorm_unpack_z_float(float *restrict dst_row, unsigned dst_str
       float *dst = dst_row;
       const uint32_t *src = (uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_float(value >> 8);
+         *dst++ = z24_unorm_to_z32_float((*src++) >> 8);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -772,9 +854,7 @@ util_format_x8z24_unorm_pack_z_float(uint8_t *restrict dst_row, unsigned dst_str
       const float *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value;
-         value = z32_float_to_z24_unorm(*src++) << 8;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = z32_float_to_z24_unorm(*src++) << 8;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -791,8 +871,7 @@ util_format_x8z24_unorm_unpack_z_32unorm(uint32_t *restrict dst_row, unsigned ds
       uint32_t *dst = dst_row;
       const uint32_t *src = (const uint32_t *)src_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value = util_cpu_to_le32(*src++);
-         *dst++ = z24_unorm_to_z32_unorm(value >> 8);
+         *dst++ = z24_unorm_to_z32_unorm((*src++) >> 8);
       }
       src_row += src_stride/sizeof(*src_row);
       dst_row += dst_stride/sizeof(*dst_row);
@@ -809,9 +888,7 @@ util_format_x8z24_unorm_pack_z_32unorm(uint8_t *restrict dst_row, unsigned dst_s
       const uint32_t *src = src_row;
       uint32_t *dst = (uint32_t *)dst_row;
       for(x = 0; x < width; ++x) {
-         uint32_t value;
-         value = z32_unorm_to_z24_unorm(*src++) << 8;
-         *dst++ = util_cpu_to_le32(value);
+         *dst++ = z32_unorm_to_z24_unorm(*src++) << 8;
       }
       dst_row += dst_stride/sizeof(*dst_row);
       src_row += src_stride/sizeof(*src_row);
@@ -901,10 +978,10 @@ util_format_z32_float_s8x24_uint_unpack_s_8uint(uint8_t *restrict dst_row, unsig
    unsigned x, y;
    for(y = 0; y < height; ++y) {
       uint8_t *dst = dst_row;
-      const uint8_t *src = src_row + 4;
+      const uint32_t *src = (uint32_t *)(src_row + 4);
       for(x = 0; x < width; ++x) {
          *dst = *src;
-         src += 8;
+         src += 2;
          dst += 1;
       }
       src_row += src_stride/sizeof(*src_row);
@@ -922,7 +999,7 @@ util_format_z32_float_s8x24_uint_pack_s_8uint(uint8_t *restrict dst_row, unsigne
       const uint8_t *src = src_row;
       uint32_t *dst = ((uint32_t *)dst_row) + 1;
       for(x = 0; x < width; ++x) {
-         *dst = util_cpu_to_le32(*src);
+         *dst = *src;
          src += 1;
          dst += 2;
       }
