@@ -36,42 +36,29 @@ extern "C" {
 struct u_gralloc;
 struct vk_device;
 struct vk_image;
-struct AHardwareBuffer;
 
-#if DETECT_OS_ANDROID
+#ifdef VK_USE_PLATFORM_ANDROID_KHR
+
 struct u_gralloc *vk_android_get_ugralloc(void);
-struct u_gralloc *vk_android_init_ugralloc(void);
-void vk_android_destroy_ugralloc(void);
+
 VkResult vk_android_import_anb(struct vk_device *device,
                                const VkImageCreateInfo *pCreateInfo,
                                const VkAllocationCallbacks *alloc,
                                struct vk_image *image);
+
 VkResult vk_android_get_anb_layout(
    const VkImageCreateInfo *pCreateInfo,
    VkImageDrmFormatModifierExplicitCreateInfoEXT *out,
    VkSubresourceLayout *out_layouts, int max_planes);
-VkResult vk_android_get_ahb_layout(
-   struct AHardwareBuffer *ahardware_buffer,
-   VkImageDrmFormatModifierExplicitCreateInfoEXT *out,
-   VkSubresourceLayout *out_layouts, int max_planes);
+
 #else
+
 static inline struct u_gralloc *
 vk_android_get_ugralloc(void)
 {
    return NULL;
 }
 
-static inline struct u_gralloc *
-vk_android_init_ugralloc(void)
-{
-   return NULL;
-}
-
-static inline void
-vk_android_destroy_ugralloc(void)
-{
-}
-
 static inline VkResult
 vk_android_import_anb(struct vk_device *device,
                       const VkImageCreateInfo *pCreateInfo,
@@ -90,18 +77,13 @@ vk_android_get_anb_layout(
    return VK_ERROR_FEATURE_NOT_PRESENT;
 }
 
-static inline VkResult
-vk_android_get_ahb_layout(
-   struct AHardwareBuffer *ahardware_buffer,
-   VkImageDrmFormatModifierExplicitCreateInfoEXT *out,
-   VkSubresourceLayout *out_layouts, int max_planes)
-{
-   return VK_ERROR_FEATURE_NOT_PRESENT;
-}
-
 #endif
 
-#if DETECT_OS_ANDROID && ANDROID_API_LEVEL >= 26
+#if defined(VK_USE_PLATFORM_ANDROID_KHR) && ANDROID_API_LEVEL >= 26
+
+struct AHardwareBuffer;
+
+uint64_t vk_android_get_front_buffer_usage(void);
 
 VkFormat vk_ahb_format_to_image_format(uint32_t ahb_format);
 
@@ -110,14 +92,34 @@ uint32_t vk_image_format_to_ahb_format(VkFormat vk_format);
 uint64_t vk_image_usage_to_ahb_usage(const VkImageCreateFlags vk_create,
                                      const VkImageUsageFlags vk_usage);
 
-bool vk_ahb_probe_format(VkFormat vk_format,
-                         VkImageCreateFlags vk_create,
-                         VkImageUsageFlags vk_usage);
+struct AHardwareBuffer *vk_alloc_ahardware_buffer(
+   const VkMemoryAllocateInfo *pAllocateInfo);
 
-struct AHardwareBuffer *
-vk_alloc_ahardware_buffer(const VkMemoryAllocateInfo *pAllocateInfo);
+VkResult vk_android_get_ahb_layout(
+   struct AHardwareBuffer *ahardware_buffer,
+   VkImageDrmFormatModifierExplicitCreateInfoEXT *out,
+   VkSubresourceLayout *out_layouts, int max_planes);
 
-#else /* DETECT_OS_ANDROID && ANDROID_API_LEVEL >= 26 */
+VkResult vk_android_get_ahb_image_properties(
+   VkPhysicalDevice pdev_handle,
+   const VkPhysicalDeviceImageFormatInfo2 *info,
+   VkImageFormatProperties2 *props);
+
+void vk_android_get_ahb_buffer_properties(
+   VkPhysicalDevice pdev_handle,
+   const VkPhysicalDeviceExternalBufferInfo *info,
+   VkExternalBufferProperties *props);
+
+bool vk_android_rp_attachment_has_external_format(
+   const VkAttachmentDescription2 *desc);
+
+#else /* defined(VK_USE_PLATFORM_ANDROID_KHR) && ANDROID_API_LEVEL >= 26 */
+
+static inline uint64_t
+vk_android_get_front_buffer_usage(void)
+{
+   return 0;
+}
 
 static inline VkFormat
 vk_ahb_format_to_image_format(uint32_t ahb_format)
@@ -138,20 +140,45 @@ vk_image_usage_to_ahb_usage(const VkImageCreateFlags vk_create,
    return 0;
 }
 
-static inline bool
-vk_ahb_probe_format(VkFormat vk_format,
-                    VkImageCreateFlags vk_create,
-                    VkImageUsageFlags vk_usage)
-{
-   return false;
-}
-
 static inline struct AHardwareBuffer *
 vk_alloc_ahardware_buffer(const VkMemoryAllocateInfo *pAllocateInfo)
 {
    return NULL;
 }
 
+static inline VkResult
+vk_android_get_ahb_layout(
+   struct AHardwareBuffer *ahardware_buffer,
+   VkImageDrmFormatModifierExplicitCreateInfoEXT *out,
+   VkSubresourceLayout *out_layouts, int max_planes)
+{
+   return VK_ERROR_FEATURE_NOT_PRESENT;
+}
+
+static inline VkResult
+vk_android_get_ahb_image_properties(
+   VkPhysicalDevice pdev_handle,
+   const VkPhysicalDeviceImageFormatInfo2 *info,
+   VkImageFormatProperties2 *props)
+{
+   return VK_ERROR_FORMAT_NOT_SUPPORTED;
+}
+
+static inline void
+vk_android_get_ahb_buffer_properties(
+   VkPhysicalDevice pdev_handle,
+   const VkPhysicalDeviceExternalBufferInfo *info,
+   VkExternalBufferProperties *props)
+{
+}
+
+static bool
+vk_android_rp_attachment_has_external_format(
+   const VkAttachmentDescription2 *desc)
+{
+   return false;
+}
+
 #endif /* ANDROID_API_LEVEL >= 26 */
 
 #ifdef __cplusplus
