@@ -29,8 +29,11 @@
 
 #include "c11/threads.h"
 #include "util/list.h"
+#include "util/simple_mtx.h"
 #include "util/u_debug.h"
 
+#include "renderdoc_app.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -176,6 +179,10 @@ struct vk_instance {
 
    /** Whether the capture mode is per-submit. */
    bool trace_per_submit;
+
+   /** For triggering renderdoc captures from inside the driver. */
+   simple_mtx_t renderdoc_mtx;
+   RENDERDOC_API_1_0_0 *renderdoc_api;
 };
 
 VK_DEFINE_HANDLE_CASTS(vk_instance, base, VkInstance,
@@ -249,6 +256,10 @@ vk_instance_add_driver_trace_modes(struct vk_instance *instance,
 uint32_t
 vk_get_negotiated_icd_version(void);
 
+void vk_instance_start_renderdoc_capture(struct vk_instance *instance);
+
+void vk_instance_end_renderdoc_capture(struct vk_instance *instance);
+
 #ifdef __cplusplus
 }
 #endif
